import os
import pickle
import matplotlib.pyplot as plt
from scipy.spatial import distance

font = {'size'   : 22}
aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
figures_dir = "../figures/"

# Top-IDP scale
top_idp = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
    'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
    'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
    'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.341,
    'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

top_idp_noser = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
    'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
    'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
    'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
    'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}
    
dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)",                'other':'Intermediate',                'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)",                "top-idp":"TOP-IDP", "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>",
    'ss_turn':"<Turn propensity>","length_translation":"Length (AA)",                'gc_third':'GC% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred long(%AA)','length_aa':'<Length> (AA)','gc':'GC%',
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff (%AA)',
                'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM(%AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff(%AA)','seg':'<SEG>', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':
                'TYR', 'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                      'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                      'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                      'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                      'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                      'freq_T': 'THR', 'freq_W':                'TRP', 'freq_V': 'VAL',
                      'freq_Y': 'TYR', 'scampi':'TM  (%AA)', 'slider':'Slider',
                      'disembl_coils':'DisEMBL                Coils (%AA)',
                      'disembl_rem465':"DisEMBL REM465 (%AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (%AA)",
                      "globpipe":"GlobPlot (%AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea',
                "full":"Full", "pfam":"Shared domains", "domains":"Exclusive domains", 
                "linkers":"Linkers"}

#datasets = ["full", "pfam", "domains", "linkers"]
datasets = ["pfam", "domains", "linkers"]

dic_res = {}
for ty in datasets:
    out_dic_fname = "props_dic_"+ty+".pickle"
    print (out_dic_fname)
    with open(out_dic_fname, 'rb') as handle:
        dic_res.update(pickle.load(handle))



props = ["freq_" + aa for aa in aas_sorted_topidp]
data = {}

for kingdom in ["Eukaryota", "Bacteria"]:
    for ty in datasets:
        
        d = []
        for prop in props:
            value = float(dic_res[ty][kingdom][prop])/float(dic_res[ty][kingdom]["total_residues"])
            d += [value]
            
        label = dic_labels[kingdom] + "-" + dic_labels[ty]
                
        data[label] = d
            

import numpy
import pandas as pd

data_dic = {}
for s1 in data.keys():
    data_dic[s1] = {}
    for s2 in data.keys():
        data_dic[s1][s2] = round(numpy.corrcoef(data[s1], data[s2])[0, 1],3)
        #data_dic[s1][s2] = 1/(1+distance.euclidean(data[s1],data[s2]))
df = pd.DataFrame(data_dic)
print(df.to_latex())

#for pair in itertools.permutations(data.keys(),2):
#    print(pair[0])
        
#numpy.corrcoef(list1, list2)[0, 1]

        
import seaborn as sns
sns.set()
sns.set(font_scale=1.4)

sns.clustermap(df,cmap="Purples") # ,method="single",metric='correlation')

plt.savefig(figures_dir + "heatmap.eps",bbox_inches="tight")
