#!/usr/bin/env python3
import pandas as pd
import os
from Bio.Seq import Seq
from Bio import SeqIO
from Bio import AlignIO
import Bio.Align
import re
from Bio.SeqUtils.ProtParam import ProteinAnalysis
import math
import os.path
from Bio import motifs
import sys
import csv
from random import *
import numpy
from scipy.stats.stats import rankdata, tiecorrect
from scipy.stats import stats
#from statsmodels.formula.api import ols
from scipy.stats.stats import linregress
from scipy.stats import ttest_ind
from scipy.stats import mannwhitneyu
from scipy.spatial import distance
from scipy.stats import norm
import os.path as op
import sys
sys.path.insert(0, op.join(op.dirname(__file__), ".."))
from goatools.obo_parser import GODag
from goatools.mapslim import mapslim
from goatools.associations import read_associations


def get_GOslim(term):
    #print (term)
    if term not in go_dag:
        #sys.stderr.write(("term %s not found!\n" % term))
        #sys.exit(1)
        direct_anc="NONE"
        all_anc="NONE"
    else:
        direct_anc, all_anc = mapslim(term, go_dag, goslim_dag)
    return direct_anc,all_anc
        
def count_subst(a,b):
    for i in len(a):
        i=j
    return i

def get_phylum(s):
    try:
        return s.split(";")[1]
    except:
        return None    

def get_kingdom(s):
    try:
        return s.split(";")[0]
    except:
        return None    


if __name__ == '__main__':
    aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
    aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
                   'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
                   'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}
    ssindex = {'H':0, 'I':0, 'G':0, 'E':1, 'B':1, 'S':2, 'T':2, 'C':2}
    STDOUT = csv.writer(sys.stdout)
    STDOUT.writerow( ( "GO", "aa","bac_freq","arch_freq","euk_freq","numPFAM","Mann-P","Mann-T","Ttest-Z","Ttest-P",
                       "mean_bac","mean_arch","mean_euk","sd_bac","sd_arch","sd_euk","Z"))

    data_dir = "../data/"
    results_dir = "../data/"
    input_dir = "../data/pfam/"
    output_dir = "../data/"

    obo_file = data_dir + "go-basic.obo"
    assert os.path.exists(obo_file), "file %s not found!" % obo_file

    slim_obo_file = data_dir+"goslim_generic.obo"
    assert os.path.exists(slim_obo_file), "file %s not found!" % slim_obo_file
    go_dag = GODag(obo_file)
    goslim_dag = GODag(slim_obo_file)

    # We can read the relevant information from this file..
    filename=data_dir+'PFAMinfo_medGC.csv'
    df = pd.read_csv(filename)
    pfam_list = df.pfam.unique()
    # We allow max 20 (10 currently) GO terms per Pfam family
    maxgolist=20
    maxpfamlist=1000
    maxpfam=20000
    maxgo=999999
    pfam2go=numpy.zeros((maxpfam,maxgolist))
    go2pfam=numpy.zeros((maxgo,maxpfamlist))
    pfamlist={}
    golist={}

    

    #for f in sys.argv[1:]:
    #    file_list += [f]
    with open(data_dir+'pfam2go.txt') as fp:
        for line in fp:
            if re.match('^Pfam:',line):
                p=line.split()
                foo,pfam=p[0].split(':')
                pfamnum=int(re.sub(r'PF','',pfam))
                foo,go=line.split(';')
                go=re.sub(r'\n','',go)
                go=re.sub(r'\s','',go)
                pfam=re.sub(r'\s','',pfam)
                gonum=int(re.sub(r'GO:','',go))
                #print ("GOTEST:",go,":",type(go))
                direct,all=get_GOslim(go)
                #print ("Goterms",go,direct,pfam)
                if (not direct == "NONE"):
                    if (pfam in pfam_list):
                        go=direct.pop()
                        #print ("PFAM",pfam,go)
                        if (not pfam in pfamlist):
                            pfamlist[pfam]=0
                        if (not go in golist):
                            golist[go]=0
                        #print ("GO2PFAM",go,pfam,gonum,pfamnum,golist[go],pfamlist[pfam])
                        gonum=int(re.sub(r'GO:','',go))
                        pfam2go[pfamnum][pfamlist[pfam]]=gonum
                        go2pfam[gonum][golist[go]]=pfamnum
                        pfamlist[pfam]+=1
                        golist[go]+=1
    #print (pfamlist)
    mingo=0
    newgolist={}
    newpfamlist={}
    for go in golist:
        if golist[go]>mingo:
            arch_freq=numpy.zeros(20)
            bac_freq=numpy.zeros(20)
            euk_freq=numpy.zeros(20)
            numarch=0
            numbac=0
            numeuk=0
            gonum=int(re.sub(r'GO:','',go))
            #print ("TEST1",go,golist[go],mingo)
            archseq=numpy.zeros((golist[go],20))
            bacseq=numpy.zeros((golist[go],20))
            eukseq=numpy.zeros((golist[go],20))
            #print ("DF",df.pfam)
            #print ("AA",df.aa)
            for i in range(golist[go]):
                pfam="PF%05d" % int(go2pfam[gonum][i])
                #print ("TEST",i,gonum,go2pfam[gonum][i],pfam)
                for j in range(20):
                    bf=df.loc[ (df["pfam"]==pfam) & (df["aa"]==aas[j]) ]["bac_freq"].values
                    if (len(bf)>0):
                        bac_freq[j]+=bf[0]
                        bacseq[i,j]=bf[0]
                        ef=df.loc[ (df["pfam"]==pfam) & (df["aa"]==aas[j]) ]["euk_freq"].values
                        euk_freq[j]+=ef[0]
                        eukseq[i,j]=ef[0]
                        af=df.loc[ (df["pfam"]==pfam) & (df["aa"]==aas[j]) ]["arch_freq"].values
                        arch_freq[j]+=ef[0]
                        archseq[i,j]=ef[0]

            for i in range(20):
                a=archseq[:, [i]]
                b=bacseq[:, [i]]
                e=eukseq[:, [i]]
                #print ("Bac",b)
                #print ("Euk",e)
                (A, B) = ttest_ind(b, e,equal_var=False)
                #print ("T-test",T,P,numpy.mean(b),numpy.mean(e),numpy.sqrt(numpy.var(b)),numpy.sqrt(numpy.var(b)))
                try: 
                    (T,P) = mannwhitneyu(b,e,alternative='two-sided')
                except:
                    T=0.
                    P=9999.
                mean_bac=numpy.mean(b)
                mean_arch=numpy.mean(a)
                mean_euk=numpy.mean(e)
                sd_bac=numpy.sqrt(numpy.var(b))
                sd_arch=numpy.sqrt(numpy.var(a))
                sd_euk=numpy.sqrt(numpy.var(e))
                Z=(mean_euk-mean_bac)/math.sqrt(sd_euk*sd_euk+sd_bac*sd_bac+1.e-20)
                STDOUT.writerow( ( go, aas[i],bac_freq[i]/sum(bac_freq),arch_freq[i]/sum(arch_freq),euk_freq[i]/sum(euk_freq),len(b),P,T,A[0],B[0],mean_bac,mean_arch,mean_euk,sd_bac,sd_arch,sd_euk,Z))
