#!/usr/bin/env python3
import pandas as pd
import os
from Bio.Seq import Seq
from Bio import SeqIO
from Bio import AlignIO
import Bio.Align
import re
from Bio.SeqUtils.ProtParam import ProteinAnalysis
import math
from Bio.PDB import *
import os.path
from Bio import motifs
import sys
from random import *
import numpy as np
from scipy.optimize import curve_fit

class PDBSelect(Select):
    def accept_residue(self, residue):
        if residue.get_name()=='GLY':
            return True
        else:
            return False

def count_subst(a,b):
    for i in len(a):
        i=j
    return i

def get_phylum(s):
    try:
        return s.split(";")[1]
    except:
        return None    

def get_kingdom(s):
    try:
        return s.split(";")[0]
    except:
        return None    

def parse_annotation(filename):

    tax_id = int(filename.split("_")[1].split(".")[0])
    #print (tax_id)

    df = pd.read_csv(filename)
    n_proteins = len(df.query_id)

    df_mean = df.mean()
    
    columns = ["length", "top-idp", "iupred_long", "iupred_short","seg","ss_alpha", "ss_beta", "ss_coil", "ss_turn","hessa"]
    columns += ["freq_" + aa for aa in aas]
    columns += ["pfamA_acc"]
    
    ret_dic = {}    
    ret_dic["taxon_id"] = tax_id
    ret_dic["phylum"] = taxid2phylum.get(tax_id,pd.np.nan)
    ret_dic["kingdom"] = taxid2kingdom.get(tax_id,pd.np.nan)
    ret_dic["GC"] = taxid2gc.get(tax_id,pd.np.nan)
    ret_dic["count_protein"] = n_proteins

    for c in columns:
        ret_dic[c] = df_mean[c]
    
    #gcs = df_reference.loc[df_reference["TaxID"] == tax_id]["GC%"].astype(float)
    #ret_dic["GC"] = pd.np.mean(list(gcs))

    return ret_dic

if __name__ == '__main__':
    aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
    aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
                   'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
                   'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}
    ssindex = {'H':0, 'I':0, 'G':0, 'E':1, 'B':1, 'S':2, 'T':2, 'C':2}

    data_dir = "../data/"
    pdb_dir = "../data/"
    results_dir = "../data/"
    input_dir = "../data/pfam/"
    output_dir = "../data/"
    cifparser = MMCIFParser()
    pdbparser = PDBParser()
    
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    genomes_viruses_file = data_dir + "viruses.txt"

    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_virus = pd.read_csv(genomes_viruses_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks,df_virus])
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']    
    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]
    taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    taxid2kingdom = {}

    for t in taxid2group:
        gr = taxid2group[t]
        taxid2kingdom[t] = group2kingdom[gr]
    
    taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']


    # This takes too long
    codes={"V":"Viruses","E":"Eukaryota","B":"Bacteria","A":"Archaea","O":"Other","X":"Unknown"}
    taxid2kingdom={}
    memo2taxid={}
    with open(data_dir+'speclist.txt') as fp:
        for line in fp:
            if re.match('.*:\sN=',line):
                s=line.split()

                taxid=s[2].replace(':','')
                mnemonic=s[0]
                kingdom=codes[s[1]]
                memo2taxid[mnemonic]=taxid
                taxid2kingdom[taxid]=kingdom
                #print (line, taxid,mnemonic,kingdom)

    mycotaxa = []
    for t in name2taxid:
        tax = t.split()[0]
        if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
            mycotaxa += [name2taxid[t]]
        
    pfam_list = []
    for f in os.listdir(input_dir):
        #if f.endswith("222-full.ss"):
        if f.endswith(".ss"):
            basename=re.sub(r'-full.ss','',f)
            bacfile=basename+"_bac.fa"
            archfile=basename+"_arch.fa"
            eukfile=basename+"_euk.fa"
            if (os.path.isfile(input_dir+bacfile) and  os.path.isfile(input_dir+eukfile) and  os.path.isfile(input_dir+archfile)):
                if (os.path.getsize(input_dir+bacfile)>0 and  os.path.getsize(input_dir+archfile)>0 and  os.path.getsize(input_dir+eukfile)>0):
                    pfam_list += [basename]

    # Now we also read bacterial genomes to calculate expected frequencies at 50%GC
    df = pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-GC-All.csv").dropna(axis=0)
    good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    fit_fn={}
    valuefifty={}
    for prop in props:
        temp_df = df.loc[(df.kingdom.isin(['Bacteria'])) & (df.taxon_id.isin(good_taxa))][list(set([prop,"GC_genomic"]))].convert_objects(convert_numeric=True)
        temp_df=temp_df.dropna()
        prop_1_values = temp_df["GC_genomic"].values
        prop_2_values = temp_df[prop].values
        fit = np.polyfit(list(prop_1_values),list(prop_2_values),1)
        fit_fn[prop] = np.poly1d(fit)
        valuefifty[prop]=fit_fn[prop][0]+50.*fit_fn[prop][1]
        #print (prop,fit_fn[prop],fit_fn[prop][0],fit_fn[prop][1],valuefifty[prop])

    #for f in sys.argv[1:]:
    #    file_list += [f]
    sumbac_helix=np.zeros(20)
    sumbac_sheet=np.zeros(20)
    sumbac_loop=np.zeros(20)
    sumbac_diso=np.zeros(20)
    sumarch_helix=np.zeros(20)
    sumarch_sheet=np.zeros(20)
    sumarch_loop=np.zeros(20)
    sumarch_diso=np.zeros(20)
    sumeuk_helix=np.zeros(20)
    sumeuk_sheet=np.zeros(20)
    sumeuk_loop=np.zeros(20)
    sumeuk_diso=np.zeros(20)
    #print (pfam_list)
    numdiso_euk=0
    numhelix_euk=0
    numsheet_euk=0
    numloop_euk=0
    numdiso_bac=0
    numhelix_bac=0
    numsheet_bac=0
    numloop_bac=0
    numdiso_arch=0
    numhelix_arch=0
    numsheet_arch=0
    numloop_arch=0

    sumbac_helix_GC=np.zeros(20)
    sumbac_sheet_GC=np.zeros(20)
    sumbac_loop_GC=np.zeros(20)
    sumbac_diso_GC=np.zeros(20)
    sumarch_helix_GC=np.zeros(20)
    sumarch_sheet_GC=np.zeros(20)
    sumarch_loop_GC=np.zeros(20)
    sumarch_diso_GC=np.zeros(20)
    sumeuk_helix_GC=np.zeros(20)
    sumeuk_sheet_GC=np.zeros(20)
    sumeuk_loop_GC=np.zeros(20)
    sumeuk_diso_GC=np.zeros(20)
    #print (pfam_list)
    numdiso_euk_GC=0
    numhelix_euk_GC=0
    numsheet_euk_GC=0
    numloop_euk_GC=0
    numdiso_bac_GC=0
    numhelix_bac_GC=0
    numsheet_bac_GC=0
    numloop_bac_GC=0
    numdiso_arch_GC=0
    numhelix_arch_GC=0
    numsheet_arch_GC=0
    numloop_arch_GC=0
    for pfam in pfam_list:
        arch= AlignIO.read(input_dir+pfam+"_arch.fa", "fasta")
        bac= AlignIO.read(input_dir+pfam+"_bac.fa", "fasta")
        euk = AlignIO.read(input_dir+pfam+"_euk.fa", "fasta")
        ss = AlignIO.read(input_dir+pfam+"-full.ss", "fasta")
        

        length=len(bac[0].seq)
        numarch=len(arch)
        numbac=len(bac)
        numeuk=len(euk)
        numss=len(ss)
        arch_freq=np.zeros((length,20))
        bac_freq=np.zeros((length,20))
        euk_freq=np.zeros((length,20))
        ss_freq=np.zeros((length,3))
        arch_freq_GC=np.zeros((length,20))
        bac_freq_GC=np.zeros((length,20))
        euk_freq_GC=np.zeros((length,20))
        #ss_freq_GC=np.zeros((length,3))
        #print ("LENGTH",length,len(bac),len(euk))
        #print (record.name)
        for i in range(0,len(bac)):
            mtemp=re.split("[_\/]",bac[i].name)
            mnemonic=mtemp[1]
            tax_id=memo2taxid.get(mnemonic,pd.np.nan)
            #print ("TEST",pfam,mtemp,mnemonic)
            for j in range(0,length):
                seq=bac[i].seq[j].upper()
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', seq)):
                    bac_freq[j][aaindex[seq]]+=1
                    try:
                        GC = float(taxid2gc[int(tax_id)])
                    except:
                        continue
                    
                    expected=fit_fn["freq_"+seq][0]+GC*fit_fn["freq_"+seq][1]
                    # This is actually (A+B*50)/(A+B*GC)
                    factor=(valuefifty["freq_"+seq]/expected)
                    bac_freq_GC[j][aaindex[seq]]+=factor
                    #print (j,seq,GC,valuefifty["freq_"+seq],expected,factor)
        for i in range(0,len(arch)):
            mtemp=re.split("[_\/]",arch[i].name)
            mnemonic=mtemp[1]
            tax_id=memo2taxid.get(mnemonic,pd.np.nan)
            for j in range(0,length):
                seq=arch[i].seq[j].upper()
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', seq)):
                    arch_freq[j][aaindex[seq]]+=1
                    try:
                        GC = float(taxid2gc[int(tax_id)])
                    except:
                        continue
                    expected=fit_fn["freq_"+seq][0]+GC*fit_fn["freq_"+seq][1]
                    factor=(valuefifty["freq_"+seq]/expected)
                    arch_freq_GC[j][aaindex[seq]]+=factor
                    #print (j,seq,GC,valuefifty["freq_"+seq],expected,factor)
        for i in range(0,len(euk)):
            mtemp=re.split("[_\/]",euk[i].name)
            mnemonic=mtemp[1]
            tax_id=memo2taxid.get(mnemonic,pd.np.nan)
            for j in range(0,length):
                seq=euk[i].seq[j].upper()
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', seq)):
                    euk_freq[j][aaindex[seq]]+=1
                    #print ("TEST",i,j,euk[i].seq[j],aaindex[euk[i].seq[j]],euk_freq[j][aaindex[euk[i].seq[j]]])
                    try:
                        GC = float(taxid2gc[int(tax_id)])
                    except:
                        continue
                    expected=fit_fn["freq_"+seq][0]+GC*fit_fn["freq_"+seq][1]
                    factor=(valuefifty["freq_"+seq]/expected)
                    euk_freq_GC[j][aaindex[seq]]+=factor
                    #print (j,seq,GC,valuefifty["freq_"+seq],expected,factor)
        for i in range(0,len(ss)):
            for j in range(0,length):
                if (re.match('[GHIEBTSC]', ss[i].seq[j])):
                    ss_freq[j][ssindex[ss[i].seq[j]]]+=1
        for j in range(0,length):
            sumbac=0.
            sumarch=0.
            sumeuk=0.
            sumbac_GC=0.
            sumarch_GC=0.
            sumeuk_GC=0.
            sumss=ss_freq[j][0]+ss_freq[j][1]+ss_freq[j][2]+1.e-20
            #print ("ITER", j,sumss,ss_freq[j][0],ss_freq[j][1],ss_freq[j][2])
            for i in range(20):
                sumeuk+=euk_freq[j][i]
                sumbac+=bac_freq[j][i]
                sumarch+=arch_freq[j][i]
                sumeuk_GC+=euk_freq_GC[j][i]
                sumbac_GC+=bac_freq_GC[j][i]
                sumarch_GC+=arch_freq_GC[j][i]

            print (sumeuk,sumbac,sumarch,sumeuk_GC,sumbac_GC,sumarch_GC)


            if ((ss_freq[j][0]/sumss > 0.5)):
                if sumeuk>0:
                    numhelix_euk+=1
                    for i in range(20):
                        sumeuk_helix[i]+=euk_freq[j][i]/sumeuk
                if sumeuk_GC>0:
                    numhelix_euk_GC+=1
                    for i in range(20):
                        sumeuk_helix_GC[i]+=euk_freq_GC[j][i]/sumeuk_GC
                if sumbac>0:
                    numhelix_bac+=1
                    for i in range(20):
                        sumbac_helix[i]+=bac_freq[j][i]/sumbac
                if sumbac_GC>0:
                    numhelix_bac_GC+=1
                    for i in range(20):
                        sumbac_helix_GC[i]+=bac_freq_GC[j][i]/sumbac_GC
                if sumarch>0:
                    numhelix_arch+=1
                    for i in range(20):
                        sumarch_helix[i]+=arch_freq[j][i]/sumarch
                if sumarch_GC>0:
                    numhelix_arch_GC+=1
                    for i in range(20):
                        sumarch_helix_GC[i]+=arch_freq_GC[j][i]/sumarch_GC
            elif (ss_freq[j][1]/sumss > 0.5):
                if sumeuk>0:
                    numsheet_euk+=1
                    for i in range(20):
                        sumeuk_sheet[i]+=euk_freq[j][i]/sumeuk
                if sumeuk_GC>0:
                    numsheet_euk_GC+=1
                    for i in range(20):
                        sumeuk_sheet_GC[i]+=euk_freq_GC[j][i]/sumeuk
                if sumbac>0:
                    numsheet_bac+=1
                    for i in range(20):
                        sumbac_sheet[i]+=bac_freq[j][i]/sumbac
                if sumbac_GC>0:
                    numsheet_bac_GC+=1
                    for i in range(20):
                        sumbac_sheet_GC[i]+=bac_freq_GC[j][i]/sumbac_GC
                if sumarch>0:
                    numsheet_arch+=1
                    for i in range(20):
                        sumarch_sheet[i]+=arch_freq[j][i]/sumarch
                if sumarch_GC>0:
                    numsheet_arch_GC+=1
                    for i in range(20):
                        sumarch_sheet_GC[i]+=arch_freq_GC[j][i]/sumarch_GC
            elif (ss_freq[j][2]/sumss > 0.5):                
                if sumeuk>0:
                    numloop_euk+=1
                    for i in range(20):
                        sumeuk_loop[i]+=euk_freq[j][i]/sumeuk
                if sumeuk_GC>0:
                    numloop_euk_GC+=1
                    for i in range(20):
                        sumeuk_loop_GC[i]+=euk_freq_GC[j][i]/sumeuk_GC
                if sumbac>0:
                    numloop_bac+=1
                    for i in range(20):
                        sumbac_loop[i]+=bac_freq[j][i]/sumbac
                if sumbac_GC>0:
                    numloop_bac_GC+=1
                    for i in range(20):
                        sumbac_loop_GC[i]+=bac_freq_GC[j][i]/sumbac_GC
                if sumarch>0:
                    numloop_arch+=1
                    for i in range(20):
                        sumarch_loop[i]+=arch_freq[j][i]/sumarch
                if sumarch_GC>0:
                    numloop_arch_GC+=1
                    for i in range(20):
                        sumarch_loop_GC[i]+=arch_freq_GC[j][i]/sumarch_GC
            else:
                if sumeuk>0:
                    numdiso_euk+=1
                    for i in range(20):
                        sumeuk_diso[i]+=euk_freq[j][i]/sumeuk
                if sumeuk_GC>0:
                    numdiso_euk_GC+=1
                    for i in range(20):
                        sumeuk_diso_GC[i]+=euk_freq_GC[j][i]/sumeuk_GC
                if sumbac>0:
                    numdiso_bac+=1
                    for i in range(20):
                        sumbac_diso[i]+=bac_freq[j][i]/sumbac
                if sumbac_GC>0:
                    numdiso_bac_GC+=1
                    for i in range(20):
                        sumbac_diso_GC[i]+=bac_freq_GC[j][i]/sumbac_GC
                if sumarch>0:
                    numdiso_arch+=1
                    for i in range(20):
                        sumarch_diso[i]+=arch_freq[j][i]/sumarch
                if sumarch_GC>0:
                    numdiso_arch_GC+=1
                    for i in range(20):
                        sumarch_diso_GC[i]+=arch_freq_GC[j][i]/sumarch_GC

    num=len(pfam_list)
    
    #print ("TEST",numhelix_bac,sumbac_helix.sum())
    print ("num","AA","bac-helix","arch-helix","euk-helix","bac-sheet","arch-sheet","euk-sheet",
           "bac-loop","arch-loop","euk-loop","bac-diso","arch-diso","euk-diso")

    for i in range(20):
        print (i,aas[i],
               round(100*sumbac_helix[i]/numhelix_bac,1),round(100*sumarch_helix[i]/numhelix_arch,1),round(100*sumeuk_helix[i]/numhelix_euk,1),
               round(100*sumbac_sheet[i]/numsheet_bac,1),round(100*sumarch_sheet[i]/numsheet_arch,1),round(100*sumeuk_sheet[i]/numsheet_euk,1),
               round(100*sumbac_loop[i]/numloop_bac,1),round(100*sumarch_loop[i]/numloop_arch,1),round(100*sumeuk_loop[i]/numloop_euk,1),
               round(100*sumbac_diso[i]/numdiso_bac,1),round(100*sumarch_diso[i]/numdiso_arch,1),round(100*sumeuk_diso[i]/numdiso_euk,1))               


    np.save(output_dir+"sumbac_helix.npy",sumbac_helix)
    np.save(output_dir+"sumarch_helix.npy",sumarch_helix)
    np.save(output_dir+"sumeuk_helix.npy",sumeuk_helix)
    np.save(output_dir+"sumbac_sheet.npy",sumbac_sheet)
    np.save(output_dir+"sumarch_sheet.npy",sumarch_sheet)
    np.save(output_dir+"sumeuk_sheet.npy",sumeuk_sheet)
    np.save(output_dir+"sumbac_loop.npy",sumbac_loop)
    np.save(output_dir+"sumarch_loop.npy",sumarch_loop)
    np.save(output_dir+"sumeuk_loop.npy",sumeuk_loop)
    np.save(output_dir+"sumbac_diso.npy",sumbac_diso)
    np.save(output_dir+"sumarch_diso.npy",sumarch_diso)
    np.save(output_dir+"sumeuk_diso.npy",sumeuk_diso)


    print ("num/GC_GC","AA_GC","bac-helix_GC","arch-helix_GC","euk-helix_GC","bac-sheet_GC","arch-sheet_GC","euk-sheet_GC",
           "bac-loop_GC","arch-loop_GC","euk-loop_GC","bac-diso_GC","arch-diso_GC","euk-diso_GC")

    for i in range(20):
        print (i,aas[i],
               round(100*sumbac_helix_GC[i]/numhelix_bac_GC,1),round(100*sumarch_helix_GC[i]/numhelix_arch_GC,1),round(100*sumeuk_helix_GC[i]/numhelix_euk_GC,1),
               round(100*sumbac_sheet_GC[i]/numsheet_bac_GC,1),round(100*sumarch_sheet_GC[i]/numsheet_arch_GC,1),round(100*sumeuk_sheet_GC[i]/numsheet_euk_GC,1),
               round(100* sumbac_loop_GC[i]/ numloop_bac_GC,1),round(100* sumarch_loop_GC[i]/ numloop_arch_GC,1),round(100* sumeuk_loop_GC[i]/ numloop_euk_GC,1),
               round(100* sumbac_diso_GC[i]/ numdiso_bac_GC,1),round(100* sumarch_diso_GC[i]/ numdiso_arch_GC,1),round(100* sumeuk_diso_GC[i]/ numdiso_euk_GC,1)) 


        
    np.save(output_dir+"sumbac_helix_GC.npy",sumbac_helix_GC)
    np.save(output_dir+"sumarch_helix_GC.npy",sumarch_helix_GC)
    np.save(output_dir+"sumeuk_helix_GC.npy",sumeuk_helix_GC)
    np.save(output_dir+"sumbac_sheet_GC.npy",sumbac_sheet_GC)
    np.save(output_dir+"sumarch_sheet_GC.npy",sumarch_sheet_GC)
    np.save(output_dir+"sumeuk_sheet_GC.npy",sumeuk_sheet_GC)
    np.save(output_dir+"sumbac_loop_GC.npy",sumbac_loop_GC)
    np.save(output_dir+"sumarch_loop_GC.npy",sumarch_loop_GC)
    np.save(output_dir+"sumeuk_loop_GC.npy",sumeuk_loop_GC)
    np.save(output_dir+"sumbac_diso_GC.npy",sumbac_diso_GC)
    np.save(output_dir+"sumarch_diso_GC.npy",sumarch_diso_GC)
    np.save(output_dir+"sumeuk_diso_GC.npy",sumeuk_diso_GC)
