#!/usr/bin/env python3


def bar_plot_s1_vs_s2(k1,k2,figname=None,weighted=False, uselog = False):
    fig, ax = plt.subplots(figsize=(6.,3))

    clr = "red"
    if weighted:
        clr = "green"
        
    data_aa = []
    for aa in aas_sorted_topidp:
        prop = "freq_" + aa

        prop_values = {}
        cat1 = df.loc[df.kingdom == k1][prop]
        cat2 = df.loc[df.kingdom == k2][prop]
        prop_values_1=np.mean(cat1)
        prop_values_2=np.mean(cat2)

        if uselog == True:
            value = np.log(prop_values_1/prop_values_2)
        else:
            value = prop_values_1-prop_values_2
            
        if weighted:
            value *= top_idp[aa]
            
        data_aa += [value]

        
    ax.bar(range(len(data_aa)),data_aa, color=clr)

    #ylim = max([abs(v) for v in ax.get_ylim()])
    if not weighted:
        ylim = 0.03
    else:
        ylim = 0.018
    #ax.set_ylim([-ylim, ylim])
    ax.set_xticks([0.0 + x for x in range(len(aas_sorted_topidp))])
    ax.set_xticklabels(aas_sorted_topidp)
    
    if uselog == True:
        ax.set_ylabel("log( freq(" + k1+ ") / freq(" + k2+ ") )")
    else:
        ax.set_ylabel("freq(" + k1+ ") - freq(" + k2+ ")")
    
    title = dic_labels[k1] +  " vs "  + dic_labels[k2] 
    ax.set_title(title)
    
    ax.yaxis.grid()
    ax.xaxis.grid()
    
    if figname != None:
        print(figname)
        fig.savefig(figname, bbox_inches = "tight")
    else:
        plt.show()
        
    plt.close()
    


def get_topidp(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp[aa]*data["freq_"+aa]
    return score

def get_topidp_noser(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noser[aa]*data["freq_"+aa]
    return score

def get_topidp_noserpro(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noserpro[aa]*data["freq_"+aa]
    return score

def get_topidp_noserlyspro(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noserlyspro[aa]*data["freq_"+aa]
    return score
    
# coding: utf-8
def get_aa_freq(aa,gc):
    codons = aa2codons[aa]
    prob_tot = []
    for codon in codons:
        prob_codon = []
        for N in codon:
            prob = gc/2
            if N in {"A","T","U"}:
                prob = 0.5-(gc/2)
            prob_codon += [prob]
        #print(codon, prob_codon,functools.reduce(mul,prob_codon))
        prob_tot += [functools.reduce(mul,prob_codon)] # Use functools.reduce() if you really need it; however, 99 percent of the time an explicit for loop is more readable
    prob_tot = sum(prob_tot)
    return prob_tot


from operator import mul
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import seaborn as sns
import functools
from scipy.stats import ttest_ind
from scipy.stats import mannwhitneyu
from scipy.spatial import distance
from scipy.stats import norm
#from numpy import sqrt, abs, round
import numpy as np
import math as math
from matplotlib import colors
from matplotlib import cm

def darken(col):
    c = colors.hex2color(col)
    nc = []
    for e in c:
        ne = e-0.4
        if ne < 0: ne = 0
        nc += [ne]
    return nc

    

def twoSampZ(X1, X2, mudiff, sd1, sd2, n1, n2):
    pooledSE = sqrt(sd1**2/n1 + sd2**2/n2)
    z = ((X1 - X2) - mudiff)/pooledSE
    pval = 2*(1 - norm.cdf(abs(z)))
    return round(z, 3), round(pval, 4)


if __name__ == "__main__":
    figures_dir="../figures/"
    dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)", "top-idp":"TOP-IDP", "topidp-S":"TOP-IDP - S","topidp-SKP":"TOP-IDP - SKP","topidp-SP":"TOP-IDP - SP",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'GC% (third nucleotide)','taxon_gc':'GC% (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
    'random':'Expected','iupred_long':'IUpred long (%AA)','length_aa':'<Length> (AA)','gc':'GC%',
    'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)', 'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM (%AA)',"volatility":"<Volatility>", 'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity> (Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with  expected', 'aa_corr_dot':'AA freq\ncorrelation with   expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A':
    'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P':'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'ALA', 'freq_C':
    'CYS', 'freq_E': 'GLU', 'freq_D': 'ASP', 'freq_G': 'GLY',
    'freq_F': 'PHE', 'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K':
    'LYS', 'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
    'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S': 'SER', 'freq_R':
    'ARG', 'freq_T': 'THR', 'freq_W': 'TRP', 'freq_V': 'VAL',
    'freq_Y': 'TYR', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Viruses':'Viruses'}
        
    ylabels = {"GC":"Fraction","GC_genomic":"Fraction", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity>", "top-idp":"<TOP-IDP>","topidp-S":"<TOP-IDP-noS>","topidp-SKP":"<TOP-IDP-noSKP>","topidp-SP":"<TOP-IDP-noSP>",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'fraction','taxon_gc':'fraction','taxon_gc_r':'fraction',
    'random':'fraction','iupred_long':'Fraction of AA','length_aa':'<Length>','gc':'fraction',
    'iupred_short':'Fration of residues','iupred_cutoff':'Fraction of residues', 'idp':'<TOP-IDP>','tm':'Fraction of residues',"volatility":"<Volatility>", 'iupred_short':'Fraction of residues','iupred_cutoff':'Fraction of residues','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity>','gc':'Fraction','aa_corr':'Correlation', 'aa_corr_dot':'correlation','aa_corr_r':'Correlation', 'A':'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'Fraction of residues', 'freq_C':
    'Fraction of residues', 'freq_E': 'Fraction of residues', 'freq_D': 'Fraction of residues', 'freq_G': 'Fraction of residues',
    'freq_F': 'Fraction of residues', 'freq_I': 'Fraction of residues', 'freq_H': 'Fraction of residues', 'freq_K':
    'Fraction of residues', 'freq_M': 'Fraction of residues', 'freq_L': 'Fraction of residues', 'freq_N': 'Fraction of residues',
    'freq_Q': 'Fraction of residues', 'freq_P': 'Fraction of residues', 'freq_S': 'Fraction of residues', 'freq_R':
    'Fraction of residues', 'freq_T': 'Fraction of residues', 'freq_W': 'Fraction of residues', 'freq_V': 'Fraction of residues',
    'freq_Y': 'Fraction of residues', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea'}
        
    top_idp = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.341,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    top_idp_noser = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}
        
    top_idp_noserpro = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0., 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}
        
    top_idp_noserlyspro = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0., 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0., 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}
        
    dic_limits = {'freq_C':[0,0.03], 'freq_S':[0.03,0.1], 'length':[0,1000]}
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']

    codons2aa = {
        'ATA':'I', 'ATC':'I', 'ATT':'I', 'ATG':'M', 'ACA':'T', 'ACC':'T', 'ACG':'T', 'ACT':'T',
        'AAC':'N', 'AAT':'N', 'AAA':'K', 'AAG':'K', 'AGC':'S', 'AGT':'S', 'AGA':'R', 'AGG':'R',
        'CTA':'L', 'CTC':'L', 'CTG':'L', 'CTT':'L', 'CCA':'P', 'CCC':'P', 'CCG':'P', 'CCT':'P',
        'CAC':'H', 'CAT':'H', 'CAA':'Q', 'CAG':'Q', 'CGA':'R', 'CGC':'R', 'CGG':'R', 'CGT':'R',
        'GTA':'V', 'GTC':'V', 'GTG':'V', 'GTT':'V', 'GCA':'A', 'GCC':'A', 'GCG':'A', 'GCT':'A',
        'GAC':'D', 'GAT':'D', 'GAA':'E', 'GAG':'E', 'GGA':'G', 'GGC':'G', 'GGG':'G', 'GGT':'G',
        'TCA':'S', 'TCC':'S', 'TCG':'S', 'TCT':'S', 'TTC':'F', 'TTT':'F', 'TTA':'L', 'TTG':'L',
        'TAC':'Y', 'TAT':'Y', 'TAA':'_', 'TAG':'_', 'TGC':'C', 'TGT':'C', 'TGA':'_', 'TGG':'W'}
        
    aa2codons = {}
    aa2codons["ALL"] = []
    for c in codons2aa:
        aa2codons[codons2aa[c]] = aa2codons.get(codons2aa[c],[]) + [c]
        if c != "_":
            aa2codons["ALL"] += [c]
            

# In[2]:

    #df = pd.read_csv("../data/df_uniprot_reference_proteomes_per_species_dec2017.csv").dropna()
    df = pd.read_csv("../data/df_uniprot_reference_proteomes_per_species-sum-GC-All.csv").dropna()
    df["GC_genomic"]=df["GC_genomic"].astype(float)
    df["GC"]=df["GC"].astype(float)
    # We do calculate top-idp from AAfreq (with and without Ser)

    #idpS=df.apply(get_topidp, axis=1) This was just for checking
    #idpnoS=df.apply(get_topidp_noser, axis=1)
    df['topidp-S']=df.apply(get_topidp_noser, axis=1)
    df['topidp-SKP']=df.apply(get_topidp_noserlyspro, axis=1)
    df['topidp-SP']=df.apply(get_topidp_noserpro, axis=1)
    #print(idpS,idpnoS,df)
# In[30]:


    #kingdoms = df.kingdom.dropna().unique()

    #boxcolr=["red" for aa in aas_sorted_topidp]
    #boxcolr+["blue","blue","blue","green"]
    boxcolr=["red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","blue","blue","blue","green"]
    boxcolrnoS=["red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","blue","blue","blue","lightblue","lightblue","lightblue","green"]
    
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    
    kingdoms= ["Bacteria","Archaea","Eukaryota","Viruses"]
    phyla = df.phylum.dropna().unique()
    
    kingdom2color = {}
    for k, c in zip(kingdoms, ['#E20048', '#5064d0','#00B060',  '#FFA500']):
        kingdom2color[k] = c
        

    phylum2color = {}
    phylum2color_gc = {}

    for phylum in phyla:
        tdf = df.loc[df.phylum == phylum]
        kingdom = tdf.kingdom.iloc[0]
        col = kingdom2color[kingdom]
        phylum2color[phylum] = col
        
        gc = pd.np.mean(pd.np.array([float(f) for f in tdf.GC.replace("-",pd.np.nan).dropna()]))

        # color according to GC only
        # print (phylum,gc)
        if gc > 50:
            col = "darkred"
        elif gc > 45:
            col = "pink"
        elif gc > 40:
            col = "lightgreen"
        else:
            col = "darkgreen"
        cmap = cm.get_cmap('viridis')
        max=71
        min=34
        col=cmap( 1-(gc-min)/(max-min))
        phylum2color_gc[phylum] = col


    
    data = {}

    for phylum in phyla:

        tdf = df.loc[df.phylum == phylum]
        if len(tdf.length) > 10:
            d = []
            for prop in props:
                value = float(tdf[prop].mean())
                d += [value]
                
            #data[list(tdf.kingdom)[0] + " - " + phylum] = d
            data[phylum] = d


    data_dic = {}
    for s1 in data.keys():
        data_dic[s1] = {}
        for s2 in data.keys():
            #data_dic[s1][s2] = round(pd.np.corrcoef(data[s1], data[s2])[0, 1],3)
            data_dic[s1][s2] = 1/(1+distance.euclidean(data[s1],data[s2]))

    df_r = pd.DataFrame(data_dic)




    
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    props += ["iupred_long", "iupred_short", "top-idp", "topidp-S" ,"topidp-SKP" ,"topidp-SP" , "length"]
    
    for prop in props:
        print (prop)
        fig, axes = plt.subplots()

        data = []

        for k in kingdoms:
            d = df.loc[df.kingdom == k][prop]
            
            if prop in ["iupred_long", "iupred_short"]:
                d = d*100
                
            data += [d]
    #axes.violinplot(dataset = data, showmedians = True)
        bp = axes.boxplot(data,patch_artist=True)
    
        dic_colors = {0:'#E20048', 1:'#5064d0',2:'#00B060',  3:'orange'}
    
        for i,box in enumerate(bp['boxes']):
            box.set(color = dic_colors[i], linewidth=2)
            box.set(facecolor = dic_colors[i])
        
        
    #~ pylab.setp(bp['boxes'],color='black',lw=2)
        plt.setp(bp['whiskers'], color='black')
        plt.setp(bp['medians'], color='black')
        plt.setp(bp['fliers'], color='black', marker='+')

        if prop == "length":
            axes.set_ylim(0,1000)
        elif prop == "top-idp":
            axes.set_ylim(0,0.2)
        elif prop == "topidp-S":
            axes.set_ylim(0,0.2)
        elif prop == "topidp-SP":
            axes.set_ylim(-0.05,0.05)
        elif prop == "topidp-SKP":
            axes.set_ylim(-0.05,0.05)
        else:
            axes.set_ylim(0,50)
            
        axes.set_xticks(range(1,len(kingdoms)+1))
        axes.set_xticklabels(kingdoms)
        axes.set_ylabel(ylabels[prop])
        axes.set_title(dic_labels[prop])
        
        #axes.set_ylabel(dic_labels[prop])

    #~ plt.show()
        fig.savefig(figures_dir+"boxplot_kingdoms_" + prop + ".eps")

        plt.close()


    #exit()

        #for aa in aas_sorted_topidp:
        #prop = "freq_" + aa
        #for k in ["Eukaryota"]:

    props = ["freq_" + aa for aa in aas_sorted_topidp]
    props += ["iupred_long", "iupred_short", "top-idp", "topidp-S" ,"topidp-SKP" ,"topidp-SP" , "length"]
        
    for k in kingdoms:
        for l in kingdoms:
            figname = figures_dir + "aa_freq_change_" + k + "_vs_" + l +  ".eps"
            bar_plot_s1_vs_s2(k,l,figname, False, False)

    for k in kingdoms:
        for l in kingdoms:
            figname = figures_dir + "aa_freq_change_weighted_" + k + "_vs_" + l +  ".eps"
            bar_plot_s1_vs_s2(k,l,figname, True, False)

    for k in kingdoms:
        for l in kingdoms:
            data=[]
            fig, ax = plt.subplots(figsize=(6.,3))
            for prop in props:
                cat1 = df.loc[df.kingdom == k][prop]
                cat2 = df.loc[df.kingdom == l][prop]
                a, b = ttest_ind(cat1, cat2,equal_var=False)
                c, d  = mannwhitneyu(cat1, cat2,alternative='two-sided')
                mean1=np.mean(cat1)
                mean2=np.mean(cat2)
                sd1=np.sqrt(np.var(cat1))
                sd2=np.sqrt(np.var(cat2))
                Z=(mean1-mean2)/math.sqrt(sd1*sd1+sd2*sd2)
                print ("T-test: ",prop,k,l,mean1,mean2,a,b,c,d)
                print ("Z-score: ",prop,k,l,mean1,mean2,sd1,sd2,Z)
                data += [Z]
            figname=figures_dir+"boxplot-Z-test_noS_"+k +"_" + l + ".eps"
            ax.bar(range(len(data)),data, color=boxcolrnoS)
            ax.set_ylim([-1.4,2.9])
            #~ ax.set_xticks([0.5 + x for x in range(len(props))])
            ax.set_xticks([x for x in range(len(props))])
            ax.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
            ax.set_title("Difference between " + k + " and " + l)
            ax.set_ylabel("Z-score")
            ax.yaxis.grid()
            ax.xaxis.grid()
            fig.savefig(figname, bbox_inches = "tight")
            plt.close()

    props = ["freq_" + aa for aa in aas_sorted_topidp]
    props += ["iupred_long", "iupred_short", "top-idp" , "length"]
        
    
    for k in kingdoms:
        for l in kingdoms:
            data=[]
            fig, ax = plt.subplots(figsize=(6.,3))
            for prop in props:
                cat1 = df.loc[df.kingdom == k][prop]
                cat2 = df.loc[df.kingdom == l][prop]
                a, b = ttest_ind(cat1, cat2,equal_var=False)
                c, d  = mannwhitneyu(cat1, cat2,alternative='two-sided')
                mean1=np.mean(cat1)
                mean2=np.mean(cat2)
                sd1=np.sqrt(np.var(cat1))
                sd2=np.sqrt(np.var(cat2))
                Z=(mean1-mean2)/math.sqrt(sd1*sd1+sd2*sd2)
                print ("T-test: ",prop,k,l,mean1,mean2,a,b,c,d)
                print ("Z-score: ",prop,k,l,mean1,mean2,sd1,sd2,Z)
                data += [Z]
            figname=figures_dir+"boxplot-Z-test_"+k +"_" + l + ".eps"
            ax.bar(range(len(data)),data, color=boxcolr)
            ax.set_ylim([-1.4,2.9])
            #~ ax.set_xticks([0.5 + x for x in range(len(props))])
            ax.set_xticks([x for x in range(len(props))])
            ax.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
            ax.set_title("Difference between " + k + " and " + l)
            ax.set_ylabel("Z-score")
            ax.yaxis.grid()
            ax.xaxis.grid()
            fig.savefig(figname, bbox_inches = "tight")
            plt.close()


    #exit()
    # plot a series of properties as boxplots, with each box corresplonding to phylum
    
    # the properties to plot

    df[df.kingdom == "Eukaryota"].fillna("Unknown Eukaryota", inplace=True)
    df[df.kingdom == "Bacteria"].fillna("Unknown Bacteria", inplace=True)
    df[df.kingdom == "Archaea"].fillna("Unknown Archaea", inplace=True)
    df[df.kingdom == "Viruses"].fillna("Unknown Virus", inplace=True)
    phyla_euk = df[df.kingdom == "Eukaryota"].phylum.dropna().unique()
    phyla_proks = df[df.kingdom == "Bacteria"].phylum.dropna().unique()
    phyla_arch = df[df.kingdom == "Archaea"].phylum.dropna().unique()
    phyla_virus = df[df.kingdom == "Viruses"].phylum.unique()
    # Many Viruses have no phyla needs to be changed.
    all_phyla = list(phyla_proks)  + list(phyla_arch) + list(phyla_euk) + list(phyla_virus) 
    #all_phyla = list(phyla_proks) + list(phyla_euk)
    # print phyla_virus
    
    # plots by phylum (modified from proks_vs_euks.py

    for prop in props:    
        data = []
        labels = []
        # Current data has same name for different phylums so do not work
        #for phylum in all_phyla:
        #    dd = df[df.phylum == phylum][prop].dropna()
        #    if len(dd) > 10:
        #        length = df[df.phylum == phylum]["length"].dropna()
        #        data += [dd]
        #        labels += [phylum]
        dfs=df[df.kingdom == "Bacteria"]
        for phylum in phyla_proks:
            dd = dfs[dfs.phylum == phylum][prop].dropna()
            if len(dd) > 10:
                length = dfs[dfs.phylum == phylum]["length"].dropna()
                data += [dd]
                labels += [phylum]
                box.set(color='#E20048', linewidth=2)
                box.set(facecolor = '#E20048' )
                
        dfs=df[df.kingdom == "Archaea"]
        for phylum in phyla_arch:
            dd = dfs[dfs.phylum == phylum][prop].dropna()
            if len(dd) > 10:
                length = dfs[dfs.phylum == phylum]["length"].dropna()
                data += [dd]
                labels += [phylum]
                box.set(color='#5064d0', linewidth=2)
                box.set(facecolor = '#5064d0' )

        dfs=df[df.kingdom == "Eukaryota"]
        for phylum in phyla_euk:
            dd = dfs[dfs.phylum == phylum][prop].dropna()
            if len(dd) > 10:
                length = dfs[dfs.phylum == phylum]["length"].dropna()
                data += [dd]
                labels += [phylum]
                box.set(color='#00B060', linewidth=2)
                box.set(facecolor = '#00B060' )

        dfs=df[df.kingdom == "Viruses"]
        for phylum in phyla_virus:
            dd = dfs[dfs.phylum == phylum][prop].dropna()
            if len(dd) > 10:
                length = dfs[dfs.phylum == phylum]["length"].dropna()
                data += [dd]
                labels += [phylum]
                box.set(color='orange', linewidth=2)
                box.set(facecolor = 'orange' )


        
        
                
        fig, ax = plt.subplots(figsize=(6,3))
        ax.set_ylabel('Fraction')
        plt.xticks(rotation=90,fontsize=6)
        bp = ax.boxplot(data,labels=labels,patch_artist=True)
        
        #dic_colors= {"Eukaryota":'#00B060', "Bacteria":'#E20048'}
        

        for l,box in zip(labels,bp['boxes']):
            if l in set(phyla_euk):
                box.set(color='#00B060', linewidth=2)
                box.set(facecolor = '#00B060' )
            elif l in set(phyla_virus):
                box.set(color='orange', linewidth=2)
                box.set(facecolor = 'orange' )
            elif l in set(phyla_arch):
                box.set(color='#5064d0', linewidth=2)
                box.set(facecolor = '#5064d0' )
            elif l in set(phyla_proks):
                box.set(color='#E20048', linewidth=2)
                box.set(facecolor = '#E20048' )
            else:
                box.set(color='grey', linewidth=2)
                box.set(facecolor = 'grey' )

        #~ pylab.setp(bp['boxes'],color='black',lw=2)
        plt.setp(bp['whiskers'], color='black')
        plt.setp(bp['medians'], color='black')
        plt.setp(bp['fliers'], color='black', marker='+')
        
    
        if prop in dic_limits:
            ax.set_ylim(dic_limits[prop])
        ax.yaxis.grid()
        ax.grid(color='grey', linestyle='-', linewidth=1)
        ax.xaxis.grid(False)
    
        ax.set_title(dic_labels[prop])
        
        figname = figures_dir + "boxplots_by_kingdom_phylum_" + prop + ".eps"
        print(figname)
        fig.savefig(figname,bbox_inches = "tight")
        #fig.show()
        plt.close()

        ##################################################
    
    dic_limits={"hessa":[0.85,1.25],"GC":[20,80],"GC_genomic":[20,80],"taxon_gc":[30,65],
                    "taxon_gc_r":[30,65],"gc_third":[30,65],
                'aa_corr':[0.8,1],'aa_corr_dot':[0.8,1.0],'aa_corr_r':[0.4,1],
                "top-idp":[0,0.15], 'W':[0.006,0.02], 'C':[0.01,0.04],
                'H':[0.01,0.04], 'M':[0.01,0.04], 'Y':[0.01,0.06],
                'Q':[0.02,0.05], 'P':[0.02,0.11], 'F':[0.01,0.08],
                'R':[0.02,0.1], 'G':[0,0.12], 'D':[0.02,0.06],
                'A':[0,0.15], 'V':[0.04,0.08], 'N':[0.01,0.08],
                'T':[0.04,0.08], 'E':[0.02,0.08], 'I':[0,0.12],
                'K':[0.01,0.1], 'S':[0.05,0.11], 'L':[0.06,0.12],
                'freq_W':[0.0,0.025], 'freq_C':[0.00,0.04],
                'freq_H':[0.01,0.035], 'freq_M':[0.01,0.065],
                'freq_Y':[0.01,0.06], 'freq_Q':[0.01,0.07],
                'freq_P':[0.0,0.08], 'freq_F':[0.018,0.08],
                'freq_R':[0.02,0.11], 'freq_G':[0.02,0.12],
                'freq_D':[0.02,0.1], 'freq_A':[0,0.18],
                'freq_V':[0.04,0.11], 'freq_N':[0.01,0.095],
                'freq_T':[0.03,0.08], 'freq_E':[0.02,0.1],
                'freq_I':[0.02,0.14], 'freq_K':[0.01,0.13],
                'freq_S':[0.03,0.11], 'freq_L':[0.075,0.15],
                "iupred_long":[0,40], "tm":[0,12],'idp':[0,0.20],
                "seg":[0,25],"length_aa":[0,720],'iupred_short':[0,40],
                'iupred_cutoff':[0,50],'volatility':[0.7,0.85],'hydropathy':[-1,1],
                "ss_alpha":[-0.05,0.03],"ss_beta":[-0.1,0.02],"ss_coil":[-0.03,0.02],"ss_turn":[-0.11,0.0],
                'slider':[10,80],'disembl_coils':[40,100],'disembl_rem465':[0,80],'disembl_hotloops':[10,90],
                "globpipe":[0,100] } # hardcoded axis limits


    dic_colors = {"Eukaryota":"#9daf6a", "Bacteria":"#b45a5c", "Archaea":"#377eb8","Viruses":"orange"}
    dic_colors_fit = {"Eukaryota":"#bdcf8a", "Bacteria":"#f43a3c", "Archaea":"#579ed8","Viruses":"orange"}
    dic_markers = {"Eukaryota":"o", "Bacteria":"x", "Archaea":"*","Viruses":"^"}
        

    props = ["freq_" + aa for aa in aas_sorted_topidp]

    mycotaxa = []
        
    properties_x = ["GC"]
    #properties_x = ["GC", "GC3", "GC_genomic"]

    #~ properties_y = [u'freq_A', u'freq_C', u'freq_D']
    properties_y = [u'freq_A', u'freq_C', u'freq_D', u'freq_E', u'freq_F', u'freq_G',
        u'freq_H', u'freq_I', u'freq_K', u'freq_L', u'freq_M', u'freq_N',
        u'freq_P', u'freq_Q', u'freq_R', u'freq_S', u'freq_T', u'freq_V',
        u'freq_W', u'freq_Y']

    # Now we plot properies X vs properties Y
    for prop_1 in properties_x:
        for prop_2 in props: # properties_y:
            fig, ax = plt.subplots(figsize=(6,6))
        
            # plot the values as a scatter plot
            prop_1_values = []
            prop_2_values = []
            #if prop_1 in df_genes_random.columns and prop_2 in df_genes_random.columns:
            #    prop_1_random = running_mean(pd.np.array(df_genes_random.sort_values(prop_1)[prop_1].astype(float)),500)
            #    prop_2_random = running_mean(pd.np.array(df_genes_random.sort_values(prop_1)[prop_2].astype(float)),500)
            #    ax.plot(prop_1_random,prop_2_random,color="black",label="Random")
            for kingdom in  kingdoms:
                #fig, ax = plt.subplots(figsize=(6,6))

                temp_df = df.loc[(df.kingdom == kingdom)][list(set([prop_1,prop_2]))].dropna()
                #temp_df=temp_df[temp_df[prop_1]  != '-']
                #temp_df[df[prop_2].apply(lambda x: x.isnumeric())]

                #print("TEST",prop_2,kingdom,temp_df.shape)
                prop_1_values = temp_df[prop_1].values
                prop_2_values = temp_df[prop_2].values

                    
                #print(prop_1_values)
                #print(prop_2_values)
                #print("TEST2",len(prop_1_values),len(prop_2_values))
                

                ax.scatter(prop_1_values,prop_2_values, edgecolor="none", facecolor=dic_colors[kingdom],alpha = 1,s=50,label=kingdom,marker=dic_markers[kingdom])                
                #figname = figures_dir + prop_2 + "_vs_" + prop_1 + kingdom + ".eps"
                #print(figname)
                #fig.savefig(figname)
                
            # plot the theoretical line
            #fig, ax = plt.subplots(figsize=(6,6))
            if prop_2.split("_")[0] == "freq" and prop_1 in ["GC","GC3","GC_genomic"]:
                aa = prop_2.split("_")[1]
                p = []
                gcY=[]
                for gc in np.arange(10,90,0.1):
                    p += [get_aa_freq(aa, gc/100)]
                    gcY += [gc]
                #ax.scatter(gcY,p, edgecolor="none", facecolor="black",alpha = 1,s=50,label="Theoretical")
                #ax.plot(np.arange(10,90,0.1),p,color="black",lw=2,label="Theoretical")
                #print("GC: ",gcY,p)
                ax.plot(gcY,p,color="black",lw=2,label="Theoretical")

            ax.set_xticks([20,30,40,50,60,70,80])
            ax.set_xlim([10,90])

            #set the limits
            #if prop_1 in dic_limits:
            #    print "Limit1",dic_limits[prop_1]
            #    ax.set_xlim(dic_limits[prop_1])
            #    ax.set_xticks
            #if prop_2 in dic_limits:
            #    #print ("Limit2",dic_limits[prop_2])
            #    ax.set_ylim(dic_limits[prop_2])
                
                
            # set the labels
            ax.set_xlabel(dic_labels.get(prop_1,prop_1))
            ax.set_ylabel(dic_labels.get(prop_2,prop_2)+" frequency")
            
            # plot legend
            ax.legend(loc="upper right")
            
            
        
            
            figname = figures_dir + prop_2 + "_vs_" + prop_1 + ".eps"
            print(figname)
            fig.savefig(figname)
            #fig.show()
