#!/usr/bin/env python3
from __future__ import print_function
import pickle


if __name__ == '__main__':


# coding: utf-8

# In[2]:
    import pandas as pd
    import os
    import functools
    import matplotlib 
    matplotlib.use('Agg')
    import functools
    from scipy.stats import ttest_ind
    from scipy.stats import mannwhitneyu
    from scipy.spatial import distance
    from scipy.stats import norm
    import math as math
    import numpy as np
    from scipy.stats.stats import rankdata, tiecorrect
    from scipy.stats import stats
    from scipy.stats.stats import linregress
    #matplotlib.mathtext.SHRINK_FACTOR = 1
    import seaborn as sns    
    #%matplotlib inline
    from scipy.optimize import curve_fit
    
    data_dir = "../data/"
    figures_dir = "../figures_medGC/"
    kingdoms = ["Bacteria","Archea","Eukaryota","Viruses"]
    
    if not os.path.exists(figures_dir):
        os.makedirs(figures_dir)

# open the three reference files
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    
    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks],sort=True)

    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


    taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    taxid2kingdom = {}

    for t in taxid2group:
        gr = taxid2group[t]
        taxid2kingdom[t] = group2kingdom[gr]
    
    taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']


    # load random genes
    df_genes_random = pd.read_feather(data_dir + "df_genes_random.feather")


    # load datasets (per species)
    pfam_sets = ["full", "pfam", "domains", "linkers", "C-linkers", "M-linkers", "N-linkers"]
    uniprot_sets = ["All","None","Shared","Unique"]
    datasets=pfam_sets+uniprot_sets
    #datasets += ["domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]

    
    dfs = {}
    for ty in pfam_sets:   
        print (data_dir + "df_" + ty+ "_annotation_by_species.csv")
        df = pd.read_csv(data_dir + "df_" + ty+ "_annotation_by_species.csv").dropna(axis=0)
        df["numdiso_long"]=df.iupred_long
        df["numdiso_short"]=df.iupred_short
        df["numseg"]=df.seg
        #print(ty,df.numdiso_short,df.iupred_short,df.length_translation)
        dfs[ty] = df

    #print ("full",dfs["full"])
    for ty in uniprot_sets:   
        print (data_dir + "df_uniprot_reference_proteomes_per_species-sum-" + ty+ ".csv")
        df = pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-" + ty+ ".csv").dropna(axis=0)
        df["numdiso_long"]=df.iupred_long
        df["numdiso_short"]=df.iupred_short
        df["numseg"]=df.seg
        #print(ty,df.numdiso_short,df.iupred_short,df.length_translation)

        dfs[ty] = df



    

    # these taxa should be excluded because their genetic code is different, so translation is wrong
    mycotaxa = []
    for t in name2taxid:
        tax = t.split()[0]
        if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
            mycotaxa += [name2taxid[t]]
        
    good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)

 

    # In[6]:

    # function to get a given AA frequency given the GC content
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']

            
    

    #get_aa_freq("ALL", 0.1)


    propsA = ["freq_" + aa for aa in aas_sorted_topidp]
#    props += ["iupred_long", "iupred_short", "top-idp", "topidp-S" ,"topidp-SKP" ,"topidp-SP" , "length"]
    propsB = ["seg","iupred_long", "iupred_short", "top-idp","hessa","ss_alpha", "ss_beta", "ss_coil", "ss_turn"]
    propsC = ["length_translation",'numdiso_long',"numdiso_short","numseg"]
    #print (df)
    props=propsA+propsB+propsC
    newprops = props
    newprops += ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    newprops += ["freq_" + aa + "-fifty" for aa in aas_sorted_topidp]

    
        
    minGC=20.
    maxGC=60.
    for d in datasets:
        dfs[d].dropna()

        for prop in propsA+propsB:
    # First we turn everything to frequencies
            dfs[d][prop]=pd.to_numeric(dfs[d][prop])
            length = dfs[d]["length_translation"]
            dfs[d][prop] = dfs[d][prop]/length

        for prop in propsC:
    # First we turn everything to frequencies
            dfs[d][prop]=pd.to_numeric(dfs[d][prop])
            num=dfs[d].count_protein
            #print (d,prop,dfs[d][prop],num)
            if ( prop != "length_translation" or d in uniprot_sets) :
                dfs[d][prop] = dfs[d][prop]/num
    
        for prop in ["GC_genomic"]:
            dfs[d][prop]=pd.to_numeric(dfs[d][prop],errors='coerce')
            dfs[d].dropna()
        #print("1",d,dfs[d].count())
        dfs[d]=dfs[d].loc[(dfs[d]["GC_genomic"].astype(float) > minGC) & (dfs[d]["GC_genomic"].astype(float)<maxGC) ]
        #print("2",d,dfs[d].count())

    properties_x = ["GC_genomic"]

    # Now we need to create GC-shift
    #kingdom='Bacteria' # We use both Eukaryotes and Bacteria
    #print (df)
    df = dfs["All"]
    df=df.dropna()

    # We only fit to bacteria to have this as a baseline..
    for prop_1 in properties_x:
        for prop_2 in propsA:
            props+=[prop_2+"-shift"]
            temp_df = df.loc[(df.kingdom.isin(['Bacteria'])) & (df.taxon_id.isin(good_taxa))][list(set([prop_1,prop_2,"iupred_short","length_translation"]))]
            #temp_df[prop_1]=pd.to_numeric(temp_df[prop_1])
            #temp_df[prop_2]=pd.to_numeric(temp_df[prop_2])
            #temp_df["iupred_short"]=pd.to_numeric(temp_df["iupred_short"])
            #temp_df["length_translation"]=pd.to_numeric(temp_df["length_translation"])
            temp_df=temp_df.dropna()
            prop_1_values = temp_df[prop_1].values
            prop_2_values = temp_df[prop_2].values
            fit = np.polyfit(list(prop_1_values),list(prop_2_values),1)
            fit_fn = np.poly1d(fit)
            fit2 = np.polyfit(list(prop_1_values),list(prop_2_values),2)
            fit2_fn = np.poly1d(fit2)
            valuefifty=fit_fn[0]+50.*fit_fn[1]
            valuefifty2=fit_fn[0]+50.*fit_fn[1]+50.*50.*fit_fn[2]
            # Apply is very slow
            # Linear fit gave strange Serine values..

            #print ("FIT",d,prop_2,fit_fn,valuefifty)
            #print ("FIT2",d,prop_2,fit2_fn,valuefifty2)
            for d in datasets:
                dfs[d][prop_2+"-exp"] = dfs[d][prop_1]*fit_fn[1]+fit_fn[0]
                dfs[d][prop_2+"-shift"] = dfs[d][prop_2]-dfs[d][prop_2+"-exp"]
                dfs[d][prop_2+"-fifty"] = dfs[d][prop_2+"-shift"]+valuefifty
    df = dfs["All"]
    for d in datasets:
        dfs[d].to_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-medGC-" + d+ ".csv")
        out_dic_fname = "props_dic_medGC_"+d+".pickle"        
        dic_res = {}
        dic_res[d] = {}
        with open(out_dic_fname, 'wb') as handle:
            for kingdom in ["Eukaryota", "Bacteria","Archaea","Viruses"]:
            #print "\t", kingdom
                dic_res[d][kingdom] = {}
                kdf = dfs[d].loc[dfs[d].kingdom == kingdom]
                adf = dfs["full"].loc[dfs["full"].kingdom == kingdom]
                for prop in newprops:
                    if prop == "length_translation" and d in pfam_sets:
                        dic_res[d][kingdom][prop] = kdf[prop].sum()/int(adf["count_protein"].sum())
                    else:
                        dic_res[d][kingdom][prop] = kdf[prop].mean()
        #print (ty,dic_res)
            pickle.dump(dic_res, handle, protocol=pickle.HIGHEST_PROTOCOL)
    # sys.exit()
