#!/usr/bin/env python
import matplotlib
matplotlib.use('Agg')
import pandas as pd
import numpy as np
import os
import re
import seaborn as sns
import matplotlib.pyplot as plt

aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
               'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
               'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}

dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)", "top-idp":"TOP-IDP", "topidp-S":"TOP-IDP - S","topidp-SKP":"TOP-IDP - SKP","topidp-SP":"TOP-IDP - SP",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'GC% (third nucleotide)','taxon_gc':'GC% (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
    'random':'Expected','iupred_long':'IUpred long (%AA)','length_aa':'<Length> (AA)','gc':'GC%',
    'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)', 'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM (%AA)',"volatility":"<Volatility>", 'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity> (Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with  expected', 'aa_corr_dot':'AA freq\ncorrelation with   expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A':
    'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P':'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'ALA', 'freq_C':
    'CYS', 'freq_E': 'GLU', 'freq_D': 'ASP', 'freq_G': 'GLY',
    'freq_F': 'PHE', 'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K':
    'LYS', 'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
    'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S': 'SER', 'freq_R':
    'ARG', 'freq_T': 'THR', 'freq_W': 'TRP', 'freq_V': 'VAL',
    'freq_Y': 'TYR', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Viruses':'Viruses'}
        
sns.set()
sns.set(font_scale=1.4)


data_dir="../data/"    
figures_dir="../figures_new/"    
filename=data_dir+'GOinfo.csv'
df = pd.read_csv(filename)
df = df.dropna()
# We delete all extreme Z-score as they are caused by zeros
df = df[df['Z'] < 2.5]  
df = df[df['Z'] > -2.5]  

sns.set()
sns.set(font_scale=2)
sns.set_style("whitegrid")

sns.set()
sns.set(font_scale=1.4)
sns.set_style("whitegrid")
    
# Histogram of Z-scores
for i in range(20):
    fig, ax = plt.subplots()
    dfs=df.loc[df.aa == aas[i]]["Z"]
    ax = sns.distplot(dfs)
    ax.set(xlim=(-2,2))
    plt.title('GO terms for '+dic_labels[aas[i]])
# Set x-axis label
    plt.xlabel('Z-score ')
# Set y-axis label
    plt.ylabel('Frequency')
    plt.savefig(figures_dir + "GO-"+aas[i]+"-Zscore.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close("all")

print ("\\documentclass[10pt,A4paper]{article}")
print ("\\usepackage{fullpage}")
print ("\\begin{document}")

    
# first a scatterplot
for i in range(20):
    fig, ax = plt.subplots()
    dfs=df.loc[df.aa == aas[i]]
    ax = sns.lmplot('mean_euk', # Horizontal axis
           'mean_bac', # Vertical axis
           data=dfs, # Data source
           fit_reg=False, # Don't fix a regression line
           size = 25 ) # size and dimension
    ax.set(ylim=(0, 0.15))
    ax.set(xlim=(0, 0.15))

    plt.title('Frequency in all GO of '+dic_labels[aas[i]])
# Set x-axis label
    plt.xlabel('Eukaryotic frequency')
# Set y-axis label
    plt.ylabel('Bacteria frequency')
    plt.plot([0, 0.15], [0, 0.15], linewidth=2)
    plt.savefig(figures_dir + "GO-"+aas[i]+"-freq.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close("all")

    # Now we want to plot min vs Z-score
    #dfs=dfs.assign(minsize = lambda x: np.minimum(x.bac_freq,x.euk_freq))
    #print (dfs)
    fig, ax = plt.subplots()
    ax = sns.lmplot('numPFAM', # Horizontal axis
           'Z', # Vertical axis
           data=dfs, # Data source
           fit_reg=False, # Don't fix a regression line
           size = 25 ) # size and dimension
    #ax.set(ylim=(0, 0.15))
    #ax.set(xlim=(0, 0.15))

    plt.title('Frequency in all GO terms for '+dic_labels[aas[i]])
# Set x-axis label
    plt.xlabel('num PFAM families')
# Set y-axis label
    plt.ylabel('Z-score')
    #plt.plot([0, 0.15], [0, 0.15], linewidth=2)
    plt.plot()
    plt.savefig(figures_dir + "GO-"+aas[i]+"-minsize.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close("all")

    #print (dfs.describe())
    dfspos=dfs.loc[dfs.Z >1.0]
    dfsneg=dfs.loc[dfs.Z <-1.0]
    sorted = dfspos[["GO","aa","bac_freq","euk_freq","numPFAM","Z"]].sort_values('Z')
    #print(sorted.to_latex(longtable=False))
    sorted = dfsneg[["GO","aa","bac_freq","euk_freq","numPFAM","Z"]].sort_values('Z')
    #print(sorted.to_latex(longtable=False))
    
    dfs["dif"]=dfs.euk_freq-dfs.bac_freq
    dfspos=dfs.loc[dfs.dif >0.02]
    dfsneg=dfs.loc[dfs.dif <-0.02]
    sorted = dfspos[["GO","aa","bac_freq","euk_freq","numPFAM","dif"]].sort_values('dif')
    print(sorted.to_latex(longtable=False))
    sorted = dfsneg[["GO","aa","bac_freq","euk_freq","numPFAM","dif"]].sort_values('dif')
    print(sorted.to_latex(longtable=False))

    
# Histogram of Z-scores
for i in range(20):
    fig, ax = plt.subplots()
    dfs=df.loc[df.aa == aas[i]]["Z"]
    ax = sns.distplot(dfs,color="#9daf6a",label="Eukaryota")
    ax.set(xlim=(-2,2))

    plt.title('GO for  '+dic_labels[aas[i]])
# Set x-axis label
    plt.xlabel('Z-score')
# Set y-axis label
    plt.ylabel('Frequency')
    plt.savefig(figures_dir + "GO-"+aas[i]+"-Zscore.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close("all")

#print (df)
# Histogram of Freq scores
for i in range(20):
    fig, ax = plt.subplots()
    #dfs=df.loc[df.aa == aas[i]]["Z"]
    dfs=df.loc[df.aa == aas[i]]
    ax = sns.distplot(dfs["bac_freq"],color="#b45a5c",label="Bacteria")
    #ax = sns.distplot(dfs["arch_freq"],color="#377eb8",label="Archaea")
    ax = sns.distplot(dfs["euk_freq"],color="#9daf6a",label="Eukaryota")
    #ax.set(xlim=(-2,2))
    ax.legend(loc="best",fontsize=12)

    plt.title('GO for  '+dic_labels[aas[i]])
# Set x-axis label
    plt.xlabel('Frequence')
# Set y-axis label
    plt.ylabel('Number')
    plt.savefig(figures_dir + "GO-"+aas[i]+"-freq-hist.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close("all")


df["bac_shift"]=df["bac_freq"]-df["euk_freq"]
df["arch_shift"]=df["arch_freq"]-df["euk_freq"]
df["euk_shift"]=df["euk_freq"]-df["euk_freq"]
#print (df)
for i in range(20):
    fig, ax = plt.subplots()
    #dfs=df.loc[df.aa == aas[i]]["Z"]
    dfs=df.loc[df.aa == aas[i]]
    ax = sns.distplot(dfs["bac_shift"],color="#b45a5c",label="Bacteria")
    ax = sns.distplot(dfs["arch_shift"],color="#377eb8",label="Archaea")
    #ax = sns.distplot(dfs["euk_shift"],color="#9daf6a",label="Eukaryota")
    ax.set(xlim=(-0.05,0.05))
    ax.legend(loc="best",fontsize=12)

    plt.title('Shift in Amino Acid Frequence compared with Eukaryota, '+dic_labels[aas[i]],fontsize=12)
# Set x-axis label
    plt.xlabel('Frequency shift',fontsize=12)
# Set y-axis label
    plt.ylabel('Frequency',fontsize=12)
    plt.savefig(figures_dir + "GO-"+aas[i]+"-shift-hist.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close("all")


print ("\\end{document}")
