#!/usr/bin/env python3
from __future__ import print_function

def get_aa_freq(aa,gc):
    codons = aa2codons[aa]
    prob_tot = []
    for codon in codons:
        prob_codon = []
        for N in codon:
            prob = gc/2
            if N in {"A","T","U"}:
                prob = 0.5-(gc/2)
            prob_codon += [prob]
        #print(codon, prob_codon,functools.reduce(mul,prob_codon))
        prob_tot += [functools.reduce(mul,prob_codon)] # Use functools.reduce() if you really need it; however, 99 percent of the time an explicit for loop is more readable
    prob_tot = sum(prob_tot)
    return prob_tot



##########################################################################################################################

# for each amino acid, and separately for eukarya and bacteria, plot its frequency versus GC% for 4 curves:
# 1- average freq
# 2- a linear fit of the data
# 3- the frequency as it should be if only dependent on GC
# 4- a model function (w * prob_tot + k) with estimated parameters W and K
# note: for our big dataset (from NCBI) it is also possible to plot against the GC% of the CDS

# w and k are the parameters to optimize
def my_func(x,y,w,k):
    global aminoacid
    codons = aa2codons[aminoacid]
    res = []
    for gc in x:
        prob_tot = []
        for codon in codons:
            prob_codon = []
            for N in codon:
                prob = gc/2
                if N in {"A","T","U"}:
                    prob = 0.5-(gc/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            prob_tot += [functools.reduce(mul,prob_codon)]
        prob_tot = sum(prob_tot)
        res += [w * prob_tot + k]
    return res


def get_error(obs,exp):
    err = []
    for o,e in zip(obs,exp):
        err += [abs(o - e)]
    return np.mean(err)


def clamp(value, min_v, max_v):
    return max(min(value, max_v), min_v)

##########################################################################################################################

# define a function to plot a bar plot of the difference between AA frequencies in two datasets
# d1, d2 = the two dataset (could be the same dataset); datasets are "dataset","dataset-eggnog","dataset-pfam","dataset-pfam-full"
# k1, k2 = the kingdoms: can be either "Eukaryota" or "Bacteria"


def bar_plot_s1_vs_s2(d1,d2,k1,k2,uselog = True):
    fig, ax = plt.subplots(figsize=(6.,3))
    data_aa = []
    for aa in aas_sorted_topidp:
        prop = "freq_" + aa
        prop_values = {}
        prop_values_1 = dfs[d1].loc[dfs[d1].kingdom == k1.encode("utf-8")][prop].mean()
        prop_values_2 = dfs[d2].loc[dfs[d2].kingdom == k2.encode("utf-8")][prop].mean()
        length_1 = dfs[d1].loc[dfs[d1].kingdom == k1.encode("utf-8")]["length_translation"].mean()
        length_2 = dfs[d2].loc[dfs[d2].kingdom == k2.encode("utf-8")]["length_translation"].mean()
        if uselog == True:
            value = np.log((prop_values_1/length_1)/(prop_values_2/length_2))
        else:
            value = prop_values_1/length_1-prop_values_2/length_2
        data_aa += [value]
    ax.bar(range(len(data_aa)),data_aa, color="red")
    ylim = max([abs(v) for v in ax.get_ylim()])
    ylim = 0.03
    ax.set_ylim([-ylim, ylim])
    ax.set_xticks([0.0 + x for x in range(len(aas_sorted_topidp))])
    ax.set_xticklabels(aas_sorted_topidp)
    if uselog == True:
        ax.set_ylabel("log( freq(" + k1+ ") / freq(" + k2+ ") )")
    else:
        ax.set_ylabel("freq(" + k1+ ") - freq(" + k2+ ")")
    title = dic_labels[k1] + " (" + d1 + ")" + " vs "  + dic_labels[k2] + " (" + d2 + ")"
    ax.set_title(title)
    ax.yaxis.grid()
    ax.xaxis.grid()
    if uselog == True:
        figname = figures_dir + "aa_freq_change_pfam2_log_odd_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 +  ".png"
    else:
        figname = figures_dir + "aa_freq_change_pfam2_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 + ".png"
    print(figname)
    fig.savefig(figname,dpi=300)
#    fig.show()


def fmt_scientific(x):
    a, b = '{:.2e}'.format(x).split('e')
    b = int(b)
    return r'${} \times 10^{{{}}}$'.format(a, b)

def running_mean(x, N):
    cumsum = pd.np.cumsum(pd.np.insert(x, 0, 0)) 
    return (cumsum[N:] - cumsum[:-N]) / N 



if __name__ == '__main__':


# coding: utf-8

# In[2]:

    import pandas as pd
    import os
    import functools
    import matplotlib 
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import numpy as np
    from scipy.stats.stats import rankdata, tiecorrect
    from scipy.stats import stats
    #from statsmodels.formula.api import ols
    from scipy.stats.stats import linregress
    #matplotlib.mathtext.SHRINK_FACTOR = 1
    
    #%matplotlib inline
    
    data_dir = "../data/"
    figures_dir = "../figures_new/"
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
    
    if not os.path.exists(figures_dir):
        os.makedirs(figures_dir)

    df = pd.read_csv(data_dir + "df_pfam_single_proteomes_with_pfam_and_kingdom_domains_shared.csv")
    print (df)
    pfam=df.pfamA_acc.unique()
    #pfam=df['pfamA_acc'].unique()
    df_bac = {}
    df_euk = {}
    b_data= []
    e_data= []
    for ty in pfam:   
        df_bac[ty] = df.loc[ (df.pfamA_acc == ty) & (df.kingdom=="Bacteria") & (df.length_translation>100) ]
        df_euk[ty] = df.loc[ (df.pfamA_acc == ty) & (df.kingdom=="Eukaryota") & (df.length_translation>100) ]
        for aa in  aas_sorted_topidp:
            b_data = df_bac[ty]["freq_"+aa]
            e_data = df_euk[ty]["freq_"+aa]
            if (len(b_data) > 10 and  len(e_data)>10):
                print (ty,aa,len(b_data),b_data.mean(),len(e_data),e_data.mean())
        
