#!/usr/bin/env python3
from __future__ import print_function



def darken(col):
    c = colors.hex2color(col)
    nc = []
    for e in c:
        ne = e-0.4
        if ne < 0: ne = 0
        nc += [ne]
    return nc

    

def twoSampZ(X1, X2, mudiff, sd1, sd2, n1, n2):
    pooledSE = sqrt(sd1**2/n1 + sd2**2/n2)
    z = ((X1 - X2) - mudiff)/pooledSE
    pval = 2*(1 - norm.cdf(abs(z)))
    return round(z, 3), round(pval, 4)

def get_topidp(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp[aa]*data["freq_"+aa]
    return score

def get_topidp_noser(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noser[aa]*data["freq_"+aa]
    return score

def get_topidp_noserpro(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noserpro[aa]*data["freq_"+aa]
    return score

def get_topidp_noserlyspro(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noserlyspro[aa]*data["freq_"+aa]
    return score
    
def get_aa_freq(aa,gc):
    codons = aa2codons[aa]
    prob_tot = []
    for codon in codons:
        prob_codon = []
        for N in codon:
            prob = gc/2
            if N in {"A","T","U"}:
                prob = 0.5-(gc/2)
            prob_codon += [prob]
        #print(codon, prob_codon,functools.reduce(mul,prob_codon))
        prob_tot += [functools.reduce(mul,prob_codon)] # Use functools.reduce() if you really need it; however, 99 percent of the time an explicit for loop is more readable
    prob_tot = sum(prob_tot)
    return prob_tot



##########################################################################################################################

# for each amino acid, and separately for eukarya and bacteria, plot its frequency versus GC% for 4 curves:
# 1- average freq
# 2- a linear fit of the data
# 3- the frequency as it should be if only dependent on GC
# 4- a model function (w * prob_tot + k) with estimated parameters W and K
# note: for our big dataset (from NCBI) it is also possible to plot against the GC% of the CDS

# w and k are the parameters to optimize
def my_func(x,y,w,k):
    global aminoacid
    codons = aa2codons[aminoacid]
    res = []
    for gc in x:
        prob_tot = []
        for codon in codons:
            prob_codon = []
            for N in codon:
                prob = gc/2
                if N in {"A","T","U"}:
                    prob = 0.5-(gc/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            prob_tot += [functools.reduce(mul,prob_codon)]
        prob_tot = sum(prob_tot)
        res += [w * prob_tot + k]
    return res

# App

def my_shift(x,y,w,k):
    res = []
    
    return res

def get_error(obs,exp):
    err = []
    for o,e in zip(obs,exp):
        err += [abs(o - e)]
    return np.mean(err)


def clamp(value, min_v, max_v):
    return max(min(value, max_v), min_v)

##########################################################################################################################

# define a function to plot a bar plot of the difference between AA frequencies in two datasets
# d1, d2 = the two dataset (could be the same dataset); datasets are "dataset","dataset-eggnog","dataset-pfam","dataset-pfam-full"
# k1, k2 = the kingdoms: can be either "Eukaryota" or "Bacteria"


def bar_plot_s1_vs_s2(d1,d2,k1,k2,weighted=False, uselog = False,shift=False,fifty=False):
    fig, ax = plt.subplots(figsize=(6.,3))
    data_aa = []
    if weighted or shift:
        aas=aas_sorted_topidp
    else:
        aas=aas_sorted_bacfreq
    for aa in aas:
        if shift:
            prop = "freq_" + aa + "-shift"
            name='aa_shift_change'
        elif fifty:
            prop = "freq_" + aa + "-fifty"
            name='aa_fifty_change'
        else:
            prop = "freq_" + aa
            name='aa_freq_change'
        prop_values = {}
        # We have changed this to do mean before summation - if not different numbers than in Z-test.
        #length_1 = dfs[d1].loc[(dfs[d1].kingdom == k1) & (dfs[d1].taxon_id.isin(good_taxa))]["length_translation"]
        #length_2 = dfs[d2].loc[(dfs[d2].kingdom == k2) & (dfs[d2].taxon_id.isin(good_taxa))]["length_translation"]
        prop_values_1 = np.mean(dfs[d1].loc[(dfs[d1].kingdom == k1) & (dfs[d1].taxon_id.isin(good_taxa))][prop])
        prop_values_2 = np.mean(dfs[d2].loc[(dfs[d2].kingdom == k2) & (dfs[d2].taxon_id.isin(good_taxa))][prop])
        if uselog:
            value = np.log((prop_values_1)/(prop_values_2))
        else:
            value = prop_values_1-prop_values_2
        if weighted:
            value *= 100*top_idp[aa]
        data_aa += [value]
    #pos = [data for data in data_aa if data >= 0]
    #neg = [data for data in data_aa if data < 0]

 
    x=range(len(data_aa))
    #ax.bar(x[pos],data_aa[pos], color="green")
    #ax.bar(x[neg],data_aa[neg], color="red")
    ax.bar(x,data_aa, color="green")
    ax.bar(x,data_aa, color="red")
    ylim = max([abs(v) for v in ax.get_ylim()])
    if weighted:
        ylim = 1.6
    #elif k == l :
    #    ylim = 1.1
    elif (uselog):
        ylim=3.0
    else:
        ylim=.032
    ax.set_ylim([-ylim, ylim])
    ax.set_xticks([0.0 + x for x in range(len(aas))])
    ax.set_xticklabels(aas)
    if uselog == True:
        ax.set_ylabel("log( freq(" + k1+ ") / freq(" + k2+ ") )")
    else:
        ax.set_ylabel("freq(" + k1+ ") - freq(" + k2+ ")")
        if weighted:
            ax.set_ylabel("TOP-IDP(" + k1+ " - " + k2+ ")")
        else:
            ax.set_ylabel("freq(" + k1+ ") - freq(" + k2+ ")")
    title = dic_labels[k1] + " (" + dic_labels[d1] + ")" + " vs "  + dic_labels[k2] + " (" + dic_labels[d2] + ")"
    ax.set_title(title)
    ax.yaxis.grid(True)
    ax.xaxis.grid(True)
    if uselog == True:
        if weighted:
            figname = figures_dir + name +"_weighted_plot_log_odd_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 +  ".eps"
        else:
            figname = figures_dir + name + "_plot_log_odd_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 +  ".eps"
    else:
        if weighted:
            figname = figures_dir + name +"_weighted_plot_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 + ".eps"
        else:
            figname = figures_dir + name + "_plot_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 + ".eps"
    print(figname)
    fig.savefig(figname,dpi=300)
    plt.close()
    #    fig.show()

def bar_plot(d1,k,weighted=False, uselog = False,shift=False,diff=False,fifty=False):
    fig, ax = plt.subplots(figsize=(6.,3))

    if shift:
        name='aa_shift'
    elif fifty:
        name='aa_fifty'
    elif diff:
        name='aa_diff'
    else:
        name='aa_freq'
    if weighted:
        aas=aas_sorted_topidp
    else:
        aas=aas_sorted_bacfreq

    offset_x = 0

    for kingdom in k:
        data_aa = []
        std_aa = []
        for aa in aas:
            if shift:
                prop = "freq_" + aa + "-shift"
            elif fifty:
                prop = "freq_" + aa + "-fifty"
            else:
                prop = "freq_" + aa
            #print (prop,kingdom,d1)
            if weighted:
                factor= top_idp[aa]*100
            else:
                factor=1.
            if diff:
                d="All"
                K=np.mean(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))][prop])
                Kstd=np.std(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))][prop])/np.sqrt(len(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))])-1)
                B=np.mean(dfs[d].loc[(dfs[d].kingdom == "Bacteria") & (dfs[d].taxon_id.isin(good_taxa))][prop])
                Bstd=np.mean(dfs[d].loc[(dfs[d].kingdom == "Bacteria") & (dfs[d].taxon_id.isin(good_taxa))][prop])/np.sqrt(len(dfs[d1].loc[(dfs[d1].kingdom == "Bacteria") & (dfs[d1].taxon_id.isin(good_taxa))])-1)
                data_aa+=[(K-B)*factor]
                std_aa+=[factor*(Kstd+Bstd)/2]
            else:
                data_aa+=[factor*np.mean(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))][prop])]
                std_aa+=[factor*np.std(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))][prop])/np.sqrt(len(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))])-1)]
                #print (data_aa)
        ax.bar([x+offset_x for x in range(len(data_aa))], data_aa ,yerr=std_aa,color=dic_colors[kingdom],width=0.3,label=dic_labels[kingdom])
        offset_x += 0.25
    ax.set_xticks([x+0.25 for x in range(len(data_aa))])
    #ax.set_xticklabels([dic_labels[x] for x in aas_sorted_bacfreq], rotation=90 )
    ax.set_xticklabels( aas, rotation=0 )
    if shift or diff:
        if not weighted:
            ylim=3.
            ax.set_ylim([-ylim, ylim])
    else:
        ylim=0.11
        ax.set_ylim([0, ylim])
        
    #if prop in dic_boxplot_limits:
    #    ax.set_ylim(dic_boxplot_limits[prop])
    
    ax.yaxis.grid(True)
    ax.set_title(dic_labels[d1])
    ax.set_ylabel("Frequency")
    ax.legend(loc="best")
    if weighted:
        figname = figures_dir + name +"_weighted_plot_" + d1 + ".eps"
    else:
        figname = figures_dir + name + "_plot_" + d1 +  ".eps"
    print(figname)
    fig.savefig(figname,dpi=300)
    plt.close()
    #    fig.show()

def xy_plot(d1,k,weighted=False,shift=False,fifty=False):
    fig, ax = plt.subplots(figsize=(6.,6.))

    if shift:
        name='xy_shift'
    elif fifty:
        name='xy_fifty'
    else:
        name='xy_freq'
    if weighted:
        aas=aas_sorted_topidp
    else:
        aas=aas_sorted_bacfreq

    data_bac = []
    std_bac = []
    kingdom="Bacteria"
    for aa in aas:
        if shift:
            prop = "freq_" + aa + "-shift"
        elif fifty:
            prop = "freq_" + aa + "-fifty"
        else:
            prop = "freq_" + aa
            #print (prop,kingdom,d1)
        data_bac+=[np.mean(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))][prop])]
        std_bac+=[2.58*np.std(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))][prop])/np.sqrt(len(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))])-1)]

    for kingdom in k:
        if kingdom=="Bacteria":
            continue
        data_aa = []
        std_aa = []
        for aa in aas:
            if shift:
                prop = "freq_" + aa + "-shift"
            elif fifty:
                prop = "freq_" + aa + "-fifty"
            else:
                prop = "freq_" + aa
            #print (prop,kingdom,d1)
            data_aa+=[np.mean(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))][prop])]
            std_aa+=[2.58*np.std(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))][prop])/np.sqrt(len(dfs[d1].loc[(dfs[d1].kingdom == kingdom) & (dfs[d1].taxon_id.isin(good_taxa))])-1)]
        if weighted:
            data_aa *= top_idp[aa]
            std_aa *= top_idp[aa]
        #print (data_aa,std_aa)
        ax.errorbar(x=data_bac, y=data_aa, xerr=std_bac, yerr=std_aa, label=kingdom,color=dic_colors[kingdom], fmt='o')
        for i, txt in enumerate(aas):
            ax.annotate(txt, (data_bac[i], data_aa[i]))
    if shift:
        ylim=0.03
        ax.set_ylim([-ylim, ylim])
        ax.set_xlim([-ylim, ylim])
        x=[-ylim,ylim]
        y=[-ylim,ylim]
        ax.plot(x,y,color="Black")
    else:
        ylim=0.11
        ax.set_ylim([0, ylim])
        ax.set_xlim([0, ylim])
        x=[0,ylim]
        y=[0,ylim]
        ax.plot(x,y,color="Black")

    #if prop in dic_boxplot_limits:
    #    ax.set_ylim(dic_boxplot_limits[prop])
    
    #ax.yaxis.grid(True)
    ax.set_title(dic_labels[d1])
    ax.set_ylabel("Frequency")
    ax.set_xlabel("Bacterial frequency")
    ax.legend(loc="best")
    if weighted:
        figname = figures_dir + name +"_weighted_plot_" + d1 + ".eps"
    else:
        figname = figures_dir + name + "_plot_" + d1 +  ".eps"
    print(figname)
    fig.savefig(figname,dpi=300)
    plt.close()
    #    fig.show()


def fmt_scientific(x):
    a, b = '{:.2e}'.format(x).split('e')
    b = int(b)
    return r'${} \times 10^{{{}}}$'.format(a, b)

def running_mean(x, N):
    cumsum = pd.np.cumsum(pd.np.insert(x, 0, 0)) 
    return (cumsum[N:] - cumsum[:-N]) / N 



if __name__ == '__main__':


# coding: utf-8

# In[2]:
    from operator import mul
    import pandas as pd
    import os
    import functools
    import matplotlib 
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import matplotlib.patches as mpatches
    import seaborn as sns
    import functools
    from scipy.stats import ttest_ind
    from scipy.stats import mannwhitneyu
    from scipy.spatial import distance
    from scipy.stats import norm
    import math as math
    from matplotlib import colors
    from matplotlib import cm
    import numpy as np
    from scipy.stats.stats import rankdata, tiecorrect
    from scipy.stats import stats
    #from statsmodels.formula.api import ols
    from scipy.stats.stats import linregress
    #matplotlib.mathtext.SHRINK_FACTOR = 1
    import seaborn as sns    
    #%matplotlib inline
    
    data_dir = "../data/"
    figures_dir = "../figures_new/"
    kingdoms = ["Bacteria","Archaea","Eukaryota"]
    twokingdoms = ["Prokaryota","Eukaryota"]
    
    if not os.path.exists(figures_dir):
        os.makedirs(figures_dir)

# open the three reference files
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    
    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks],sort=True)

    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


    taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    taxid2kingdom = {}

    for t in taxid2group:
        gr = taxid2group[t]
        taxid2kingdom[t] = group2kingdom[gr]
    
    taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']


    # load random genes
    df_genes_random = pd.read_feather(data_dir + "df_genes_random.feather")


    # load datasets (per species)
    pfam_sets = ["full", "pfam", "domains", "linkers", "N-linkers", "M-linkers", "C-linkers"]
    uniprot_sets = ["All","Shared","Unique","None"]
    datasets=uniprot_sets+pfam_sets

    pfam_sets2 = ["pfam", "domains", "linkers"]
    uniprot_sets2 = ["Shared","Unique","None"]
    datasets2=uniprot_sets2+pfam_sets2
    #datasets += ["domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]

    
    dfs = {}
    '''
    for ty in pfam_sets:   
        print (data_dir + "df_" + ty+ "_annotation_by_species.csv")
        df = pd.read_csv(data_dir + "df_" + ty+ "_annotation_by_species.csv").dropna(axis=0)
        df["numdiso_long"]=df.iupred_long*df.length_translation
        df["numdiso_short"]=df.iupred_short*df.length_translation
        df["numseg"]=df.seg*df.length_translation
        df.iupred_long = df.iupred_long*100
        df.iupred_short = df.iupred_short*100
        df.seg = df.seg*100
        dfs[ty] = df

    #print ("full",dfs["full"])
    for ty in uniprot_sets:   
        print (data_dir + "df_uniprot_reference_proteomes_per_species-sum-" + ty+ ".csv")
        df = pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-" + ty+ ".csv").dropna(axis=0)
        df["numdiso_long"]=df.iupred_long*df.length_translation
        df["numdiso_short"]=df.iupred_short*df.length_translation
        df["numseg"]=df.seg*df.length_translation
        df.iupred_long = df.iupred_long*100
        df.iupred_short = df.iupred_short*100
        df.seg = df.seg*100
        dfs[ty] = df
    print ("All",dfs["Shared"])
    '''


    


    # these taxa should be excluded because their genetic code is different, so translation is wrong
    mycotaxa = []
    eurotaxa = []
    for t in name2taxid:
        tax = t.split()[0]
        if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
            mycotaxa += [name2taxid[t]]
        if tax in ["Euryarchaeota"]:
            eurotaxa += [name2taxid[t]]
        

    # print some statistics
    #for ds in datasets:
    #    print(ds)
    #    print(dfs[ds].kingdom.dropna().value_counts())
    #    print()
    

    # In[6]:

    # function to get a given AA frequency given the GC content
    aas_sorted_topidp =  ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
    aas_sorted_bacfreq = ['L','A','G','V','I','E','D','S','T','R','K','P','F','N','Q','Y','M','H','W','C']

    codons2aa = {
        'ATA':'I', 'ATC':'I', 'ATT':'I', 'ATG':'M', 'ACA':'T', 'ACC':'T', 'ACG':'T', 'ACT':'T',
        'AAC':'N', 'AAT':'N', 'AAA':'K', 'AAG':'K', 'AGC':'S', 'AGT':'S', 'AGA':'R', 'AGG':'R',
        'CTA':'L', 'CTC':'L', 'CTG':'L', 'CTT':'L', 'CCA':'P', 'CCC':'P', 'CCG':'P', 'CCT':'P',
        'CAC':'H', 'CAT':'H', 'CAA':'Q', 'CAG':'Q', 'CGA':'R', 'CGC':'R', 'CGG':'R', 'CGT':'R',
        'GTA':'V', 'GTC':'V', 'GTG':'V', 'GTT':'V', 'GCA':'A', 'GCC':'A', 'GCG':'A', 'GCT':'A',
        'GAC':'D', 'GAT':'D', 'GAA':'E', 'GAG':'E', 'GGA':'G', 'GGC':'G', 'GGG':'G', 'GGT':'G',
        'TCA':'S', 'TCC':'S', 'TCG':'S', 'TCT':'S', 'TTC':'F', 'TTT':'F', 'TTA':'L', 'TTG':'L',
        'TAC':'Y', 'TAT':'Y', 'TAA':'_', 'TAG':'_', 'TGC':'C', 'TGT':'C', 'TGA':'_', 'TGG':'W'}
        
    aa2codons = {}
    aa2codons["ALL"] = []
    for c in codons2aa:
        aa2codons[codons2aa[c]] = aa2codons.get(codons2aa[c],[]) + [c]
        if c != "_":
            aa2codons["ALL"] += [c]
            
    from operator import mul
    from scipy.optimize import curve_fit
    

    #get_aa_freq("ALL", 0.1)


    # In[7]:

    dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)",                'other':'Intermediate',                'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)",                "top-idp":"TOP-IDP", "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>",
    'ss_turn':"<Turn propensity>","length_translation":"Length (AA)",                'gc_third':'GC% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred long(%AA)','length_aa':'<Length> (AA)','gc':'GC%',
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff (%AA)',
                'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM(%AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff(%AA)','seg':'<SEG>', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':
                'TYR', 'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                      'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                      'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                      'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                      'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                      'freq_T': 'THR', 'freq_W':                'TRP', 'freq_V': 'VAL',
                      'freq_Y': 'TYR', 'scampi':'TM  (%AA)', 'slider':'Slider',
                      'disembl_coils':'DisEMBL                Coils (%AA)',
                      'disembl_rem465':"DisEMBL REM465 (%AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (%AA)",
                      "globpipe":"GlobPlot (%AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Prokaryota':'Prokaryota',
                "full":"Full", "pfam":"Shared domains", "domains":"Specific domains", 
                "linkers":"Linker regions",
                "N-linkers":"N-term Linker regions",
                "M-linkers":"Middle Linker regions",
                "C-linkers":"C-term Linker regions",
                  "All":"All proteins","Unique":"Specific Proteins","Shared":"Shared Proteins","None":"No Domain proteins"
                     ,"numdiso_long":"Num disorder residues (long)"
                     ,"numdiso_short":"Num disorder residues (short)"
                     ,"numseg":"Low complexity residues",
                      'freq_A-shift': '$ALA^*$', 'freq_C-shift': '$CYS^*$', 'freq_E-shift': '$GLU^*$',
                      'freq_D-shift': '$ASP^*$', 'freq_G-shift': '$GLY^*$', 'freq_F-shift': '$PHE^*$',
                      'freq_I-shift': '$ILE^*$', 'freq_H-shift': '$HIS^*$', 'freq_K-shift': '$LYS^*$',
                      'freq_M-shift': '$MET^*$', 'freq_L-shift': '$LEU^*$', 'freq_N-shift': '$ASN^*$',
                      'freq_Q-shift': '$GLN^*$', 'freq_P-shift': '$PRO^*$', 'freq_S-shift':'$SER^*$', 
                      'freq_T-shift': '$THR^*$', 'freq_W-shift': '$TRP^*$', 'freq_V-shift': '$VAL^*$',
                      'freq_Y-shift': '$TYR^*$', 'freq_R-shift': '$ARG^*$' ,
                      'freq_A-fifty': '$ALA^*$', 'freq_C-fifty': '$CYS^*$', 'freq_E-fifty': '$GLU^*$',
                      'freq_D-fifty': '$ASP^*$', 'freq_G-fifty': '$GLY^*$', 'freq_F-fifty': '$PHE^*$',
                      'freq_I-fifty': '$ILE^*$', 'freq_H-fifty': '$HIS^*$', 'freq_K-fifty': '$LYS^*$',
                      'freq_M-fifty': '$MET^*$', 'freq_L-fifty': '$LEU^*$', 'freq_N-fifty': '$ASN^*$',
                      'freq_Q-fifty': '$GLN^*$', 'freq_P-fifty': '$PRO^*$', 'freq_S-fifty':'$SER^*$', 
                      'freq_T-fifty': '$THR^*$', 'freq_W-fifty': '$TRP^*$', 'freq_V-fifty': '$VAL^*$',
                      'freq_Y-fifty': '$TYR^*$', 'freq_R-fifty': '$ARG^*$' }

    ylabels = {"GC":"Fraction","GC_genomic":"Fraction", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity>", "top-idp":"<TOP-IDP>","topidp-S":"<TOP-IDP-noS>","topidp-SKP":"<TOP-IDP-noSKP>","topidp-SP":"<TOP-IDP-noSP>",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'fraction','taxon_gc':'fraction','taxon_gc_r':'fraction',
    'random':'fraction','iupred_long':'Fraction of AA','length_aa':'<Length>','gc':'fraction',
    'iupred_short':'Fration of residues','iupred_cutoff':'Fraction of residues', 'idp':'<TOP-IDP>','tm':'Fraction of residues',"volatility":"<Volatility>", 'iupred_short':'Fraction of residues','iupred_cutoff':'Fraction of residues','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity>','gc':'Fraction','aa_corr':'Correlation', 'aa_corr_dot':'correlation','aa_corr_r':'Correlation', 'A':'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'Fraction of residues', 'freq_C':
    'Fraction of residues', 'freq_E': 'Fraction of residues', 'freq_D': 'Fraction of residues', 'freq_G': 'Fraction of residues',
    'freq_F': 'Fraction of residues', 'freq_I': 'Fraction of residues', 'freq_H': 'Fraction of residues', 'freq_K':
    'Fraction of residues', 'freq_M': 'Fraction of residues', 'freq_L': 'Fraction of residues', 'freq_N': 'Fraction of residues',
    'freq_Q': 'Fraction of residues', 'freq_P': 'Fraction of residues', 'freq_S': 'Fraction of residues', 'freq_R':
    'Fraction of residues', 'freq_T': 'Fraction of residues', 'freq_W': 'Fraction of residues', 'freq_V': 'Fraction of residues',
    'freq_Y': 'Fraction of residues', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea'
                     ,"numdiso_long":"Num disorder residues (long)"
                     ,"numdiso_short":"Num disorder residues (short)"
                     ,"numseg":"Low complexity residues"                ,
                     'freq_A-shift': 'Shift', 'freq_C-shift': 'Shift', 'freq_E-shift': 'Shift',
                      'freq_D-shift': 'Shift', 'freq_G-shift': 'Shift', 'freq_F-shift': 'Shift',
                      'freq_I-shift': 'Shift', 'freq_H-shift': 'Shift', 'freq_K-shift': 'Shift',
                      'freq_M-shift': 'Shift', 'freq_L-shift': 'Shift', 'freq_N-shift': 'Shift',
                      'freq_Q-shift': 'Shift', 'freq_P-shift': 'Shift', 'freq_S-shift':'SER', 'freq_R-shift': 'Shift',
                      'freq_T-shift': 'Shift', 'freq_W-shift':        'Shift', 'freq_V-shift': 'Shift',
                      'freq_Y-shift': 'Shift' ,
                     'freq_A-fifty': 'Frequency', 'freq_C-fifty': 'Frequency', 'freq_E-fifty': 'Frequency',
                      'freq_D-fifty': 'Frequency', 'freq_G-fifty': 'Frequency', 'freq_F-fifty': 'Frequency',
                      'freq_I-fifty': 'Frequency', 'freq_H-fifty': 'Frequency', 'freq_K-fifty': 'Frequency',
                      'freq_M-fifty': 'Frequency', 'freq_L-fifty': 'Frequency', 'freq_N-fifty': 'Frequency',
                      'freq_Q-fifty': 'Frequency', 'freq_P-fifty': 'Frequency', 'freq_S-fifty':'SER', 'freq_R-fifty': 'Frequency',
                      'freq_T-fifty': 'Frequency', 'freq_W-fifty':        'Frequency', 'freq_V-fifty': 'Frequency',
                      'freq_Y-fifty': 'Frequency' }

    # the properties to plot
    properties_x = ["GC_genomic"]
    #properties_x = ["GC", "GC3", "GC_genomic"]

    properties_y = ['top-idp', 'hessa', 'iupred_long', 'iupred_short', 'seg',
                    'ss_alpha', 'ss_beta', 'ss_coil', 'ss_turn','numdiso_long','numdiso_short','numseg',
        'freq_A', 'freq_C', 'freq_D', 'freq_E', 'freq_F', 'freq_G',
        'freq_H', 'freq_I', 'freq_K', 'freq_L', 'freq_M', 'freq_N',
        'freq_P', 'freq_Q', 'freq_R', 'freq_S', 'freq_T', 'freq_V',
        'freq_W', 'freq_Y']
        
        
    
    top_idp = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.341,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    top_idp_noser = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    top_idp_noserlyspro = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0., 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0., 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    top_idp_noserpro = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0., 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}
        
        
    dic_limits={"hessa":[0.85,1.25],"GC":[20,80],"GC_genomic":[20,80],"taxon_gc":[30,65],"taxon_gc_r":[30,65],"gc_third":[30,65],
                'aa_corr':[0.8,1],'aa_corr_dot':[0.8,1.0],'aa_corr_r':[0.4,1],
                "top-idp":[0,0.15], 'W':[0.006,0.02], 'C':[0.01,0.04],
                'H':[0.01,0.04], 'M':[0.01,0.04], 'Y':[0.01,0.06],
                'Q':[0.02,0.05], 'P':[0.02,0.11], 'F':[0.01,0.08],
                'R':[0.02,0.1], 'G':[0,0.12], 'D':[0.02,0.06],
                'A':[0,0.15], 'V':[0.04,0.08], 'N':[0.01,0.08],
                'T':[0.04,0.08], 'E':[0.02,0.08], 'I':[0,0.12],
                'K':[0.01,0.1], 'S':[0.05,0.11], 'L':[0.06,0.12],
                'freq_W-shift':[-0.2,0.2], 'freq_C-shift':[-0.2,0.2],
                'freq_H-shift':[-0.2,0.2], 'freq_M-shift':[-0.2,0.2],
                'freq_Y-shift':[-0.2,0.2], 'freq_Q-shift':[-0.2,0.2],
                'freq_P-shift':[-0.2,0.2], 'freq_F-shift':[-0.2,0.2],
                'freq_R-shift':[-0.2,0.2], 'freq_G-shift':[-0.2,0.2],
                'freq_D-shift':[-0.2,0.2], 'freq_A-shift':[-0.2,0.2],
                'freq_V-shift':[-0.2,0.2], 'freq_N-shift':[-0.2,0.2],
                'freq_T-shift':[-0.2,0.2], 'freq_E-shift':[-0.2,0.2],
                'freq_I-shift':[-0.2,0.2], 'freq_K-shift':[-0.2,0.2],
                'freq_S-shift':[-0.2,0.2], 'freq_L-shift':[-0.2,0.2],
                    'freq_W':[0.0,0.025], 'freq_C':[0.00,0.04],
                'freq_H':[0.01,0.035], 'freq_M':[0.01,0.065],
                'freq_Y':[0.01,0.06], 'freq_Q':[0.01,0.07],
                'freq_P':[0.0,0.08], 'freq_F':[0.018,0.08],
                'freq_R':[0.02,0.11], 'freq_G':[0.02,0.12],
                'freq_D':[0.02,0.1], 'freq_A':[0,0.18],
                'freq_V':[0.04,0.11], 'freq_N':[0.01,0.095],
                'freq_T':[0.03,0.08], 'freq_E':[0.02,0.1],
                'freq_I':[0.02,0.14], 'freq_K':[0.01,0.13],
                'freq_S':[0.04,0.11], 'freq_L':[0.075,0.15],
                    'freq_W-fifty':[0.0,0.025], 'freq_C-fifty':[0.00,0.04],
                'freq_H-fifty':[0.01,0.035], 'freq_M-fifty':[0.01,0.065],
                'freq_Y-fifty':[0.01,0.06], 'freq_Q-fifty':[0.01,0.07],
                'freq_P-fifty':[0.0,0.08], 'freq_F-fifty':[0.018,0.08],
                'freq_R-fifty':[0.02,0.11], 'freq_G-fifty':[0.02,0.12],
                'freq_D-fifty':[0.02,0.1], 'freq_A-fifty':[0,0.18],
                'freq_V-fifty':[0.04,0.11], 'freq_N-fifty':[0.01,0.095],
                'freq_T-fifty':[0.03,0.08], 'freq_E-fifty':[0.02,0.1],
                'freq_I-fifty':[0.02,0.14], 'freq_K-fifty':[0.01,0.13],
                'freq_S-fifty':[0.04,0.11], 'freq_L-fifty':[0.075,0.15],
                "iupred_long":[0,40], "tm":[0,12],'idp':[0,0.20],
                "seg":[0,25],"length_aa":[0,720],'iupred_short':[0,40],
                'iupred_cutoff':[0,50],'volatility':[0.7,0.85],'hydropathy':[-1,1],
                "ss_alpha":[-0.05,0.03],"ss_beta":[-0.1,0.02],"ss_coil":[-0.03,0.02],"ss_turn":[-0.11,0.0],
                'slider':[10,80],'disembl_coils':[40,100],'disembl_rem465':[0,80],'disembl_hotloops':[10,90],
                "globpipe":[0,100] } # hardcoded axis limits


    dic_colors = {"Eukaryota":"#9daf6a", "Bacteria":"#b45a5c", "Archaea":"#377eb8"}
    dic_colors_fit = {"Eukaryota":"#bdcf8a", "Bacteria":"#f43a3c", "Archaea":"#579ed8"}
    dic_markers = {"Eukaryota":"o", "Bacteria":"x", "Archaea":"o"}

    
    boxcolr=["red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","blue","blue","blue","green"]
    boxcolrnoS=["red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","blue","blue","blue","lightblue","lightblue","lightblue","green"]

    propsA = ["freq_" + aa for aa in aas_sorted_topidp]
#    props += ["iupred_long", "iupred_short", "top-idp", "topidp-S" ,"topidp-SKP" ,"topidp-SP" , "length"]
    propsB = ["iupred_long", "iupred_short", "top-idp",'numdiso_long',"numdiso_short","numseg"]
    propsC = ["length_translation"]
    #print (df)
    props=propsA+propsB+propsC

    
        
    dfs={}
    for d in datasets:
        df=pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-GC-" + d+ ".csv").dropna(axis=0)
        # Now we add a "prokayotes" kingdom
        tempdf=df[((df.kingdom == "Archaea") | (df.kingdom == "Bacteria"))]
        tempdf.kingdom="Prokaryota"
        dfs[d]=pd.concat([df, tempdf], ignore_index=True)
        # sys.exit()

    df=dfs["All"]
    df=df.dropna()
    good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)
    
    print ("Here we start plotting...")
    #************************************************************************
    #print (df)
    for d in datasets:
        xy_plot(d,  kingdoms,False,False,False)
        xy_plot(d,  kingdoms,False,True,False)
        xy_plot(d,  kingdoms,False,False,True)
    for d in datasets:
        bar_plot(d,  kingdoms,False,False,False,False,False)
        bar_plot(d,  kingdoms,False,False,False,True,False)
        bar_plot(d,  kingdoms,False,False,True,False,False)
        bar_plot(d,  kingdoms,False,False,False,False,True)
        bar_plot(d,  kingdoms,True,False,False,False,False)
        bar_plot(d,  kingdoms,True,False,False,True,False)
        bar_plot(d,  kingdoms,True,False,True,False,False)
        bar_plot(d,  kingdoms,True,False,False,False,True)
    for k in twokingdoms:
            for j in twokingdoms:
                if (j>k):
                    continue
                for d in datasets:
                        for e in datasets:
                            if (d==e and k==j):
                                continue
                            if (d!=e and k!=j):
                                continue
                            bar_plot_s1_vs_s2(e,d, j, k,False,False,False,False)
                            bar_plot_s1_vs_s2(e,d, j, k,False,True,False,False)
                            bar_plot_s1_vs_s2(e,d, j, k,False,False,False,True)
                            bar_plot_s1_vs_s2(e,d, j, k,True,False,False,False)
                            bar_plot_s1_vs_s2(e,d, j, k,False,False,True,False)
                            bar_plot_s1_vs_s2(e,d, j, k,True,False,True,False)
    for k in kingdoms:
            for j in kingdoms:
                if (j<k):
                    continue
                for d in datasets:
                        for e in datasets:
                            if (d==e and k==j):
                                continue
                            if (d!=e and k!=j):
                                continue
                            bar_plot_s1_vs_s2(e,d, j, k,False,False,False,False)
                            bar_plot_s1_vs_s2(e,d, j, k,True,False,False,False)
                            bar_plot_s1_vs_s2(e,d, j, k,False,False,False,True)
                            bar_plot_s1_vs_s2(e,d, j, k,True,False,False,False)
                            bar_plot_s1_vs_s2(e,d, j, k,False,False,True,False)
                            bar_plot_s1_vs_s2(e,d, j, k,True,False,True,False)

    # sys.exit()
    # ************************************************************************
    props += ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    props += ["freq_" + aa + "-fifty" for aa in aas_sorted_topidp]
    for prop in props:
        print (prop)
        fig, axes = plt.subplots()


        data = []

        for k in kingdoms:
                #if prop in ["iupred_long", "iupred_short"]:
                #    d = d*100  # ALready multiplied with 100
            #d = [temp/length]
                #print (k,length,d)
            d = df.loc[(df.kingdom == k ) & (df.taxon_id.isin(good_taxa))][prop]
            data += [d]
            #print (data)
    #axes.violinplot(dataset = data, showmedians = True)
        bp = axes.boxplot(data,patch_artist=True)
    
        #dic_colors = {0:'#E20048', 1:'#5064d0',2:'#00B060',  3:'orange'}
        dic_colors = {0:'#E20048', 1:'#00B060',  2:'#5064d0'}
    
        for i,box in enumerate(bp['boxes']):
            box.set(color = dic_colors[i], linewidth=2)
            box.set(facecolor = dic_colors[i])
        
        
    #~ pylab.setp(bp['boxes'],color='black',lw=2)
        plt.setp(bp['whiskers'], color='black')
        plt.setp(bp['medians'], color='black')
        plt.setp(bp['fliers'], color='black', marker='+')

        axes.set_xticks(range(1,len(kingdoms)+1))
        axes.set_xticklabels(kingdoms)
        axes.set_ylabel(ylabels[prop])
        axes.set_title(dic_labels[prop])
        
        #axes.set_ylabel(dic_labels[prop])

    #~ plt.show()
        fig.savefig(figures_dir+"boxplot_kingdoms_" + prop + ".eps")
        plt.close()

    # ************************************************************************
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    props += ["top-idp","iupred_long","iupred_short","length_translation"]
    #props += ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    #datasets += uniprot_sets

    #for d in datasets:
    #    print(d,dfs[d])
    
    for k in kingdoms:
        for l in kingdoms:
            if (k < l):
                continue
            for d1 in datasets:
                for d2 in datasets:
                    if (d1==d2 and k==l):
                        continue
                    if (d1!=d2 and k!=l):
                        continue
                    data=[]
                    rawdata=[]
                    fig, ax = plt.subplots(figsize=(6.,3))
                    for prop in props:
                        cat1 = dfs[d1].loc[(dfs[d1].kingdom == k) & (dfs[d1].taxon_id.isin(good_taxa))][prop]
                        cat2 = dfs[d2].loc[(dfs[d2].kingdom == l) & (dfs[d2].taxon_id.isin(good_taxa))][prop]

                        a, b = ttest_ind(cat1, cat2,equal_var=False)
                        c, d  = mannwhitneyu(cat1, cat2,alternative='two-sided')
                        mean1=np.mean(cat1)
                        mean2=np.mean(cat2)
                        sd1=np.sqrt(np.var(cat1))
                        sd2=np.sqrt(np.var(cat2))
                        Z=(mean1-mean2)/math.sqrt(sd1*sd1+sd2*sd2)
                        print ("T-test: ",prop,k,l,d1,d2,mean1,mean2,a,b,c,d)
                        print ("Z-score: ",prop,k,l,d2,d2,mean1,mean2,sd1,sd2,Z)
                        data += [Z]
                        if (prop in ["top-idp","iupred_long","iupred_short"]):
                            mean1=mean1/100
                            mean2=mean2/100
                        rawdata += [mean1-mean2]
                    figname=figures_dir+"/boxplot-Z-test_"+k + "_" + d1 +"_vs_" + l + "_" + d2 + ".eps"
                    ax.bar(range(len(data)),data, color=boxcolr)
                    if k == l:
                        ax.set_ylim([-2.2,3.9])
                    else:
                        ax.set_ylim([-1.4,2.9])

                    ax.set_ylim([-5,5])

                    #~ ax.set_xticks([0.5 + x for x in range(len(props))])
                    ax.set_xticks([x for x in range(len(props))])
                    ax.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
                    ax.set_title("Difference between " + k + " (" + dic_labels[d1] + ") and " + l + " (" +  dic_labels[d2] + ")" )
                    ax.set_ylabel("Z-score")
                    ax.yaxis.grid(True)
                    ax.xaxis.grid(True)
                    fig.savefig(figname, bbox_inches = "tight")
                    plt.close()

                    fig2, ax2 = plt.subplots(figsize=(6.,3))
                    # Let's test the raw data..
                    figname=figures_dir+"/boxplot-raw-test_"+k + "_" + d1+"_vs_" + l + "_" + d2 + ".eps"
                    ax2.bar(range(len(rawdata)),rawdata, color=boxcolr)
                    #ax2.set_ylim([-0.03,0.03])
                    #~ ax2.set_xticks([0.5 + x for x in range(len(props))])
                    ax2.set_xticks([x for x in range(len(props))])
                    ax2.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
                    ax2.set_title("Difference between " + k + " (" + d1 + ") and " + l + " (" + d2 + ")" )
                    ax2.set_ylabel("Difference")
                    ax2.yaxis.grid()
                    ax2.xaxis.grid()
                    fig2.savefig(figname, bbox_inches = "tight")
                    plt.close()
                    
    props = ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    props += ["top-idp","iupred_long","iupred_short","length_translation"]
    #props = ["freq_" + aa for aa in aas_sorted_topidp]
    #datasets += uniprot_sets

    #for d in datasets:
    #    print(d,dfs[d])
    
    for k in kingdoms:
        for l in kingdoms:
            if (k < l):
                continue
            for d1 in datasets:
                for d2 in datasets:
                    if (d1==d2 and k==l):
                        continue
                    data=[]
                    rawdata=[]
                    fig, ax = plt.subplots(figsize=(6.,3))
                    for prop in props:
                        cat1 = dfs[d1].loc[(dfs[d1].kingdom == k) & (dfs[d1].taxon_id.isin(good_taxa))][prop]
                        cat2 = dfs[d2].loc[(dfs[d2].kingdom == l) & (dfs[d2].taxon_id.isin(good_taxa))][prop]

                        a, b = ttest_ind(cat1, cat2,equal_var=False)
                        c, d  = mannwhitneyu(cat1, cat2,alternative='two-sided')
                        mean1=np.mean(cat1)
                        mean2=np.mean(cat2)
                        sd1=np.sqrt(np.var(cat1))
                        sd2=np.sqrt(np.var(cat2))
                        Z=(mean1-mean2)/math.sqrt(sd1*sd1+sd2*sd2)
                        print ("T-test: ",prop,k,l,d1,d2,mean1,mean2,a,b,c,d)
                        print ("Z-score: ",prop,k,l,d2,d2,mean1,mean2,sd1,sd2,Z)
                        data += [Z]
                        if (prop in ["top-idp","iupred_long","iupred_short"]):
                            mean1=mean1/100
                            mean2=mean2/100
                        rawdata += [mean1-mean2]
                    figname=figures_dir+"/boxplot-Z-test_shift_"+k + "_" + d1 +"_vs_" + l + "_" + d2 + ".eps"
                    ax.bar(range(len(data)),data, color=boxcolr)
                    if k == l:
                        ax.set_ylim([-2.2,3.9])
                    else:
                        ax.set_ylim([-1.4,2.9])
                    #~ ax.set_xticks([0.5 + x for x in range(len(props))])
                    ax.set_xticks([x for x in range(len(props))])
                    ax.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
                    ax.set_title("Difference between " + k + " (" + dic_labels[d1] + ") and " + l + " (" +  dic_labels[d2] + ")" )
                    ax.set_ylabel("Z-score")
                    ax.yaxis.grid(True)
                    ax.xaxis.grid(True)
                    fig.savefig(figname, bbox_inches = "tight")
                    plt.close()

                    fig2, ax2 = plt.subplots(figsize=(6.,3))
                    # Let's test the raw data..
                    figname=figures_dir+"/boxplot-raw-test_shift_"+k + "_" + d1+"_vs_" + l + "_" + d2 + ".eps"
                    ax2.bar(range(len(rawdata)),rawdata, color=boxcolr)
                    #ax2.set_ylim([-0.03,0.03])
                    #~ ax2.set_xticks([0.5 + x for x in range(len(props))])
                    ax2.set_xticks([x for x in range(len(props))])
                    ax2.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
                    ax2.set_title("Difference between " + k + " (" + d1 + ") and " + l + " (" + d2 + ")" )
                    ax2.set_ylabel("Difference")
                    ax2.yaxis.grid()
                    ax2.xaxis.grid()
                    fig2.savefig(figname, bbox_inches = "tight")
                    plt.close()
                    
                   
    props = ["freq_" + aa + "-fifty" for aa in aas_sorted_topidp]
    props += ["top-idp","iupred_long","iupred_short","length_translation"]
    #props = ["freq_" + aa for aa in aas_sorted_topidp]
    #datasets += uniprot_sets

    #for d in datasets:
    #    print(d,dfs[d])
    
    for k in kingdoms:
        for l in kingdoms:
            if (k < l):
                continue
            for d1 in datasets:
                for d2 in datasets:
                    if (d1==d2 and k==l):
                        continue
                    data=[]
                    rawdata=[]
                    fig, ax = plt.subplots(figsize=(6.,3))
                    for prop in props:
                        cat1 = dfs[d1].loc[(dfs[d1].kingdom == k) & (dfs[d1].taxon_id.isin(good_taxa))][prop]
                        cat2 = dfs[d2].loc[(dfs[d2].kingdom == l) & (dfs[d2].taxon_id.isin(good_taxa))][prop]

                        a, b = ttest_ind(cat1, cat2,equal_var=False)
                        c, d  = mannwhitneyu(cat1, cat2,alternative='two-sided')
                        mean1=np.mean(cat1)
                        mean2=np.mean(cat2)
                        sd1=np.sqrt(np.var(cat1))
                        sd2=np.sqrt(np.var(cat2))
                        Z=(mean1-mean2)/math.sqrt(sd1*sd1+sd2*sd2)
                        print ("T-test: ",prop,k,l,d1,d2,mean1,mean2,a,b,c,d)
                        print ("Z-score: ",prop,k,l,d2,d2,mean1,mean2,sd1,sd2,Z)
                        data += [Z]
                        if (prop in ["top-idp","iupred_long","iupred_short"]):
                            mean1=mean1/100
                            mean2=mean2/100
                        rawdata += [mean1-mean2]
                    figname=figures_dir+"/boxplot-Z-test_fifty_"+k + "_" + d1 +"_vs_" + l + "_" + d2 + ".eps"
                    ax.bar(range(len(data)),data, color=boxcolr)
                    if k == l:
                        ax.set_ylim([-2.2,3.9])
                    else:
                        ax.set_ylim([-1.4,2.9])
                    #~ ax.set_xticks([0.5 + x for x in range(len(props))])
                    ax.set_xticks([x for x in range(len(props))])
                    ax.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
                    ax.set_title("Difference between " + k + " (" + dic_labels[d1] + ") and " + l + " (" +  dic_labels[d2] + ")" )
                    ax.set_ylabel("Z-score")
                    ax.yaxis.grid(True)
                    ax.xaxis.grid(True)
                    fig.savefig(figname, bbox_inches = "tight")
                    plt.close()

                    fig2, ax2 = plt.subplots(figsize=(6.,3))
                    # Let's test the raw data..
                    figname=figures_dir+"/boxplot-raw-test_fifty_"+k + "_" + d1+"_vs_" + l + "_" + d2 + ".eps"
                    ax2.bar(range(len(rawdata)),rawdata, color=boxcolr)
                    #ax2.set_ylim([-0.03,0.03])
                    #~ ax2.set_xticks([0.5 + x for x in range(len(props))])
                    ax2.set_xticks([x for x in range(len(props))])
                    ax2.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
                    ax2.set_title("Difference between " + k + " (" + d1 + ") and " + l + " (" + d2 + ")" )
                    ax2.set_ylabel("Difference")
                    ax2.yaxis.grid()
                    ax2.xaxis.grid()
                    fig2.savefig(figname, bbox_inches = "tight")
                    plt.close()
                    

    # Print latex table
    '''
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    for prop in props:
        string=dic_labels[prop]
        for kingdom in ["Eukaryota", "Bacteria"]:
            for ty in datasets:
                value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]
                length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                string += " & " + str(round(100*(np.mean(value))/(np.mean(length)),2)) + "\%" 
        string += "\\\\ "
        print (string)
    '''        
    


    props = ["freq_" + aa for aa in aas_sorted_topidp]
    props += ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    props += ["freq_" + aa + "-fifty" for aa in aas_sorted_topidp]
    for prop in props:
        string=dic_labels[prop]
        for kingdom in [ "Bacteria","Archaea","Eukaryota"]:
            for ty in datasets:
                length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]
                string += " & " + str(round(100*(np.mean(value)),1)) + "\%" 
        string += "\\\\"
        print (string)
            

    #************************************************************************

    ##########################################################################################################################
    # for each amino acid, and separately for eukarya and bacteria, plot its frequency versus the taxonomic GC% for 4 curves:
    # 1- average freq
    # 2- a linear fit of the data
    # 3- the frequency as it should be if only dependent on GC
    # 4- a model function (w * prob_tot + k) with estimated parameters W and K
    # note: for our big dataset (from NCBI) it is also possible to plot against the GC% of the CDS

    # chose which dataset to use  for the following plots



    
    #good_taxa = set(df.taxon_id.unique())
    #print ("TEST all:",df.taxon_id.unique())
    #print ("TEST good:",good_taxa)


    
    ######################################################

    # for a given dataset (ex. "dataset") plot two properties against each other


        
    dic_colors= {"Eukaryota":'#00B060', "Bacteria":'#E20048',"Archaea":'#5064d0'}
    for prop_1 in properties_x:
        for prop_2 in properties_y:
            #print ("TEST",prop_1,prop_2)
            fig, ax = plt.subplots(figsize=(6,6))
        
            # plot the values as a scatter plot
            prop_1_values = {}
            prop_2_values = {}
            for kingdom in ['Bacteria',"Archaea",'Eukaryota']:
                

                good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)
                
                temp_df = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa))][list(set([prop_1,prop_2,"iupred_short"]))].dropna()
                
                prop_1_values[kingdom] = temp_df[prop_1].values
                prop_2_values[kingdom] = temp_df[prop_2].values
                
                ax.scatter(prop_1_values[kingdom],prop_2_values[kingdom], edgecolor="none", facecolor=dic_colors[kingdom],alpha = 1,s=50,label=dic_labels[kingdom])
                #sct = ax.scatter(prop_1_values[kingdom],prop_2_values[kingdom],marker=dic_markers[kingdom], c=colors_list, edgecolor="none",alpha = 1,s=70,label=dic_labels[kingdom])
                
                # calculate and plot the fitting lines
                fit = np.polyfit(list(prop_1_values[kingdom]),list(prop_2_values[kingdom]),1)
                fit_fn = np.poly1d(fit) 
                plt.plot(prop_1_values[kingdom], fit_fn(prop_1_values[kingdom]), '--k',lw=2,color=dic_colors_fit[kingdom])

                #slope, intercept, r_value, p_value, std_err = linregress(prop_1_values, prop_2_values)
                #print(slope, intercept, r_value, p_value, std_err)
                

            # plot the random running average
            if prop_1 in df_genes_random.columns and prop_2 in df_genes_random.columns:
                prop_1_random = running_mean(pd.np.array(df_genes_random.sort_values(prop_1)[prop_1].astype(float)),500)
                prop_2_random = running_mean(pd.np.array(df_genes_random.sort_values(prop_1)[prop_2].astype(float)),500)
                ax.plot(prop_1_random,prop_2_random,color="black",label="Random")
                
            # plot the theoretical line
            if prop_2.split("_")[0] == "freq" and prop_1 in ["GC","GC3","GC_genomic"]:
                aa = prop_2.split("_")[1]
                p = []
                for gc in np.arange(0.1,0.9,0.001):
                    p += [get_aa_freq(aa, gc)]
                
                ax.plot(np.arange(10,90,0.1),p,color="blue",lw=2,label="Theoretical")


            # set the limits
            if prop_1 in dic_limits:
                ax.set_xlim(dic_limits[prop_1])
                
            if prop_2 in dic_limits:
                ax.set_ylim(dic_limits[prop_2])
                
                
            # set the labels
            ax.set_xlabel(dic_labels.get(prop_1,prop_1))
            ax.set_ylabel(dic_labels.get(prop_2,prop_2))
            
            # plot legend
            ax.legend(loc="upper right")
            
            # ranksums tests
            ranksum_string = ""
            ranksum_string += "Eukarya vs Bacteria = " + fmt_scientific(stats.ranksums(prop_2_values["Eukaryota"], prop_2_values["Bacteria"])[1]) + "\n"
            ranksum_string += "Eukarya vs Archaea = " + fmt_scientific(stats.ranksums(prop_2_values["Eukaryota"], prop_2_values["Archaea"])[1]) + "\n"
            ranksum_string += "Bacteria vs Archaea = " + fmt_scientific(stats.ranksums(prop_2_values["Bacteria"], prop_2_values["Archaea"])[1]) + "\n" + "\n"
            
            # Pearson correlation coefficient
            ranksum_string += "Eukarya R = " + str(round(stats.pearsonr(prop_1_values["Eukaryota"], prop_2_values["Eukaryota"])[0],2)) + "\n"
            ranksum_string += "Bacteria R = " + str(round(stats.pearsonr(prop_1_values["Bacteria"], prop_2_values["Bacteria"])[0],2)) + "\n"
            ranksum_string += "Archaea R = " + str(round(stats.pearsonr(prop_1_values["Archaea"], prop_2_values["Archaea"])[0],2))
            
            
            # calculate ABS difference between Euk-theoretical and Prok-theoretical
            if prop_2.split("_")[0] == "freq" and prop_1 in ["GC","GC3","GC_genomic"]:
                
                diff_from_theo = {}
                
                aa = prop_2.split("_")[1]
                for kingdom in ['Bacteria',"Archaea",'Eukaryota']:
                    p = []
                    for vx,vy in zip (prop_1_values[kingdom], prop_2_values[kingdom]):
                        p += [abs(vy-get_aa_freq(aa, vx/100.0))]
                        
                    ranksum_string += "abs(" + kingdom + " - theoretical) = " + str(round(np.mean(p),4))+ "\n"
                    
                    
            bb = ax.text(ax.get_xlim()[0], ax.get_ylim()[1], ranksum_string, fontsize=12,
                ha="left", va="top",
                bbox=dict(boxstyle="square,pad=0",
                        ec=(0.0, 0.0, 0.0,1),
                        fc=(1., 1, 1,1),),    
                    )
            
        
            
            figname = figures_dir + prop_2 + "_vs_" + prop_1 + "_pfam.eps"
            print(figname)
            fig.savefig(figname)
            #fig.show()
            
            ##########################################################################################################################
            
            
            
            
        ##########################################################################################################################
        
    # plot a series of properties as boxplots, with each box corresplonding to a dataset-kingdom combination
    
    # the properties to plot
    props = ["iupred_long", "iupred_short", "top-idp","seg","length_translation",'numdiso_long','numdiso_short','numseg','GC_genomic']
    props += ["freq_A","freq_C","freq_D","freq_E","freq_F","freq_G","freq_H","freq_I","freq_K",
                  "freq_L","freq_M","freq_N", "freq_P","freq_Q","freq_R","freq_S", "freq_T",
                  "freq_V","freq_W","freq_Y"]

    props += ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    props += ["freq_" + aa + "-fifty" for aa in aas_sorted_topidp]


    dic_boxplot_limits = {"iupred_long":[0,50], "iupred_short":[0,50], 
                      "top-idp":[0,0.15],'length_translation':[0,800],
                      'hessa':[0.85,1.2], 'seg':[0,25],"scampi":[0.02,0.1],
                    "freq_S":[0.03,0.1],"freq_P":[0.01,0.08], 
                      "freq_I":[0.02,0.11],"freq_T":[0.03,0.08],
                              "freq_A":[0.00,0.18],
                              "freq_C":[0.0,0.06],
                              "freq_D":[0.02,0.1],
                              "freq_E":[0.01,0.12],
                              "freq_F":[0.00,0.1],
                              "freq_G":[0.03,0.15],
                              "freq_H":[0.00,0.06],
                              "freq_K":[0.00,0.12],
                              "freq_L":[0.05,0.15],
                              "freq_M":[0.01,0.05],
                              "freq_N":[0.00,0.1],
                              "freq_Q":[0.01,0.1],
                              "freq_R":[0.01,0.1],
                              "freq_V":[0.03,0.12],
                              "freq_W":[0.0,0.04],
                              "freq_Y":[0.00,0.07],
                              }

    




    for prop in props:
    
        all_data = []
        labels = []
        for d in datasets:
            for k in kingdoms:

                data = dfs[d].loc[dfs[d].kingdom == k][prop].dropna()  
                length = dfs[d].loc[dfs[d].kingdom == k]["length_translation"].dropna()  
                all_data += [data]
                
                label = k + "_" + d
                labels += [label]
                
        fig, ax = plt.subplots(figsize=(4,3))
        
        plt.xticks(rotation=90)
        bp = ax.boxplot(all_data,labels=labels,patch_artist=True)
        
        
        for i,box in enumerate(bp['boxes']):
            if i%2 == 0:
                box.set(color='#93002F', linewidth=2)
                box.set(facecolor = '#E20048' )
            else:
                box.set(color='#00733E', linewidth=2)
                box.set(facecolor = '#00B060' )

        #~ pylab.setp(bp['boxes'],color='black',lw=2)
        plt.setp(bp['whiskers'], color='black')
        plt.setp(bp['medians'], color='black')
        plt.setp(bp['fliers'], color='black', marker='+')
        
        ax.set_title(dic_labels[prop])
        #ax.set_ylim(dic_boxplot_limits[prop])
        
        
        figname = figures_dir + "boxplot_" + prop + ".eps"
        print(figname)
        fig.savefig(figname,bbox_inches = "tight")
        #fig.show()

 

    
    # plot a series of properties as boxplots, with each box corresplonding to phylum
    
    # the properties to plot props = ["iupred_long", "iupred_short", 
    #"top-idp",'length_translation', 'hessa',
    #'seg','GC_genomic','numdiso_long','numdiso_short','numseg']
    
    #props = ["freq_A","freq_C","freq_D","freq_E","freq_F","freq_G","freq_H","freq_I","freq_K",
    #              "freq_L","freq_M","freq_N", "freq_P","freq_Q","freq_R","freq_S", "freq_T",
    #              "freq_V","freq_W","freq_Y"]

    #props += ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]


    phyla_euk = dfs["All"].loc[dfs["All"].kingdom == "Eukaryota"]["phylum"].unique()
    phyla_proks = dfs["All"].loc[dfs["All"].kingdom == "Bacteria"]["phylum"].unique()
    phyla_arch = dfs["All"].loc[dfs["All"].kingdom == "Archaea"]["phylum"].unique()
    all_phyla = list(phyla_proks)  + list(phyla_arch)  + list(phyla_euk)
    #all_phyla = list(phyla_proks) + list(phyla_euk) 
    
    
    for d in datasets:
    
        for prop in props:    
            data = []
            labels = []
            for phylum in all_phyla:
                dd = dfs[d].loc[dfs[d].phylum == phylum][prop].dropna()
                if len(dd) > 10:
                    length = dfs[d].loc[dfs[d].phylum == phylum]["length_translation"].dropna()
                    data += [dd]
                    labels += [phylum]
        
            fig, ax = plt.subplots(figsize=(6,3))
            ax.set_ylabel('Fraction')
            plt.xticks(rotation=90,fontsize=6)
            bp = ax.boxplot(data,labels=labels,patch_artist=True)
        
            dic_colors= {"Eukaryota":'#00B060', "Bacteria":'#E20048',"Archaea":'#5064d0'}
        
      
            for l,box in zip(labels,bp['boxes']):
                if l in set(phyla_euk):
                    box.set(color='#00B060', linewidth=2)
                    box.set(facecolor = '#00B060' )
                elif l in set(phyla_proks):
                    box.set(color='#E20048', linewidth=2)
                    box.set(facecolor = '#E20048' )
                elif l in set(phyla_arch):
                    box.set(color='#5064d0', linewidth=2)
                    box.set(facecolor = '#5064d0' )
                else:
                    box.set(color='orange', linewidth=2)
                    box.set(facecolor = 'orange' )

            #~ pylab.setp(bp['boxes'],color='black',lw=2)
            plt.setp(bp['whiskers'], color='black')
            plt.setp(bp['medians'], color='black')
            plt.setp(bp['fliers'], color='black', marker='+')
        
    
            ax.set_title(dic_labels[prop])

            '''
            if "shift" in prop:
                ylim=0.05
                ax.set_ylim([-ylim, ylim])
            elif "freq" in prop:
                ylim=0.12
                ax.set_ylim([0, ylim])
            elif "fifty" in prop:
                ylim=0.12
                ax.set_ylim([0, ylim])
            '''
            figname = figures_dir + "boxplots_by_phylum_" + d + "_" + prop + ".eps"
            ax.yaxis.grid()
            ax.grid(color='grey', linestyle='-', linewidth=1)
            ax.xaxis.grid(False)
            print(figname)
            fig.savefig(figname,bbox_inches = "tight")
            #fig.show()
            plt.close()

    



    # Print latex table
    propsA = ["freq_" + aa for aa in aas_sorted_topidp]
    propsB = ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    propsC = ["freq_" + aa + "-fifty" for aa in aas_sorted_topidp]

    for kingdom in ["Bacteria", "Archaea","Eukaryota"]:
        string =kingdom
        for ty in datasets2:
            string += " & " + ty 
        string += "\\\\ "
        print (string)
        for prop in propsA:
            string=dic_labels[prop]
            for ty in datasets:
                value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop].mean()
                length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"].mean()
                string += " & " + str(round(100*(value),2)) + "\\%" 
            string += "\\\\ "
            print (string)
        string =kingdom
        for ty in datasets:
            string += " & " + ty 
        string += "\\\\ "
        print (string)
        for prop in propsB:
            string=dic_labels[prop]
            for ty in datasets2:
                value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop].mean()
                length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"].mean()
                string += " & " + str(round(100*(value),2)) + "\\%" 
            string += "\\ "
            print (string)
            
