#!/usr/bin/env python
import matplotlib
matplotlib.use('Agg')
import pandas as pd
import numpy as np
import os
import re
import seaborn as sns
import matplotlib.pyplot as plt

aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
               'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
               'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}

dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)", "top-idp":"TOP-IDP", "topidp-S":"TOP-IDP - S","topidp-SKP":"TOP-IDP - SKP","topidp-SP":"TOP-IDP - SP",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'GC% (third nucleotide)','taxon_gc':'GC% (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
    'random':'Expected','iupred_long':'IUpred long (%AA)','length_aa':'<Length> (AA)','gc':'GC%',
    'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)', 'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM (%AA)',"volatility":"<Volatility>", 'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity> (Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with  expected', 'aa_corr_dot':'AA freq\ncorrelation with   expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A':
    'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P':'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'ALA', 'freq_C':
    'CYS', 'freq_E': 'GLU', 'freq_D': 'ASP', 'freq_G': 'GLY',
    'freq_F': 'PHE', 'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K':
    'LYS', 'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
    'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S': 'SER', 'freq_R':
    'ARG', 'freq_T': 'THR', 'freq_W': 'TRP', 'freq_V': 'VAL',
    'freq_Y': 'TYR', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Viruses':'Viruses'}
        

sns.set()
sns.set(font_scale=1.4)

pfam_list = []
if os.path.isdir("../data/substmat/"):
    input_dir="../data/substmat/"
else:
    input_dir="../data/"
data_dir="../data/"
figures_dir="../figures/"

for f in os.listdir(input_dir):
    if f.endswith("-num_ee.npy"):
        mtemp=re.split("[-]",f)
        pfam_list+=[mtemp[0]]

        
subst_bb=np.zeros((20, 20))
subst_ee=np.zeros((20, 20))
subst_be=np.zeros((20, 20))
num_e=np.zeros(20)
num_b=np.zeros(20)
num_ee=np.zeros(20)
num_bb=np.zeros(20)

for p in pfam_list:
    temp_subst_ee=np.load(input_dir+p+"-subst_ee.npy")
    temp_subst_bb=np.load(input_dir+p+"-subst_bb.npy")
    temp_subst_be=np.load(input_dir+p+"-subst_be.npy")
    temp_num_ee=np.load(input_dir+p+"-num_ee.npy")
    temp_num_bb=np.load(input_dir+p+"-num_bb.npy")
    temp_num_b=np.load(input_dir+p+"-num_b.npy")
    temp_num_e=np.load(input_dir+p+"-num_e.npy")

    subst_ee+=temp_subst_ee
    subst_bb+=temp_subst_bb
    subst_be+=temp_subst_be
    num_ee+=temp_num_ee
    num_bb+=temp_num_bb
    num_e+=temp_num_e
    num_b+=temp_num_b

if os.path.isdir(data_dir+"/substmat/"):
    np.savetxt(data_dir+"all-subst_bb.csv",subst_bb)
    np.savetxt(data_dir+"all-subst_be.csv",subst_be)
    np.savetxt(data_dir+"all-subst_ee.csv",subst_ee)
    
    np.save(data_dir+"all-subst_bb.npy",subst_bb)
    np.save(data_dir+"all-subst_be.npy",subst_be)
    np.save(data_dir+"all-subst_ee.npy",subst_ee)
    
    np.save(data_dir+"all-num_bb.npy",num_bb)
    np.save(data_dir+"all-num_ee.npy",num_ee)
    np.save(data_dir+"all-num_e.npy",num_e)
    np.save(data_dir+"all-num_b.npy",num_b)

# logodds  p(i|j)/(p(i)*p(j))
#  ((num(i)/sum(x))num(i|j)/sum(xy)/((num(i)/sum(x))*(num(j)/sum(x)))

sum_subst_bb=np.sum(subst_bb)
sum_subst_ee=np.sum(subst_ee)
sum_subst_be=np.sum(subst_be)
sum_num_bb=np.sum(num_bb)
sum_num_ee=np.sum(num_ee)
sum_num_e=np.sum(num_e)
sum_num_b=np.sum(num_b)
sum_num_be=(sum_num_e+sum_num_b)/2

#print ("BB",num_bb,subst_bb,sum_subst_bb,sum_num_bb)
#print ("EE",num_ee,subst_ee,sum_subst_ee,sum_num_ee)
#print ("BE",num_be,subst_be,sum_subst_be,sum_num_be)


#nummat_bb=np.matmul(num_bb,np.transpose(num_bb))
#nummat_ee=np.matmul(num_ee,num_ee)
#nummat_be=np.matmul(num_be,num_be)
num_bb=num_bb.reshape(20,1)
num_ee=num_ee.reshape(20,1)
num_b=num_b.reshape(20,1)
num_e=num_e.reshape(20,1)

nummat_bb=num_bb*num_bb.T
nummat_ee=num_ee*num_ee.T
nummat_be=num_e*num_b.T

freq_num_bb=np.divide(num_bb,sum_num_bb)
freq_num_ee=np.divide(num_ee,sum_num_ee)
freq_num_b=np.divide(num_b,sum_num_be)
freq_num_e=np.divide(num_e,sum_num_be)


freq_subst_bb=np.divide(subst_bb,sum_subst_bb) #*freq_num_bb.T
freq_subst_ee=np.divide(subst_ee,sum_subst_ee) #*freq_num_ee.T
freq_subst_be=np.divide(subst_be,sum_subst_be) #*freq_num_be.T


exp_subst_bb=np.divide(nummat_bb,sum_num_bb*sum_num_bb)
exp_subst_ee=np.divide(nummat_ee,sum_num_ee*sum_num_ee)
exp_subst_be=np.divide(nummat_be,sum_num_be*sum_num_be)


odds_bb=np.divide(freq_subst_bb,exp_subst_bb)
odds_ee=np.divide(freq_subst_ee,exp_subst_ee)
odds_be=np.divide(freq_subst_be,exp_subst_be)

logodds_bb=np.log(odds_bb)
logodds_ee=np.log(odds_ee)
logodds_be=np.log(odds_be)


#print ("test1",num_bb[1],subst_bb[1,1],sum_subst_bb,sum_num_bb)
#print ("test2",nummat_bb[1,1],freq_num_bb[1])
#print ("test3",freq_subst_bb[1,1],exp_subst_bb[1,1],odds_bb[1,1],logodds_bb[1,1])

#print ("BB",logodds_bb)
#print ("EE",logodds_ee)
#print ("BE",logodds_be)

be_bb=logodds_be-logodds_bb
be_ee=logodds_be-logodds_ee
bb_ee=logodds_bb-logodds_ee

beT=logodds_be-logodds_be.T
bbT=logodds_bb-logodds_bb.T
eeT=logodds_ee-logodds_ee.T

#print ("BE-BB",be_bb)
#print ("BE-EE",be_ee)

bb = pd.DataFrame(logodds_bb,columns=aas,index=aas)
ee = pd.DataFrame(logodds_ee,columns=aas,index=aas)
be = pd.DataFrame(logodds_be,columns=aas,index=aas)
be_bb = pd.DataFrame(be_bb,columns=aas,index=aas)
be_ee = pd.DataFrame(be_ee,columns=aas,index=aas)
bb_ee = pd.DataFrame(bb_ee,columns=aas,index=aas)

bbT = pd.DataFrame(bbT,columns=aas,index=aas)
eeT = pd.DataFrame(eeT,columns=aas,index=aas)
beT = pd.DataFrame(beT,columns=aas,index=aas)



#print(be.to_latex())

# Do plots
for i in range(0,20):
    b2e=freq_subst_be[i,:]
    e2b=freq_subst_be[:,i]
    e2b=e2b/np.sum(e2b)
    b2e=b2e/np.sum(b2e)
    #e2b=e2b/e2b[i]
    #b2e=b2e/b2e[i]
    aa=aas[i]
    fig, ax = plt.subplots()
    index = np.arange(20)
    bar_width = 0.35
    
    opacity = 0.4
    #error_config = {'ecolor': '0.3'}

    rects1 = plt.bar(index, e2b, bar_width,
                 alpha=opacity,
                 color='red',
                 label='Euk2Bac',edgecolor='black')

    rects2 = plt.bar(index + bar_width, b2e , bar_width,
                 alpha=opacity,
                 color='green',
                 label='Bac2Euk',edgecolor='black')
    ax.set_facecolor('white')
    plt.xlabel('Amino Acid')
    plt.ylabel('Frequency')
    plt.title('Frequency of substitutions for '+dic_labels[aa])
    plt.xticks(index + bar_width / 2, aas)
    plt.legend()
    ax.yaxis.grid()
    ax.grid(color='grey', linestyle='-', linewidth=1)
    ax.xaxis.grid(False)
    plt.tight_layout()
    #plt.show()
    plt.savefig(figures_dir + "subst-be-barplot_"+aa+".eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close()



    
fig, ax = plt.subplots()
fig.set_size_inches(18,15)


sns.heatmap(ee,cmap="bwr",vmin=-5,vmax=+5) # ,method="single",metric='correlation')annot=True,
ax.set_title("LogOdds matrix for Euk-Euk substitutions")
plt.savefig(figures_dir + "subst-ee-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(bb,cmap="bwr",vmin=-5,vmax=+5) # ,method="single",metric='correlation')annot=True,
ax.set_title("LogOdds matrix for Bac-Bac substitutions")
plt.savefig(figures_dir + "subst-bb-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(be,cmap="bwr",vmin=-5,vmax=+5) # ,method="single",metric='correlation')annot=True,
ax.set_title("LogOdds matrix for Euk-Bac substitutions")
plt.savefig(figures_dir + "subst-be-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(be_bb,cmap="bwr",vmin=-1,vmax=+1) # ,method="single",metric='correlation')annot=True,
ax.set_title("Difference in LogOdds matrix Euk-Bac vs Bac-Bac")
plt.savefig(figures_dir + "subst-be_bb-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(be_ee,cmap="bwr",vmin=-1,vmax=+1) # ,method="single",metric='correlation')annot=True,
ax.set_title("Difference in LogOdds matrix Euk-Bac vs Euk-Euk")
plt.savefig(figures_dir + "subst-be_ee-heatmap.eps",bbox_inches="tight")

# Transformed matrix

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(beT,cmap="bwr",vmin=-1,vmax=+1) # ,method="single",metric='correlation')annot=True,
ax.set_title("Difference in LogOdds matrix Euk-Bac vs Bac-Euk")
plt.savefig(figures_dir + "subst-be-eb-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.clustermap(beT,cmap="bwr",vmin=-1,vmax=+1).fig.suptitle('Difference in LogOdds matrix Euk-Bac vs Bac-Euk')
ax.set_xlabel("Bacteria")
ax.set_ylabel("Euaryota")
plt.savefig(figures_dir + "subst-be-eb-clustermap.eps",bbox_inches="tight")

for m in ["single", "complete", "average", "weighted", "centroid", "median", "ward"]:
    #    for n in ["euclidian", "minkowski", "cityblock", "correlation","hamming","jaccard","chebyshev"]:
    fig, ax = plt.subplots()
    fig.set_size_inches(18,15)
    sns.clustermap(beT,cmap="bwr",vmin=-1,vmax=+1,method=m).fig.suptitle('Difference in LogOdds matrix Euk-Bac vs Bac-Euk')
    ax.set_xlabel("Bacteria")
    ax.set_ylabel("Euaryota")
    plt.savefig(figures_dir + "subst-be-eb-"+m+"-clustermap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(bb_ee,cmap="bwr",vmin=-1,vmax=+1) # ,method="single",metric='correlation')annot=True,
ax.set_title("Difference in LogOdds matrix Bac-Bac vs Euk-Euk")
plt.savefig(figures_dir + "subst-bb-ee-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.clustermap(bb_ee,cmap="bwr",vmin=-1,vmax=+1).fig.suptitle('Difference in LogOdds matrix Bac-Bac vs Euk-Euk')
ax.set_xlabel("Bacteria")
ax.set_ylabel("Euaryota")
plt.savefig(figures_dir + "subst-bb-ee-clustermap.eps",bbox_inches="tight")


# Some more detailed analysis (of raw data)
fig, ax = plt.subplots()
fig.set_size_inches(18,15)
raw_be = pd.DataFrame(freq_subst_be,columns=aas,index=aas)
sns.heatmap(raw_be,cmap="Purples") # ,method="single",metric='correlation')annot=True,
ax.set_title("Subsitutions matrix for Bac-Euk substitutions")
plt.savefig(figures_dir + "subst-raw-be-heatmap.eps",bbox_inches="tight")
plt.close('all')
