#!/usr/bin/env python3
import pandas as pd
import os
from Bio.Seq import Seq
from Bio import SeqIO
from Bio import AlignIO
import Bio.Align
import re
from Bio.SeqUtils.ProtParam import ProteinAnalysis
import math
from Bio.PDB import *
import os.path
from Bio import motifs
import sys
from random import *
import numpy

class PDBSelect(Select):
    def accept_residue(self, residue):
        if residue.get_name()=='GLY':
            return True
        else:
            return False

def count_subst(a,b):
    for i in len(a):
        i=j
    return i

def get_phylum(s):
    try:
        return s.split(";")[1]
    except:
        return None    

def get_kingdom(s):
    try:
        return s.split(";")[0]
    except:
        return None    

if __name__ == '__main__':
    aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
    aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
                   'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
                   'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}

    data_dir = "../../data/"
    pdb_dir = data_dir
    results_dir = data_dir
    input_dir = data_dir+"/pfam/"
    output_dir = data_dir+"/substmat/"
    cifparser = MMCIFParser()
    pdbparser = PDBParser()
    
    # GC content is in prokaryotes.txt and eukaryotes.txt
    # FROM ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/
    # Reading in GC (and actually taxonomy) information
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    genomes_viruses_file = data_dir + "viruses.txt"

    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_virus = pd.read_csv(genomes_viruses_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks,df_virus])

    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]
    #print (taxid2gc)
    #print (taxid2gc[999810])

    #taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    #group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    #taxid2kingdom = {}

    #for t in taxid2group:
    #    gr = taxid2group[t]
    #    taxid2kingdom[t] = group2kingdom[gr]
    
    #taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    #name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']
    # End of reading in GC



    # This takes too long
    print ("reading taxonomy tab")
    df_taxonomy = pd.read_csv(data_dir + "taxonomy_subset.tab", sep="\t")
    print ("Set taxon index")
    memo2taxid = df_taxonomy.set_index("Mnemonic").to_dict()["Taxon"]

    highcutoff=60.
    lowcutoff=20.

    codes={"V":"Viruses","E":"Eukaryota","B":"Bacteria","A":"Archaea","O":"Other","X":"Unknown"}
    taxid2kingdom={}
    memo2taxid={}
    with open(data_dir+'speclist.txt') as fp:
        for line in fp:
            if re.match('.*:\sN=',line):
                s=line.split()
                taxid=s[2].replace(':','')
                mnemonic=s[0]
                kingdom=codes[s[1]]
                memo2taxid[mnemonic]=taxid
                taxid2kingdom[taxid]=kingdom
                #print (line, taxid,mnemonic,kingdom)
    file_list = []
    for f in os.listdir(input_dir):
        if f.endswith(".sto"):
            file_list += [input_dir + f]
    #for f in sys.argv[1:]:
    #    file_list += [f]
    cutoff=10
    alignment={}
    subst_hh=numpy.zeros((20, 20))
    subst_ll=numpy.zeros((20, 20))
    subst_hl=numpy.zeros((20, 20))
    num_h=numpy.zeros(20)
    num_l=numpy.zeros(20)
    num_hh=numpy.zeros(20)
    num_ll=numpy.zeros(20)
    #num_hl=numpy.zeros(20)
    maxseq=1000
    for f in file_list:
        #print (f)
        #high=MultipleSeqAlignment()
        #low=MultipleSeqAlignment()
        high=Bio.Align.MultipleSeqAlignment([])
        low=Bio.Align.MultipleSeqAlignment([])
        s=re.search('PF\d\d\d\d\d',f)
        pfam=s.group(0)
        if (os.path.isfile(output_dir+pfam+"-subst_hl.npy")):
            continue
        print ("Reading alignments",pfam)
        alignment = AlignIO.read(f, "stockholm")
        high_seq=[]
        low_seq=[]
        print ("Classifying alignments",pfam)
        numhigh=0
        numlow=0
        j=0
        for record in alignment:
            #print (j,numhigh,numlow)
            j+=1
            mtemp=re.split("[_]",record.name)
            mnemonic=mtemp[1]
            sprot=record.name
            tax_id=memo2taxid.get(mnemonic,pd.np.nan)
            #phylum = taxid2phylum.get(tax_id,pd.np.nan)
            kingdom = taxid2kingdom.get(tax_id,pd.np.nan)
            try:
                GC = float(taxid2gc[int(tax_id)])
            except:
                continue
            #length=len(record.seq)
            if (kingdom == 'Bacteria'):
                if (GC > highcutoff and numhigh<=maxseq):
                    high.append(record)
                    high_seq+=[record.seq]
                    numhigh+=1
                elif (GC < lowcutoff and numlow<=maxseq):
                    low.append(record)
                    low_seq+=[record.seq]
                    numlow+=1
            if ((numhigh>maxseq and numlow>maxseq) or j>10000):
                break
        #AlignIO.write(high, input_dir+pfam+"_high.fa", "fasta")
        #AlignIO.write(low, input_dir+pfam+"_low.fa", "fasta")

        # Select 100 random pair of each group (high-high, low-low and high-low)
        print ("Read sequences",numhigh,numlow)
        if (numlow<10 or numhigh<10  ):
            continue
        if ( float(numlow/numhigh)<1.e-3 or float(numhigh/numlow)<1.e-3  ):
            continue
        print ("starting randomization")
        for i in range(1,100):
            highlen=len(high)
            lowlen=len(low)
            # High-high
            x=randint(0,highlen-1)
            y=randint(0,highlen-1)
            A=high[x].seq
            B=high[y].seq
            A=A.upper()
            B=B.upper()
            #print (x,y,A,B)
            #sys.exit()
            for j in range(0,len(A)):
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', A[j]) and re.match('[ACDEFGHIKLMNPQRSTVWY]', B[j])):
                    num_hh[aaindex[A[j]]]+=1
                    num_hh[aaindex[B[j]]]+=1
                    subst_hh[aaindex[A[j]],aaindex[B[j]]]+=1
                    subst_hh[aaindex[B[j]],aaindex[A[j]]]+=1
            # Low-Low
            x=randint(0,lowlen-1)
            y=randint(0,lowlen-1)
            A=low[x].seq
            B=low[y].seq
            #print (A,B)
            A=A.upper()
            B=B.upper()
            #sys.exit()
            for j in range(0,len(A)):
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', A[j]) and re.match('[ACDEFGHIKLMNPQRSTVWY]', B[j])):
                    num_ll[aaindex[A[j]]]+=1
                    num_ll[aaindex[B[j]]]+=1
                    subst_ll[aaindex[A[j]],aaindex[B[j]]]+=1
                    subst_ll[aaindex[B[j]],aaindex[A[j]]]+=1

            # High-low
            x=randint(0,highlen-1)
            y=randint(0,lowlen-1)
            A=high[x].seq
            B=low[y].seq
            #print (A,B)
            A=A.upper()
            B=B.upper()
            #sys.exit()
            for j in  range(0,len(A)):
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', A[j]) and re.match('[ACDEFGHIKLMNPQRSTVWY]', B[j])):
                    num_h[aaindex[A[j]]]+=1
                    num_l[aaindex[B[j]]]+=1
                    subst_hl[aaindex[A[j]],aaindex[B[j]]]+=1

        #print ("HH",num_hh,subst_hh)
        #print ("LL",num_ll,subst_ll)
        #print ("HL",num_hl,subst_hl)

        numpy.save(output_dir+pfam+"-num_l.npy",num_l)
        numpy.save(output_dir+pfam+"-num_h.npy",num_h)
        numpy.save(output_dir+pfam+"-num_ll.npy",num_ll)
        numpy.save(output_dir+pfam+"-num_hh.npy",num_hh)
        numpy.save(output_dir+pfam+"-subst_hh.npy",subst_hh)
        numpy.save(output_dir+pfam+"-subst_ll.npy",subst_ll)
        numpy.save(output_dir+pfam+"-subst_hl.npy",subst_hl)
