#!/usr/bin/env python3
from __future__ import print_function

def get_aa_freq(aa,gc):
    codons = aa2codons[aa]
    if (float(gc) > 1.0):
        gc=float(gc)/100
    prob_tot = []
    for codon in codons:
        prob_codon = []
        for N in codon:
            prob = float(gc)/2
            if N in {"A","T","U"}:
                prob = 0.5-(float(gc)/2)
            prob_codon += [prob]
        #print(codon, prob_codon,functools.reduce(mul,prob_codon))
        prob_tot += [functools.reduce(mul,prob_codon)] # Use functools.reduce() if you really need it; however, 99 percent of the time an explicit for loop is more readable
    prob_tot = sum(prob_tot)
    return prob_tot


def get_gc_freq(aa):
    codons = aa2codons[aa]
    prob_tot = []
    a=0.
    g=0.
    for codon in codons:
        for N in codon:
            if N in {"A","T","U"}:
                a+=1.
            else:
                g+=1.
    gc=g/(g+a)
    return gc

def logtransform(aa,gc,freq):
    if math.isnan(float(gc)):
        gc=50.
    aafreq=get_aa_freq(aa,gc)
    logfreq=np.log(freq/aafreq)
    return (logfreq)

##########################################################################################################################

# for each amino acid, and separately for eukarya and bacteria, plot its frequency versus GC% for 4 curves:
# 1- average freq
# 2- a linear fit of the data
# 3- the frequency as it should be if only dependent on GC
# 4- a model function (w * prob_tot + k) with estimated parameters W and K
# note: for our big dataset (from NCBI) it is also possible to plot against the GC% of the CDS

# Now we try to formulate this as a single function that can be used in all cases,


# We do not include stopcodons here
# w and k are the parameters to optimize
def my_func_old(x,w,k):
    global aminoacid,ydata_mean
    codons = aa2codons[aminoacid]
    stopcodons = aa2codons['_']
    res = []
    mean=ydata_mean
    for gc in x:
        stop_tot = 0.
        for codon in stopcodons:
            prob_codon = []
            for N in codon:
                #print (gc)
                prob = float(gc)/2
                if N in {"A","T","U"}:
                    prob = 0.5-(float(gc)/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            stop_tot += functools.reduce(mul,prob_codon)
        prob_tot = 0.
        for codon in codons:
            prob_codon = []
            for N in codon:
                #print (gc)
                prob = float(gc)/2
                if N in {"A","T","U"}:
                    prob = 0.5-(float(gc)/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            prob_tot += functools.reduce(mul,prob_codon)
        #prob_tot = sum(prob_tot)
        prob_tot=prob_tot*(1/(1-stop_tot))  # To compensate for stop codons.
        res += [w * prob_tot + k]
    return res



def my_func(x,a,b):
    global aminoacid,ydata_mean
    codons = aa2codons[aminoacid]
    stopcodons = aa2codons['_']
    res = []
    mean=ydata_mean
    prob_avg = len(codons)/61.
    for gc in x:
        stop_tot = 0.
        for codon in stopcodons:
            prob_codon = []
            for N in codon:
                #print (gc)
                prob = float(gc)/2
                if N in {"A","T","U"}:
                    prob = 0.5-(float(gc)/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            stop_tot += functools.reduce(mul,prob_codon)
        prob_tot = 0.
        for codon in codons:
            prob_codon = []
            for N in codon:
                prob = float(gc)/2
                if N in {"A","T","U"}:
                    prob = 0.5-(float(gc)/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            prob_tot += functools.reduce(mul,prob_codon)
        #prob_tot = sum(prob_tot)
        prob_tot=prob_tot*(1/(1-stop_tot))  # To compensate for stop codons.
        res += [a * (prob_tot - prob_avg) + prob_avg + b]  # This just means  [w * prob_tot + k],where a=w and k=prob_avg(1-a)+b
        # res += [w * prob_tot + k]

    return res


# This is to replace a linear function, but with gc=0.5 as the zero point
def my_func_lin(x,w,k):
    global aminoacid,ydata_mean
    codons = aa2codons[aminoacid]
    prob_avg = len(codons)/61.
    res=[]
    for gc in x:
        res += [w + prob_avg + k*(gc-0.5)]
    return res


def my_func_poly(x,w,k,l):
    res=[]
    for gc in x:
        res += [w + k*(gc-0.5)+l*(gc-0.5)*(gc-0.5)]
    return res


def get_error(obs,exp):
    err = []
    err2 = []
    N=0.
    for o,e in zip(obs,exp):
        N+=1.
        err += [abs(o - e)]
    return np.mean(err),np.std(err)

def get_abserror(obs,exp):
    err = []
    err2 = []
    N=0.
    for o,e in zip(obs,exp):
        N+=1.
        err += [(o - e)]
    return np.mean(err),np.std(err)


def T_test(obs,expA,expB):
    from scipy import stats
    errA = []
    errB = []
    for o,e in zip(obs,expA):
        errA += [abs(o - e)]
    for o,e in zip(obs,expB):
        errB += [abs(o - e)]
    stat,pval=stats.ttest_ind(errA,errB, equal_var = False)
    return stat,pval


def clamp(value, min_v, max_v):
    return max(min(value, max_v), min_v)

##########################################################################################################################

# define a function to plot a bar plot of the difference between AA frequencies in two datasets
# d1, d2 = the two dataset (could be the same dataset); datasets are "dataset","dataset-eggnog","dataset-pfam","dataset-pfam-full"
# k1, k2 = the kingdoms: can be either "Eukaryota" or "Bacteria"


def bar_plot_s1_vs_s2(d1,d2,k1,k2,uselog = True):
    fig, ax = plt.subplots(figsize=(6.,3))
    data_aa = []
    for aa in aas_sorted_topidp:
        prop = "freq_" + aa
        prop_values = {}
        prop_values_1 = dfs[d1].loc[dfs[d1].kingdom == k1.encode("utf-8")][prop].mean()
        prop_values_2 = dfs[d2].loc[dfs[d2].kingdom == k2.encode("utf-8")][prop].mean()
        length_1 = dfs[d1].loc[dfs[d1].kingdom == k1.encode("utf-8")]["length"].mean()
        length_2 = dfs[d2].loc[dfs[d2].kingdom == k2.encode("utf-8")]["length"].mean()
        if uselog == True:
            value = np.log((prop_values_1/length_1)/(prop_values_2/length_2))
        else:
            value = prop_values_1/length_1-prop_values_2/length_2
        data_aa += [value]
    ax.bar(range(len(data_aa)),data_aa, color="red")
    ylim = max([abs(v) for v in ax.get_ylim()])
    ylim = 0.03
    ax.set_ylim([-ylim, ylim])
    ax.set_xticks([0.0 + x for x in range(len(aas_sorted_topidp))])
    ax.set_xticklabels(aas_sorted_topidp)
    if uselog == True:
        ax.set_ylabel("log( freq(" + k1+ ") / freq(" + k2+ ") )")
    else:
        ax.set_ylabel("freq(" + k1+ ") - freq(" + k2+ ")")
    title = dic_labels[k1] + " (" + d1 + ")" + " vs "  + dic_labels[k2] + " (" + d2 + ")"
    ax.set_title(title)
    ax.yaxis.grid()
    ax.xaxis.grid()
    if uselog == True:
        figname = figures_dir + "aa_freq_change_log_odd_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 +  ".png"
    else:
        figname = figures_dir + "aa_freq_change_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 + ".png"
    print(figname)
    fig.savefig(figname,dpi=300)
    plt.close()
    #    fig.show()


def fmt_scientific(x):
    a, b = '{:.2e}'.format(x).split('e')
    b = int(b)
    return r'${} \times 10^{{{}}}$'.format(a, b)

def running_mean(x, N):
    cumsum = pd.np.cumsum(pd.np.insert(x, 0, 0)) 
    return (cumsum[N:] - cumsum[:-N]) / N 



if __name__ == '__main__':


# coding: utf-8

# In[2]:

    import pandas as pd
    import os,sys
    import functools
    import matplotlib 
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import numpy as np
    from scipy.stats.stats import rankdata, tiecorrect
    from scipy.stats import stats
    #from statsmodels.formula.api import ols
    from scipy.stats.stats import linregress
    #matplotlib.mathtext.SHRINK_FACTOR = 1
    
    #%matplotlib inline
    
    data_dir = "../../data/"
    figures_dir = "../figures/"
    
    if not os.path.exists(figures_dir):
        os.makedirs(figures_dir)

# open the three reference files
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    
    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks])
    col_dict=["green","red","blue","yellow"]
    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


    taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    taxid2kingdom = {}

    for t in taxid2group:
        gr = taxid2group[t]
        taxid2kingdom[t] = group2kingdom[gr]
    
    taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']


    # load random genes
    df_genes_random = pd.read_feather(data_dir + "df_genes_random.feather")


    # load datasets (per species)
    #datasets = ["full", "pfam", "domains", "linkers"]
    #datasets += ["domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]

    # load datasets (per species)
    #datasets_dom = ["full", "pfam", "domains", "linkers"]
    #datasets_prot = ["full","domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]
    
    #dfs = {}
    #for ty in datasets:   
    #    df = pd.read_feather(data_dir + "df_" + ty+ "_annotation_by_species.feather")
    #    df.iupred_long = df.iupred_long*100
    #    df.iupred_short = df.iupred_short*100
    #    df.seg = df.seg*100
    #    dfs[ty] = df
        # print("TEST: ",ty,dfs[ty])


    df = pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species.csv")
    df=df[df['GC']  != '-']        
    #~ dfs = {}
    #~ dfs["dataset"] = pd.read_feather(data_dir + "df_annotations_by_species.feather")
    #~ dfs["dataset-eggnog"] = pd.read_feather(data_dir + "df_annotations_eggnog_by_species.feather")
    #~ dfs["dataset-pfam"] = pd.read_feather(data_dir + "df_annotations_pfam_by_species.feather")
    #~ dfs["dataset-pfam-full"] = pd.read_feather(data_dir + "df_annotations_pfam_full_sequences_by_species.feather")


    # these taxa should be excluded because their genetic code is different, so translation is wrong
    mycotaxa = []
    for t in name2taxid:
        tax = t.split()[0]
        if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
            mycotaxa += [name2taxid[t]]
        

    # print some statistics
    #for ds in datasets:
    #    print(ds)
    #    print(dfs[ds].kingdom.dropna().value_counts())
    #    print()
    

    # Top-IDP scale
    top_idp = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.341,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    # In[6]:

    # function to get a given AA frequency given the GC content
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
    aas_sorted_gc =     ['I','N','K','F','Y','M','L','D','C','Q','E','H','S','T','V','W','R','A','G','P']
    aas =               ['A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y']

    codons2aa = {
        'ATA':'I', 'ATC':'I', 'ATT':'I', 'ATG':'M', 'ACA':'T', 'ACC':'T', 'ACG':'T', 'ACT':'T',
        'AAC':'N', 'AAT':'N', 'AAA':'K', 'AAG':'K', 'AGC':'S', 'AGT':'S', 'AGA':'R', 'AGG':'R',
        'CTA':'L', 'CTC':'L', 'CTG':'L', 'CTT':'L', 'CCA':'P', 'CCC':'P', 'CCG':'P', 'CCT':'P',
        'CAC':'H', 'CAT':'H', 'CAA':'Q', 'CAG':'Q', 'CGA':'R', 'CGC':'R', 'CGG':'R', 'CGT':'R',
        'GTA':'V', 'GTC':'V', 'GTG':'V', 'GTT':'V', 'GCA':'A', 'GCC':'A', 'GCG':'A', 'GCT':'A',
        'GAC':'D', 'GAT':'D', 'GAA':'E', 'GAG':'E', 'GGA':'G', 'GGC':'G', 'GGG':'G', 'GGT':'G',
        'TCA':'S', 'TCC':'S', 'TCG':'S', 'TCT':'S', 'TTC':'F', 'TTT':'F', 'TTA':'L', 'TTG':'L',
        'TAC':'Y', 'TAT':'Y', 'TAA':'_', 'TAG':'_', 'TGC':'C', 'TGT':'C', 'TGA':'_', 'TGG':'W'}
        
    aa2codons = {}
    aa2codons["ALL"] = []
    for c in codons2aa:
        aa2codons[codons2aa[c]] = aa2codons.get(codons2aa[c],[]) + [c]
        if c != "_":
            aa2codons["ALL"] += [c]
            
    from operator import mul
    

    #get_aa_freq("ALL", 0.1)


    # In[7]:

    dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)",                'other':'Intermediate',                'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)",                "top-idp":"TOP-IDP", "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>",
    'ss_turn':"<Turn propensity>","length":"Length (AA)",                'gc_third':'GC% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred long(%AA)','length_aa':'<Length> (AA)','gc':'GC%',
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff (%AA)',
                'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM(%AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff(%AA)','seg':'<SEG>', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':
                'TYR', 'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                      'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                      'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                      'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                      'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                      'freq_T': 'THR', 'freq_W':                'TRP', 'freq_V': 'VAL',
                      'freq_Y': 'TYR', 'scampi':'TM  (%AA)', 'slider':'Slider',
                      'disembl_coils':'DisEMBL                Coils (%AA)',
                      'disembl_rem465':"DisEMBL REM465 (%AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (%AA)",
                      "globpipe":"GlobPlot (%AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Viruses':'Viruses'}



    
    


    from scipy.optimize import curve_fit



    


    ##########################################################################################################################
    # for each amino acid, and separately for eukarya and bacteria, plot its frequency versus the taxonomic GC% for 4 curves:
    # 1- average freq
    # 2- a linear fit of the data
    # 3- the frequency as it should be if only dependent on GC
    # 4- a model function (w * prob_tot + k) with estimated parameters W and K
    # note: for our big dataset (from NCBI) it is also possible to plot against the GC% of the CDS

    # chose which dataset to use  for the following plots
    #df = dfs["full"]
    
    #good_taxa = set(df.taxon_id.unique())
    good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)

    res = {}

    sum_avg =  {}
    sum_gc =  {}
    sum_func = {}
    sum_func2 = {}
    sum_func3 = {}
    sum_fit =  {}
    sum_fit2 = {}

    # None of the one onepar sets work very well
    #datasets=['gc','avg','poly','lin','onepar','gcplus','gcmult','twopar']
    datasets=['gc','avg','lin','twopar','twopar-lim','gcplus','gcmult']

    
    
    
    tempcols=[]
    for setA in datasets:
        tempcols+=['err['+setA+']']
        tempcols+=['errZ['+setA+']']
        tempcols+=['abserr['+setA+']']
        tempcols+=['abssd['+setA+']']
        tempcols+=['par1['+setA+']']
        tempcols+=['par2['+setA+']']
        tempcols+=['sd1['+setA+']']
        tempcols+=['sd2['+setA+']']
        for setB in datasets:
            tempcols+=['Z['+setA+"-"+setB+']','P['+setA+"-"+setB+']']

    features=['gc']
    features+=tempcols
    cols=['kingdom', 'aa', 'gc','TOP-IDP']
    cols+=list(tempcols)


    sumlist=[]
    

    #print(cols)
    #sys.exit()
    for kingdom in ["Bacteria", "Eukaryota", "Archaea" , "Viruses"]:
        sum={}
        sumZ={}
        sumZval={}
        sumPval={}
        abssum={}
        abssumZ={}
        abssumZval={}
        abssumPval={}
        res[kingdom] = {}
        for setA in datasets:
            sum[setA] =0.0
            abssum[setA] =0.0
            sumZ[setA] =0.0
            for setB in datasets:
                sumPval[setA+"-"+setB]=0.0
                sumZval[setA+"-"+setB]=0.0

            
        fitdata={}
        for aa in aas_sorted_gc:
            
            global aminoacid
            aminoacid = aa
            codons = aa2codons[aminoacid]

            global ydata_mean
            par1={}
            par2={}
            sd1={}
            sd2={}
            
            # get the distribution of the frequency in bacteria
            #d = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa))][["freq_"+aa,"GC","length"]].sort_values("GC").dropna(subset=["GC"])


            #d = df.loc[(df.kingdom == kingdom) && (df.gc  != '-' )][["freq_"+aa,"GC","length"]].sort_values("GC").dropna(subset=["GC"])

            d = df.loc[(df.kingdom == kingdom) ][["freq_"+aa,"GC","length"]].sort_values("GC").dropna(subset=["GC"])
            d=d[d["GC"]  != '-']
            xdata=[]
            ydata=[]
            for a in d["GC"]:
                #print(a,type(a),float(a))
                xdata+=[float(a)/100]
            #temp_df = df.loc[(df.kingdom == kingdom)][list(set(["freq_"+aa,"GC","length"]))].dropna()
                
            #xdata = d["GC"] #/100.0
            ydata = d["freq_"+aa] # /d["length"]

            fig, ax = plt.subplots()
            
            # plot the data
            ax.scatter(xdata,ydata,color="lightgrey",edgecolor="None",alpha=0.3,label = "Data")
            
            
            # get an estimate of the error given only the average AA frequence
            ydata_mean = ydata.mean()
            ydata_std = ydata.std()
            avg_ydata = [ydata_mean for _ in ydata]
            fitdata['avg']=avg_ydata            
            abserr,abssd= get_abserror(ydata,fitdata['avg'])
            ax.plot(xdata,avg_ydata,color="blue",linestyle="--",label = "Average, err= "+str(round(abserr,4)))
            par1['avg']=0
            par2['avg']=avg_ydata[0]-(len(codons)/61)
            sd1['avg']=0
            sd2['avg']=abssd

            
            # create a first estimate of the fitting function and plot its
            initial_w = 1
            initial_k = 0
            #print (xdata,ydata)
            estimated_ydata = my_func(xdata,initial_w,initial_k)
            fitdata['gc']=estimated_ydata
            abserr,abssd= get_abserror(ydata,fitdata['gc'])
            ax.plot(xdata,estimated_ydata,color="darkred",linestyle="--",label = "GC, Err= "+str(round(abserr,4)))
            par1['gc']=1
            par2['gc']=0
            sd1['gc']=0
            sd2['gc']=0

            
            # do a curve fit to estimate the parameters
            popt, pcov = curve_fit(my_func, xdata, ydata )
            est_w, est_k = popt
            #print (popt)
            optimized_ydata = my_func(xdata, est_w, est_k)
            fitdata['twopar']=optimized_ydata
            abserr,abssd= get_abserror(ydata,fitdata['twopar'])
            #ax.plot(xdata, optimized_ydata, color="orange",lw=2, label='2par, err= '+str(round(abserr,4)))
            par1['twopar']=est_w
            par2['twopar']=est_k
            sd1['twopar'],sd2['twopar']=np.sqrt(np.diag(pcov))
    
            
            # do a curve fit to estimate the parameters (limited to 0,1 this is wrong)
            #par_bounds=[0.,1.]
            par_bounds=[[0.,-np.inf],[1.,+np.inf]]
            popt2, pcov2 = curve_fit(my_func, xdata,ydata, p0=[1.,0.],bounds=par_bounds )
            est2_w, est2_k = popt2
            optimized2_ydata = my_func(xdata, est2_w, est2_k)
            fitdata['twopar-lim']=optimized2_ydata
            abserr,abssd= get_abserror(ydata,fitdata['twopar-lim'])
            ax.plot(xdata, optimized2_ydata, color="pink",lw=2, label='twopar-lim, err= '+str(round(abserr,4)))
            par1['twopar-lim']=est2_w
            par2['twopar-lim']=est2_k
            sd1['twopar-lim'],sd2['twopar-lim']=np.sqrt(np.diag(pcov2))
            
            

            # do a curve fit to estimate the parameters GC plus afixed number
            par_bounds=[[0.999999999,-np.inf],[1.0000000001,+np.inf]]
            popt_gcplus, pcov_gcplus = curve_fit(my_func, xdata,ydata, p0=[1.,0.],bounds=par_bounds )
            est_gcplus_w,est_gcplus_k  = popt_gcplus
            gcplus_ydata = my_func(xdata, est_gcplus_w,est_gcplus_k)
            fitdata['gcplus']=gcplus_ydata
            abserr,abssd= get_abserror(ydata,fitdata['gcplus'])
            ax.plot(xdata, gcplus_ydata, color="purple",lw=2, label='gcplus, err='+str(round(abserr,4)))
            par1['gcplus']=est_gcplus_w
            par2['gcplus']=est_gcplus_k
            sd1['gcplus']=0
            tmp,sd2['gcplus']=np.sqrt(np.diag(pcov_gcplus))

            # do a curve fit to estimate the parameters GC multiplied with a constant
            par_bounds=[[-np.inf,-1.e-20],[np.inf,1.e-20]]
            popt_gcmult, pcov_gcmult = curve_fit(my_func, xdata,ydata, p0=[1.,0.],bounds=par_bounds )
            est_gcmult_w,est_gcmult_k  = popt_gcmult
            gcmult_ydata = my_func(xdata, est_gcmult_w,est_gcmult_k)
            fitdata['gcmult']=gcmult_ydata
            abserr,abssd= get_abserror(ydata,fitdata['gcmult'])
            ax.plot(xdata, gcmult_ydata, color="red",lw=1, label='gcmult, err= '+str(round(abserr,4)))
            par1['gcmult']=est_gcmult_w
            par2['gcmult']=est_gcmult_k
            sd1['gcmult'],tmp=np.sqrt(np.diag(pcov_gcmult))
            sd2['gcmult']=0
            
            
            # calculate and plot the fitting lines
            popt_lin, pcov_lin = curve_fit(my_func_lin, xdata,ydata)
            fitted_ydata = my_func_lin(xdata, popt_lin[0], popt_lin[1])
            fitdata['lin']=fitted_ydata
            abserr,abssd= get_abserror(ydata,fitdata['lin'])
            plt.plot(xdata, fitted_ydata, lw=2,color="green",label="Linear, err= "+str(round(abserr,4)))
            par1['lin']=popt_lin[1]
            par2['lin']=popt_lin[0]
            sd1['lin'],sd2['lin']=np.sqrt(np.diag(pcov_lin))
            
            
            ## calculate and plot the fitting lines
            #fit2 = np.polyfit(xdata,ydata,2)
            #fit2_fn = np.poly1d(fit2) 
            #fitted2_ydata = fit2_fn(xdata)
            #plt.plot(xdata, fitted2_ydata, lw=2,color="green",label="Polynomial Fit")
            #fitdata['poly']=fitted2_ydata
            #par1['poly']=fit2_fn[0]
            #par2['poly']=fit2_fn[1]
            #sd1['poly']=fit2_fn[0]
            #sd2['poly']=fit2_fn[1]
            
            # All 
            err={}
            errZ={}
            abserr={}
            abserrZ={}
            sd={}
            abssd={}
            for set in datasets:
                err[set],sd[set]= get_error(ydata,fitdata[set])
                #print ("Error",kingdom,aa,set,error[set],errZ[set])
                sum[set]+=err[set]/20.
            for set in datasets:
                abserr[set],abssd[set]= get_abserror(ydata,fitdata[set])
                #print ("Error",kingdom,aa,set,error[set],errZ[set])
                abssum[set]+=abserr[set]/20.
            for set in datasets:
                errZ[set]= err[set]/sd['lin']
                sumZ[set]+=errZ[set]/20.
                
            for set in datasets:
                print ("Param-func2:",aa,kingdom,par1[set],par2[set],err[set],errZ[set])


            #Some T-test
            # We need to test all combinations of models against the polynomial and linear models.
            Z={}
            P={}
            for setA in datasets:
                for setB in datasets:
                    Z[setA+"-"+setB],P[setA+"-"+setB]=T_test(ydata,fitdata[setA],fitdata[setB])
                    sumZval[setA+"-"+setB]+=Z[setA+"-"+setB]/20
                    sumPval[setA+"-"+setB]+=P[setA+"-"+setB]/20
                    print ("Stat: ",setA,setB,aa,err[setA],err[setB],kingdom,Z[setA+"-"+setB],P[setA+"-"+setB])

                    
            gc=get_gc_freq(aa)

            line=[kingdom, aa, gc,top_idp[aa]]
            for setA in datasets:
                line+=[err[setA]]
                line+=[errZ[setA]]
                line+=[abserr[setA]]
                line+=[abssd[setA]]
                line+=[par1[setA]]
                line+=[par2[setA]]
                line+=[sd1[setA]]
                line+=[sd2[setA]]
                for setB in datasets:
                    line+=[Z[setA+"-"+setB],P[setA+"-"+setB]]
            sumlist.append(line)

            
            #sys.exit()

            text_str=''
            for set in datasets:
                text_str += set+" err = " + str(round(err[set],4)) + "\n"
            
            #print("FIT",aa,kingdom,text_str)
            #bb = ax.text(ax.get_xlim()[0], ax.get_ylim()[1], text_str, fontsize=12,
            #        ha="left", va="top",
            #        bbox=dict(boxstyle="square,pad=0",
            #                ec=(0.0, 0.0, 0.0,1),
            #                fc=(1., 1, 1,1),),    
            #            )
            
            
            ax.set_xlabel("GC%")
            ax.set_ylabel("Frequency")
            ax.set_title(dic_labels[aa] +  " - " + dic_labels[kingdom])
            ax.legend(loc="lower left")
            
            
            fig_title =  figures_dir + "aafreq_" + aa + "_" + kingdom   + ".eps"
            
            print(fig_title)
            fig.savefig(fig_title)
            plt.close()
            #fig.show()
            
        #############################


        line=[kingdom,'sum',1.0, 1.0]
        for setA in datasets:
            line+=[sum[setA]]
            line+=[sumZ[setA]]
            line+=[abssum[setA]]
            line+=[abssd[setA]]
            line+=[0.0]
            line+=[0.0]
            line+=[0.0]
            line+=[0.0]
            for setB in datasets:
                line+=[sumZval[setA+"-"+setB],sumPval[setA+"-"+setB]]
        sumlist.append(line)
            
    #print(sumlist)
    summary=pd.DataFrame(sumlist,columns=cols)
    #summary=pd.DataFrame(sumlist,columns=features)
    #print(summary)
    for kingdom in ["Bacteria", "Eukaryota", "Archaea", "Viruses"]:
        # A very simpe barplot
        #print (summary)
        d = summary[(summary.kingdom == kingdom) ].sort_values(by=['gc'])
        for feature in features:
            #print (kingdom,feature)
            #print (d[feature])
            fig, ax =plt.subplots()
            xdata=d['aa'].tolist()
            #xdata=summary.loc[summary.kingdom == kingdom ]['aa'] .sort_values(by=['gc'])
            x = np.arange(21)
            plt.bar(x,d[feature])
            plt.xticks(x,xdata)
            ax.set_ylabel("<Value> " + feature)
            title="Values for " + feature + " in " + kingdom
            ax.set_title(title)
            figname = figures_dir + "barplot-aa_" + feature + "_" + kingdom + ".eps"
            fig.savefig(figname,dpi=300)
            plt.close()
        # examples
    #print (summary)

    
        # Let us make specific plots for par2 (with sd)
        for feature in datasets:
            fig, ax =plt.subplots()
            xdata=d['aa'].tolist()
            #xdata=summary.loc[summary.kingdom == kingdom ]['aa'] .sort_values(by=['gc'])
            x = np.arange(21)
            plt.bar(x,d['par2['+feature+']'],yerr=d['sd2['+feature+']'])
            plt.xticks(x,xdata)
            ax.set_ylabel("<Shift> " )
            title="Values for shift in " + kingdom
            ax.set_title(title)
            figname = figures_dir + "barplot-aa_sd_" + feature + "_" + kingdom + ".eps"
            fig.savefig(figname,dpi=300)
            plt.close()
            
        #print (summary)
        # We need to make more than one table

        errcols=['aa','kingdom','gc','TOP-IDP']
        errZcols=list(errcols)
        Zsets=list(errcols)
        Psets=list(errcols)
        abserrcols=list(errcols)
        abserrcols+=['abserr[gc]','abssd[gc]','err[gc]','errZ[gc]']
        for set in datasets:
            errcols+=['err['+set+']']
            errZcols+=['errZ['+set+']']
            Zsets+=['Z['+set+'-lin]']
            Psets+=['P['+set+'-lin]']
            
        #print (errcols)
        
        tempdf=summary.loc[(summary.kingdom==kingdom)][errcols].sort_values('gc')
        print(tempdf.to_latex(float_format='%.4f')) # float_format='%.5f')

        tempdf=summary.loc[(summary.kingdom==kingdom)][abserrcols].sort_values('gc')
        print(tempdf.to_latex(float_format='%.4f')) # float_format='%.5f')

        tempdf=summary.loc[(summary.kingdom==kingdom)][errZcols].sort_values('gc')
        print(tempdf.to_latex(float_format='%.2f'))

        
        tempdf=summary.loc[(summary.kingdom==kingdom)][Zsets].sort_values('gc')
        print(tempdf.to_latex(float_format='%.1f'))
        tempdf=summary.loc[(summary.kingdom==kingdom)][Psets].sort_values('gc')
        print(tempdf.to_latex(float_format='%.1E'))

    # We will make some combined plots
    bar_width=0.2
    opacity = 0.4

    for feature in datasets:
        i=0
        fig, ax =plt.subplots()
        for kingdom in ["Bacteria", "Eukaryota", "Archaea", "Viruses"]:
            d = summary[(summary.kingdom == kingdom) ].sort_values(by=['gc'])
            xdata=d['aa'].tolist()
            #xdata=summary.loc[summary.kingdom == kingdom ]['aa'] .sort_values(by=['gc'])
            x = np.arange(21)
            plt.bar(x+i*bar_width,d['par2['+feature+']'],yerr=d['sd2['+feature+']'],bar_width,label=kingdom,color=col_dict[i],edgecolor='black')
            i+=1
        ax.set_facecolor('white')
        plt.xticks(x + bar_width/2,xdata)
        plt.legend()
        ax.set_ylabel("<Shift> " )
        title="Values for shift " 
        ax.set_title(title)
        figname = figures_dir + "barplot-aa_sd_" + feature  + ".eps"
        fig.savefig(figname,dpi=300)
        plt.close()
            
        
    # Finally make some summary data
    tempdf=summary.loc[(summary.aa=='sum')][errcols]
    print(tempdf.to_latex(float_format='%.4f'))
    # Finally make some summary data
    tempdf=summary.loc[(summary.aa=='sum')][errZcols]
    print(tempdf.to_latex(float_format='%.4f'))
    tempdf=summary.loc[(summary.aa=='sum')][Zsets]
    print(tempdf.to_latex(float_format='%.2f'))
    #tempdf=summary.loc[(summary.aa=='sum')][Psets]
    #print(tempdf.to_latex(float_format='%.1E'))
            
    
