#!/usr/bin/env python
import pandas as pd
import os
from Bio.Seq import Seq
from Bio import SeqIO
from Bio import AlignIO
from Bio.SeqRecord import SeqRecord
from Bio.Alphabet import IUPAC
import Bio.Align
import re
from Bio.SeqUtils.ProtParam import ProteinAnalysis
import math
import os.path
from Bio import motifs
import sys


def get_phylum(s):
    try:
        return s.split(";")[1]
    except:
        return None    

def get_kingdom(s):
    try:
        return s.split(";")[0]
    except:
        return None    

def split_ali(alignment,name):
    vir=Bio.Align.MultipleSeqAlignment([])
    arch=Bio.Align.MultipleSeqAlignment([])
    bact=Bio.Align.MultipleSeqAlignment([])
    euk=Bio.Align.MultipleSeqAlignment([])
    vir_highGC=Bio.Align.MultipleSeqAlignment([])
    arch_highGC=Bio.Align.MultipleSeqAlignment([])
    bact_highGC=Bio.Align.MultipleSeqAlignment([])
    euk_highGC=Bio.Align.MultipleSeqAlignment([])
    vir_medGC=Bio.Align.MultipleSeqAlignment([])
    arch_medGC=Bio.Align.MultipleSeqAlignment([])
    bact_medGC=Bio.Align.MultipleSeqAlignment([])
    euk_medGC=Bio.Align.MultipleSeqAlignment([])
    vir_lowGC=Bio.Align.MultipleSeqAlignment([])
    arch_lowGC=Bio.Align.MultipleSeqAlignment([])
    bact_lowGC=Bio.Align.MultipleSeqAlignment([])
    euk_lowGC=Bio.Align.MultipleSeqAlignment([])
    for record in alignment:
        #print (record.name)
        mtemp=re.split("[_]",record.name)
        mnemonic=mtemp[1]
        sprot=record.name
        tax_id=memo2taxid.get(mnemonic,pd.np.nan)
        #phylum = taxid2phylum.get(tax_id,pd.np.nan)
        kingdom = taxid2kingdom.get(tax_id,pd.np.nan)
        try:
            GC = float(taxid2gc[int(tax_id)])
        except:
            continue
        vir_seq=[]
        arch_seq=[]
        bact_seq=[]
        euk_seq=[]
        vir_seq_highGC=[]
        arch_seq_highGC=[]
        bact_seq_highGC=[]
        euk_seq_highGC=[]
        vir_seq_medGC=[]
        arch_seq_medGC=[]
        bact_seq_medGC=[]
        euk_seq_medGC=[]
        vir_seq_lowGC=[]
        arch_seq_lowGC=[]
        bact_seq_lowGC=[]
        euk_seq_lowGC=[]
        length=len(record.seq)
        #print (tax_id,kingdom,GC)
        #print (bact,euk,record)
        if (kingdom == 'Bacteria'):
            #print ("Bacteria",record)
            bact.append(record)
            bact_seq+=[record.seq]
            if (GC > highcutoff):
                bact_highGC.append(record)
                bact_seq_highGC+=[record.seq]
            elif (GC < lowcutoff):
                bact_lowGC.append(record)
                bact_seq_lowGC+=[record.seq]
            else:
                bact_medGC.append(record)
                bact_seq_medGC+=[record.seq]

        elif(kingdom == 'Eukaryota'):
            euk.append(record)
            euk_seq+=[record.seq]
            if (GC > highcutoff):
                euk_highGC.append(record)
                euk_seq_highGC+=[record.seq]
            elif (GC < lowcutoff):
                euk_lowGC.append(record)
                euk_seq_lowGC+=[record.seq]
            else:
                euk_medGC.append(record)
                euk_seq_medGC+=[record.seq]
        elif(kingdom == 'Archaea'):
            arch.append(record)
            arch_seq+=[record.seq]
            if (GC > highcutoff):
                arch_highGC.append(record)
                arch_seq_highGC+=[record.seq]
            elif (GC < lowcutoff):
                arch_lowGC.append(record)
                arch_seq_lowGC+=[record.seq]
            else:
                arch_medGC.append(record)
                arch_seq_medGC+=[record.seq]
        elif(kingdom == 'Viruses'):
            vir.append(record)
            vir_seq+=[record.seq]
            if (GC > highcutoff):
                vir_highGC.append(record)
                vir_seq_highGC+=[record.seq]
            elif (GC < lowcutoff):
                vir_lowGC.append(record)
                vir_seq_lowGC+=[record.seq]
            else:
                vir_medGC.append(record)
                vir_seq_medGC+=[record.seq]
    AlignIO.write(bact, name+"_bac.fa", "fasta")
    AlignIO.write(euk, name+"_euk.fa", "fasta")
    AlignIO.write(arch, name+"_arch.fa", "fasta")
    AlignIO.write(vir, name+"_vir.fa", "fasta")
    AlignIO.write(bact_highGC, name+"_bac_highGC.fa", "fasta")
    AlignIO.write(euk_highGC, name+"_euk_highGC.fa", "fasta")
    AlignIO.write(arch_highGC, name+"_arch_highGC.fa", "fasta")
    AlignIO.write(vir_highGC, name+"_vir_highGC.fa", "fasta")
    AlignIO.write(bact_medGC, name+"_bac_medGC.fa", "fasta")
    AlignIO.write(euk_medGC, name+"_euk_medGC.fa", "fasta")
    AlignIO.write(arch_medGC, name+"_arch_medGC.fa", "fasta")
    AlignIO.write(vir_medGC, name+"_vir_medGC.fa", "fasta")
    AlignIO.write(bact_lowGC, name+"_bac_lowGC.fa", "fasta")
    AlignIO.write(euk_lowGC, name+"_euk_lowGC.fa", "fasta")
    AlignIO.write(arch_lowGC, name+"_arch_lowGC.fa", "fasta")
    AlignIO.write(vir_lowGC, name+"_vir_lowGC.fa", "fasta")
    return()
    
if __name__ == '__main__':
    aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']

    data_dir = "../../data/"
    results_dir = data_dir
    input_dir = data_dir+"/pfam/"
    pfam_dir = data_dir+"/pfam/"

    # GC content is in prokaryotes.txt and eukaryotes.txt
    # FROM ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/
    # Reading in GC (and actually taxonomy) information
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    genomes_viruses_file = data_dir + "viruses.txt"

    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_virus = pd.read_csv(genomes_viruses_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks,df_virus])

    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]
    #print (taxid2gc)
    #print (taxid2gc[999810])

    #taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    #group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    #taxid2kingdom = {}

    #for t in taxid2group:
    #    gr = taxid2group[t]
    #    taxid2kingdom[t] = group2kingdom[gr]
    
    #taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    #name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']
    # End of reading in GC
    
    
    # This takes too long
    codes={"V":"Viruses","E":"Eukaryota","B":"Bacteria","A":"Archaea","O":"Other","X":"Unknown"}
    taxid2kingdom={}
    memo2taxid={}
    with open(data_dir+'speclist.txt') as fp:
        for line in fp:
            if re.match('.*:\sN=',line):
                s=line.split()
                taxid=s[2].replace(':','')
                mnemonic=s[0]
                kingdom=codes[s[1]]
                memo2taxid[mnemonic]=taxid
                taxid2kingdom[taxid]=kingdom
                #print (line, taxid,mnemonic,kingdom)


    highcutoff=60.
    lowcutoff=20.


    with open(data_dir+'speclist.txt') as fp:
        for line in fp:
            if re.match('.*:\sN=',line):
                s=line.split()
                taxid=s[2].replace(':','')
                mnemonic=s[0]
                kingdom=codes[s[1]]
                memo2taxid[mnemonic]=taxid
                taxid2kingdom[taxid]=kingdom
                #print (line, taxid,mnemonic,kingdom)
    file_list = []
    for f in os.listdir(input_dir):
        if f.endswith(".sto"):
            file_list += [input_dir + f]
    #for f in sys.argv[1:]:
    #    file_list += [f]
    cutoff=10
    alignment={}
    for f in file_list:
        #print (f)
        #bact=MultipleSeqAlignment()
        #euk=MultipleSeqAlignment()
        s=re.search('PF\d\d\d\d\d',f)
        pfam=s.group(0)

        name=pfam_dir+pfam
        #if os.path.isfile(name+"_vir_highGC.fa"):
        #    continue
        alignment = AlignIO.read(f, "stockholm")
        split_ali(alignment,name)
        
        #sys.exit()
        # Now we need to produce one version that filters away positions with >75% gaps)
        nogapali=Bio.Align.MultipleSeqAlignment([])
        name=pfam_dir+pfam+"_mingap"
        length=len(alignment[0].seq)
        fraction={}
        for i in range(0,length):
            match=0
            gap=0
            #print ("test",i)
            for record in alignment:
                seq=record.seq[i].upper()
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]',seq)):
                    match+=1.
                else:
                    gap+=1.
                #print (i,seq,match,gap)
            fraction[i]=match/(match+gap)
        num=len(alignment)
        j=0
        cutoff=0.25
        empty_seq=Seq("")
        for record in alignment:
            k=0
            sequence=SeqRecord(empty_seq,id=record.id)
            for i in range(0,length):
                if fraction[i]>cutoff:
                    seq=record.seq[i].upper()
                    if (re.match('[ACDEFGHIKLMNPQRSTVWY]',seq)):
                        aa=seq
                    else:
                        aa="-"
                    #sequence.seq(k,alphabet=ProteinAlphabet(),character=aa)
                    sequence.seq+=Seq(aa,IUPAC.protein)
                    sequence.name=record.name
                    #print ("TEST",i,k,j,fraction[i],seq,sequence.seq)
                    k+=1
            nogapali.append(sequence)
        #print (nogapali)
        split_ali(nogapali,name)
        #sys.exit()


                    
