#!/usr/bin/env python
import matplotlib
matplotlib.use('Agg')
import pandas as pd
import numpy as np
import os
import re
import seaborn as sns
import matplotlib.pyplot as plt

aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
aas_sorted_gc =     ['I','N','K','F','Y','M','L','D','C','Q','E','H','S','T','V','W','R','A','G','P']
aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
               'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
               'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}
aaindex_gc = {'I':0,'N':1,'K':2,'F':3,'Y':4,'M':5,'L':6,'D':7,'C':8,'Q':9,'E':10,'H':11,'S':12,'T':13,'V':14,'W':15,'R':16,'A':17,'G':18,'P':19}
aaindex_topidp = {'W':0,'F':1,'Y':2,'I':3,'M':4,'L':5,'V':6,'N':7,'C':8,'T':9,'A':10,'G':11,'R':12,'D':13,'H':14,'Q':15,'S':16,'K':17,'E':18,'P':19}

dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)", "top-idp":"TOP-IDP", "topidp-S":"TOP-IDP - S","topidp-SKP":"TOP-IDP - SKP","topidp-SP":"TOP-IDP - SP",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'GC% (third nucleotide)','taxon_gc':'GC% (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
    'random':'Expected','iupred_long':'IUpred long (%AA)','length_aa':'<Length> (AA)','gc':'GC%',
    'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)', 'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM (%AA)',"volatility":"<Volatility>", 'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity> (Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with  expected', 'aa_corr_dot':'AA freq\ncorrelation with   expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A':
    'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P':'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'ALA', 'freq_C':
    'CYS', 'freq_E': 'GLU', 'freq_D': 'ASP', 'freq_G': 'GLY',
    'freq_F': 'PHE', 'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K':
    'LYS', 'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
    'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S': 'SER', 'freq_R':
    'ARG', 'freq_T': 'THR', 'freq_W': 'TRP', 'freq_V': 'VAL',
    'freq_Y': 'TYR', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Viruses':'Viruses'}
        

sns.set()
sns.set(font_scale=1.4)

pfam_list = []

data_dir="../../data/"

if os.path.isdir(data_dir+"/substmat/"):
    input_dir=data_dir+"/substmat/"
else:
    input_dir=data_dir
figures_dir="../figures/"

for f in os.listdir(input_dir):
    if f.endswith("-num_ll.npy"):
        mtemp=re.split("[-]",f)
        pfam_list+=[mtemp[0]]

        
subst_hh=np.zeros((20, 20))
subst_ll=np.zeros((20, 20))
subst_hl=np.zeros((20, 20))
num_l=np.zeros(20)
num_h=np.zeros(20)
num_ll=np.zeros(20)
num_hh=np.zeros(20)

for p in pfam_list:
    temp_subst_ll=np.load(input_dir+p+"-subst_ll.npy")
    temp_subst_hh=np.load(input_dir+p+"-subst_hh.npy")
    temp_subst_hl=np.load(input_dir+p+"-subst_hl.npy")
    temp_num_ll=np.load(input_dir+p+"-num_ll.npy")
    temp_num_hh=np.load(input_dir+p+"-num_hh.npy")
    temp_num_h=np.load(input_dir+p+"-num_h.npy")
    temp_num_l=np.load(input_dir+p+"-num_l.npy")

    subst_ll+=temp_subst_ll
    subst_hh+=temp_subst_hh
    subst_hl+=temp_subst_hl
    num_ll+=temp_num_ll
    num_hh+=temp_num_hh
    num_l+=temp_num_l
    num_h+=temp_num_h

if os.path.isdir(data_dir+"/substmat/"):
    np.savetxt(data_dir+"all-subst_hh.csv",subst_hh)
    np.savetxt(data_dir+"all-subst_hl.csv",subst_hl)
    np.savetxt(data_dir+"all-subst_ll.csv",subst_ll)
    
    np.save(data_dir+"all-subst_hh.npy",subst_hh)
    np.save(data_dir+"all-subst_hl.npy",subst_hl)
    np.save(data_dir+"all-subst_ll.npy",subst_ll)
    
    np.save(data_dir+"all-num_hh.npy",num_hh)
    np.save(data_dir+"all-num_ll.npy",num_ll)
    np.save(data_dir+"all-num_l.npy",num_l)
    np.save(data_dir+"all-num_h.npy",num_h)

# logodds  p(i|j)/(p(i)*p(j))
#  ((num(i)/sum(x))num(i|j)/sum(xy)/((num(i)/sum(x))*(num(j)/sum(x)))

sum_subst_hh=np.sum(subst_hh)
sum_subst_ll=np.sum(subst_ll)
sum_subst_hl=np.sum(subst_hl)
sum_num_hh=np.sum(num_hh)
sum_num_ll=np.sum(num_ll)
sum_num_l=np.sum(num_l)
sum_num_h=np.sum(num_h)
sum_num_hl=(sum_num_l+sum_num_h)/2

#print ("HH",num_hh,subst_hh,sum_subst_hh,sum_num_hh)
#print ("LL",num_ll,subst_ll,sum_subst_ll,sum_num_ll)
#print ("HL",num_hl,subst_hl,sum_subst_hl,sum_num_hl)


#nummat_hh=np.matmul(num_hh,np.transpose(num_hh))
#nummat_ll=np.matmul(num_ll,num_ll)
#nummat_hl=np.matmul(num_hl,num_hl)
num_hh=num_hh.reshape(20,1)
num_ll=num_ll.reshape(20,1)
num_h=num_h.reshape(20,1)
num_l=num_l.reshape(20,1)

nummat_hh=num_hh*num_hh.T
nummat_ll=num_ll*num_ll.T
nummat_hl=num_l*num_h.T

freq_num_hh=np.divide(num_hh,sum_num_hh)
freq_num_ll=np.divide(num_ll,sum_num_ll)
freq_num_h=np.divide(num_h,sum_num_hl)
freq_num_l=np.divide(num_l,sum_num_hl)


freq_subst_hh=np.divide(subst_hh,sum_subst_hh) #*freq_num_hh.T
freq_subst_ll=np.divide(subst_ll,sum_subst_ll) #*freq_num_ll.T
freq_subst_hl=np.divide(subst_hl,sum_subst_hl) #*freq_num_hl.T


exp_subst_hh=np.divide(nummat_hh,sum_num_hh*sum_num_hh)
exp_subst_ll=np.divide(nummat_ll,sum_num_ll*sum_num_ll)
exp_subst_hl=np.divide(nummat_hl,sum_num_hl*sum_num_hl)


odds_hh=np.divide(freq_subst_hh,exp_subst_hh)
odds_ll=np.divide(freq_subst_ll,exp_subst_ll)
odds_hl=np.divide(freq_subst_hl,exp_subst_hl)

logodds_hh=np.log(odds_hh)
logodds_ll=np.log(odds_ll)
logodds_hl=np.log(odds_hl)

#print ("test1",num_hh[1],subst_hh[1,1],sum_subst_hh,sum_num_hh)
#print ("test2",nummat_hh[1,1],freq_num_hh[1])
#print ("test3",freq_subst_hh[1,1],exp_subst_hh[1,1],odds_hh[1,1],logodds_hh[1,1])

#print ("HH",logodds_hh)
#print ("LL",logodds_ll)
#print ("HL",logodds_hl)

hl_hh=logodds_hl-logodds_hh
hl_ll=logodds_hl-logodds_ll

hl_lh=logodds_hl-logodds_hl.T
#hhT=logodds_hh-logodds_hh.T
#llT=logodds_ll-logodds_ll.T
hh_ll=logodds_hh-logodds_ll

#print ("HL-HH",hl_hh)
#print ("HL-LL",hl_ll)

hh = pd.DataFrame(logodds_hh,columns=aas,index=aas)
ll = pd.DataFrame(logodds_ll,columns=aas,index=aas)
hl = pd.DataFrame(logodds_hl,columns=aas,index=aas)
hl_hh = pd.DataFrame(hl_hh,columns=aas,index=aas)
hl_ll = pd.DataFrame(hl_ll,columns=aas,index=aas)

#hhT = pd.DataFrame(hhT,columns=aas,index=aas)
#llT = pd.DataFrame(llT,columns=aas,index=aas)
hl_lh = pd.DataFrame(hl_lh,columns=aas,index=aas)
hh_ll = pd.DataFrame(hh_ll,columns=aas,index=aas)


#print(hl.to_latex())

# Do plots
for i in range(0,20):
    b2e=freq_subst_hl[i,:]
    e2b=freq_subst_hl[:,i]
    e2b=e2b/np.sum(e2b)
    b2e=b2e/np.sum(b2e)
    #e2b=e2b/e2b[i]
    #b2e=b2e/b2e[i]
    aa=aas[i]
    fig, ax = plt.subplots()
    index = np.arange(20)
    bar_width = 0.35
    
    opacity = 0.4
    #error_config = {'ecolor': '0.3'}

    rects1 = plt.bar(index, e2b, bar_width,
                 alpha=opacity,
                 color='red',
                 label='Low2High',edgecolor='black')

    rects2 = plt.bar(index + bar_width, b2e , bar_width,
                 alpha=opacity,
                 color='green',
                 label='High2Low',edgecolor='black')
    ax.set_facecolor('white')
    plt.xlabel('Amino Acid')
    plt.ylabel('Frequency')
    plt.title('Frequency of substitutions for '+dic_labels[aa])
    plt.xticks(index + bar_width / 2, aas)
    plt.legend()
    ax.yaxis.grid()
    ax.grid(color='grey', linestyle='-', linewidth=1)
    ax.xaxis.grid(False)
    plt.tight_layout()
    #plt.show()
    plt.savefig(figures_dir + "subst-hl-barplot_"+aa+".eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close()

fig, ax = plt.subplots()
fig.set_size_inches(18,15)

lh=hl.T
#for i in range(0,20):
temp=ll[aas_sorted_gc]
ll_gc=temp.reindex(aas_sorted_gc)

temp=hh[aas_sorted_gc]
hh_gc=temp.reindex(aas_sorted_gc)

temp=hl[aas_sorted_gc]
hl_gc=temp.reindex(aas_sorted_gc)

temp=lh[aas_sorted_gc]
lh_gc=temp.reindex(aas_sorted_gc)

temp=hl_ll[aas_sorted_gc]
hl_ll_gc=temp.reindex(aas_sorted_gc)

temp=hl_hh[aas_sorted_gc]
hl_hh_gc=temp.reindex(aas_sorted_gc)

temp=hh_ll[aas_sorted_gc]
hh_ll_gc=temp.reindex(aas_sorted_gc)

temp=hl_lh[aas_sorted_gc]
hl_lh_gc=temp.reindex(aas_sorted_gc)

raw_hl = pd.DataFrame(freq_subst_hl,columns=aas,index=aas)
temp=raw_hl[aas_sorted_gc]
raw_hl_gc=temp.reindex(aas_sorted_gc)

        
sns.heatmap(ll_gc,cmap="bwr",vmin=-5,vmax=+5) # ,method="single",metric='correlation')annot=True,
ax.set_title("LogOdds matrix for Low-Low substitutions")
plt.savefig(figures_dir + "subst-ll-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(hh_gc,cmap="bwr",vmin=-5,vmax=+5) # ,method="single",metric='correlation')annot=True,
ax.set_title("LogOdds matrix for High-High substitutions")
plt.savefig(figures_dir + "subst-hh-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(hl_gc,cmap="bwr",vmin=-5,vmax=+5) # ,method="single",metric='correlation')annot=True,
ax.set_title("LogOdds matrix for Low-High substitutions")
plt.savefig(figures_dir + "subst-hl-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(lh_gc,cmap="bwr",vmin=-5,vmax=+5) # ,method="single",metric='correlation')annot=True,
ax.set_title("LogOdds matrix for High-Low substitutions")
plt.savefig(figures_dir + "subst-lh-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(hl_hh_gc,cmap="bwr",vmin=-1,vmax=+1) # ,method="single",metric='correlation')annot=True,
ax.set_title("Difference in LogOdds matrix Low-High vs High-High")
plt.savefig(figures_dir + "subst-hl_hh-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(hl_ll_gc,cmap="bwr",vmin=-1,vmax=+1) # ,method="single",metric='correlation')annot=True,
ax.set_title("Difference in LogOdds matrix Low-High vs Low-Low")
plt.savefig(figures_dir + "subst-hl_ll-heatmap.eps",bbox_inches="tight")

# Transformed matrix

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(hl_lh_gc,cmap="bwr",vmin=-1,vmax=+1) # ,method="single",metric='correlation')annot=True,
ax.set_title("Difference in LogOdds matrix Low-High vs High-Low")
plt.savefig(figures_dir + "subst-hl-lh-heatmap.eps",bbox_inches="tight")

fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(hh_ll_gc,cmap="bwr",vmin=-1,vmax=+1)# ,method="single",metric='correlation')annot=True,
ax.set_title("Difference in LogOdds matrix High-High vs LowHigh-Low")
plt.savefig(figures_dir + "subst-hh-ll-heatmap.eps",bbox_inches="tight")

# Some more detailed analysis (of raw data)
fig, ax = plt.subplots()
fig.set_size_inches(18,15)
sns.heatmap(raw_hl_gc,cmap="Purples") # ,method="single",metric='correlation')annot=True,
ax.set_title("Subsitutions matrix for High-Low substitutions")
plt.savefig(figures_dir + "subst-raw-hl-heatmap.eps",bbox_inches="tight")

# Methods: single, complete, average (default), weighted, cenroid, median, ward
# Metric: euclidian, minkowski, cityblock, correlation,hamming,jaccard,chebyshev
# Metric has no influence
# Zscore none (default), 0 row, 1 columns


plt.close('all')

# Clustermaps 


for m in ["single", "complete", "average", "weighted", "centroid", "median", "ward"]:
    #    for n in ["euclidian", "minkowski", "cityblock", "correlation","hamming","jaccard","chebyshev"]:
    fig, ax = plt.subplots()
    fig.set_size_inches(18,15)
    sns.clustermap(hl_lh,cmap="bwr",vmin=-1,vmax=+1,method=m).fig.suptitle("Difference in LogOdds matrix Low-High vs High-Low")
    plt.savefig(figures_dir + "subst-hl-lh-"+m+"-clustermap.eps",bbox_inches="tight")

    fig, ax = plt.subplots()
    fig.set_size_inches(18,15)
    sns.clustermap(hl_lh,cmap="bwr",vmin=-1,vmax=+1,method=m).fig.suptitle("Difference in LogOdds matrix Low-High vs High-Low")
    plt.savefig(figures_dir + "subst-hl-lh-"+m+"-clustermap.eps",bbox_inches="tight")
    
    fig, ax = plt.subplots()
    fig.set_size_inches(18,15)
    sns.clustermap(hh_ll,cmap="bwr",vmin=-1,vmax=+1,method=m).fig.suptitle("Difference in LogOdds matrix High-High vs Low-Low")
    plt.savefig(figures_dir + "subst-hh-ll-"+m+"-clustermap.eps",bbox_inches="tight")



    plt.close('all')
